@@ -1,16 +1,12 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import abc
 import copy
@@ -19,43 +15,26 @@ import os
 import pathlib
 import time
 import warnings
-from typing import TYPE_CHECKING, Any, ClassVar, Dict, List, Optional, Protocol, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Protocol
+
+from typing_extensions import deprecated
 
 from gt4py import storage as gt_storage
 from gt4py.cartesian import definitions as gt_definitions, utils as gt_utils
-
-from . import pyext_builder
-from .module_generator import BaseModuleGenerator, ModuleData, make_args_data_from_gtir
+from gt4py.cartesian.backend import pyext_builder
+from gt4py.cartesian.backend.module_generator import (
+    BaseModuleGenerator,
+    ModuleData,
+    make_args_data_from_gtir,
+)
 
 
 if TYPE_CHECKING:
     from gt4py.cartesian.stencil_builder import StencilBuilder
     from gt4py.cartesian.stencil_object import StencilObject
 
-REGISTRY = gt_utils.Registry()
-
-
-def from_name(name: str) -> Optional[Type["Backend"]]:
-    return REGISTRY.get(name, None)
-
-
-def register(backend_cls: Type["Backend"]) -> None:
-    assert issubclass(backend_cls, Backend) and backend_cls.name is not None
-
-    if isinstance(backend_cls.name, str):
-        gt_storage.register(backend_cls.name, backend_cls.storage_info)
-        return REGISTRY.register(backend_cls.name, backend_cls)
-
-    else:
-        raise ValueError(
-            "Invalid 'name' attribute ('{name}') in backend class '{cls}'".format(
-                name=backend_cls.name, cls=backend_cls
-            )
-        )
-
 
 class Backend(abc.ABC):
-
     #: Backend name
     name: ClassVar[str]
 
@@ -65,7 +44,7 @@ class Backend(abc.ABC):
     #:    - versioning: is versioning on?
     #:    - description [optional]
     #:    - type
-    options: ClassVar[Dict[str, Any]]
+    options: ClassVar[dict[str, Any]]
 
     #: Backend-specific storage parametrization:
     #:
@@ -83,15 +62,15 @@ class Backend(abc.ABC):
     #:
     #:  Languages should be spelled using the official spelling
     #:  but lower case ("python", "fortran", "rust").
-    languages: ClassVar[Optional[Dict[str, Any]]] = None
+    languages: ClassVar[dict[str, Any] | None] = None
 
     # __impl_opts:
     #   "disable-code-generation": bool
     #   "disable-cache-validation": bool
 
-    builder: "StencilBuilder"
+    builder: StencilBuilder
 
-    def __init__(self, builder: "StencilBuilder"):
+    def __init__(self, builder: StencilBuilder) -> None:
         self.builder = builder
 
     @classmethod
@@ -108,7 +87,7 @@ class Backend(abc.ABC):
         return filtered_options
 
     @abc.abstractmethod
-    def load(self) -> Optional[Type["StencilObject"]]:
+    def load(self) -> type[StencilObject] | None:
         """
         Load the stencil class from the generated python module.
 
@@ -123,7 +102,7 @@ class Backend(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         """
         Generate the stencil class from GTScript's internal representation.
 
@@ -139,99 +118,46 @@ class Backend(abc.ABC):
         pass
 
     @property
-    def extra_cache_info(self) -> Dict[str, Any]:
-        """Provide additional data to be stored in cache info file (sublass hook)."""
+    def extra_cache_info(self) -> dict[str, Any]:
+        """Provide additional data to be stored in cache info file (subclass hook)."""
         return {}
 
     @property
-    def extra_cache_validation_keys(self) -> List[str]:
+    def extra_cache_validation_keys(self) -> list[str]:
         """List keys from extra_cache_info to be validated during consistency check."""
         return []
 
 
-class CLIBackendMixin(Backend):
-    @abc.abstractmethod
-    def generate_computation(self) -> Dict[str, Union[str, Dict]]:
-        """
-        Generate the computation source code in a way agnostic of the way it is going to be used.
+REGISTRY = gt_utils.Registry[type[Backend]]()
 
-        Returns
-        -------
-        Dict[str, str | Dict] of source file names / directories -> contents:
-            If a key's value is a string it is interpreted as a file name and the value as the
-            source code of that file
-            If a key's value is a Dict, it is interpreted as a directory name and it's
-            value as a nested file hierarchy to which the same rules are applied recursively.
-            The root path is relative to the build directory.
 
-        Raises
-        ------
-        NotImplementedError
-            If the backend does not support usage outside of JIT compilation / generation.
+def from_name(name: str) -> type[Backend]:
+    """Return a backend by name."""
+    backend_cls = REGISTRY.get(name, None)
+    if backend_cls is None:
+        raise NotImplementedError(
+            f"Backend '{name}' is not implemented. Options are: {REGISTRY.names}."
+        )
+    return backend_cls
 
-        Example
-        -------
-        .. code-block:: python
 
-            def mystencil(...):
-                ...
+def register(backend_cls: type[Backend]) -> type[Backend]:
+    """Register a backend."""
+    assert issubclass(backend_cls, Backend) and backend_cls.name is not None
 
-            options = BuildOptions(name="mystencil", ...)
-            ir = frontend.generate(mystencil, {}, options)
-            stencil_src = backend.generate_computation(ir, options)
+    if isinstance(backend_cls.name, str):
+        gt_storage.register(backend_cls.name, backend_cls.storage_info)
+        return REGISTRY.register(backend_cls.name, backend_cls)
 
-            print(stencil_src)
-
-            # this might be output from a fictional backend:
-            {
-                "mystencil_project": {
-                    "src": {
-                        "stencil.cpp",
-                        "helpers.cpp"
-                    },
-                    "include": {
-                        "stencil.hpp"
-                    },
-                }
-            }
-
-        This can now be automatically be turned into a folder hierarchy that makes sense
-        and can be incorporated into an external build system.
-
-        """
-        raise NotImplementedError
-
-    @abc.abstractmethod
-    def generate_bindings(self, language_name: str) -> Dict[str, Union[str, Dict]]:
-        """
-        Generate bindings source code from ``language_name`` to the target language of the backend.
-
-        Returns
-        -------
-        Analog to :py:meth:`generate_computation` but containing bindings source code, The
-        dictionary contains a tree of directories with leaves being a mapping from filename to
-        source code pairs, relative to the build directory.
-
-        Raises
-        ------
-        RuntimeError
-            If the backend does not support the bindings language
-
-        """
-        languages = getattr(self, "languages", {"bindings": {}})
-        name = getattr(self, "name", "")
-        if language_name not in languages["bindings"]:
-            raise NotImplementedError(
-                f"Backend {name} does not implement bindings for {language_name}"
-            )
-        return {}
+    raise ValueError(
+        f"Invalid 'name' attribute ('{backend_cls.name}') in backend class '{backend_cls}'."
+    )
 
 
 class BaseBackend(Backend):
+    MODULE_GENERATOR_CLASS: ClassVar[type[BaseModuleGenerator]]
 
-    MODULE_GENERATOR_CLASS: ClassVar[Type["BaseModuleGenerator"]]
-
-    def load(self) -> Optional[Type["StencilObject"]]:
+    def load(self) -> type[StencilObject] | None:
         build_info = self.builder.options.build_info
         if build_info is not None:
             start_time = time.perf_counter()
@@ -252,18 +178,17 @@ class BaseBackend(Backend):
 
         return stencil_class
 
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         self.check_options(self.builder.options)
         return self.make_module()
 
-    def _load(self) -> Type["StencilObject"]:
+    def _load(self) -> type[StencilObject]:
         stencil_class_name = self.builder.class_name
         file_name = str(self.builder.module_path)
         stencil_module = gt_utils.make_module_from_file(stencil_class_name, file_name)
-        stencil_class = getattr(stencil_module, stencil_class_name)
+        stencil_class: type[StencilObject] = getattr(stencil_module, stencil_class_name)
         stencil_class.__module__ = self.builder.module_qualname
         stencil_class._gt_id_ = self.builder.stencil_id.version
-        stencil_class._file_name = file_name
         stencil_class.definition_func = staticmethod(self.builder.definition)
 
         return stencil_class
@@ -273,19 +198,18 @@ class BaseBackend(Backend):
         unknown_options = set(options.backend_opts.keys()) - set(self.options.keys())
         if unknown_options:
             warnings.warn(
-                f"Unknown options '{unknown_options}' for backend '{self.name}'", RuntimeWarning
+                f"Unknown options '{unknown_options}' for backend '{self.name}'",
+                RuntimeWarning,
+                stacklevel=2,
             )
 
-    def make_module(
-        self,
-        **kwargs: Any,
-    ) -> Type["StencilObject"]:
+    def make_module(self) -> type[StencilObject]:
         build_info = self.builder.options.build_info
         if build_info is not None:
             start_time = time.perf_counter()
 
         file_path = self.builder.module_path
-        module_source = self.make_module_source(**kwargs)
+        module_source = self.make_module_source()
 
         if not self.builder.options._impl_opts.get("disable-code-generation", False):
             file_path.parent.mkdir(parents=True, exist_ok=True)
@@ -299,37 +223,14 @@ class BaseBackend(Backend):
 
         return module
 
-    def make_module_source(self, *, args_data: Optional[ModuleData] = None, **kwargs: Any) -> str:
+    def make_module_source(self, *, args_data: ModuleData | None = None) -> str:
         """Generate the module source code with or without stencil id."""
         args_data = args_data or make_args_data_from_gtir(self.builder.gtir_pipeline)
-        source = self.MODULE_GENERATOR_CLASS()(args_data, self.builder, **kwargs)
-        return source
+        return self.MODULE_GENERATOR_CLASS(self.builder)(args_data)
 
 
 class MakeModuleSourceCallable(Protocol):
-    def __call__(self, *, args_data: Optional[ModuleData] = None, **kwargs: Any) -> str:
-        ...
-
-
-class PurePythonBackendCLIMixin(CLIBackendMixin):
-    """Mixin for CLI support for backends deriving from BaseBackend."""
-
-    builder: "StencilBuilder"
-
-    #: stencil python source generator method:
-    #:  In order to use this mixin, the backend class must implement
-    #:  a :py:meth:`make_module_source` method or derive from
-    #:  :py:meth:`BaseBackend`.
-    make_module_source: MakeModuleSourceCallable
-
-    def generate_computation(self) -> Dict[str, Union[str, Dict]]:
-        file_name = self.builder.module_path.name
-        source = self.make_module_source(ir=self.builder.gtir)
-        return {str(file_name): source}
-
-    def generate_bindings(self, language_name: str) -> Dict[str, Union[str, Dict]]:
-        """Pure python backends typically will not support bindings."""
-        return super().generate_bindings(language_name)
+    def __call__(self, *, args_data: ModuleData | None = None) -> str: ...
 
 
 class BasePyExtBackend(BaseBackend):
@@ -350,7 +251,7 @@ class BasePyExtBackend(BaseBackend):
         return self.builder.pkg_path.joinpath(self.pyext_module_name + "_BUILD")
 
     @property
-    def extra_cache_info(self) -> Dict[str, Any]:
+    def extra_cache_info(self) -> dict[str, Any]:
         pyext_file_path = self.builder.backend_data.get("pyext_file_path", None)
         pyext_md5 = ""
         if pyext_file_path:
@@ -362,23 +263,23 @@ class BasePyExtBackend(BaseBackend):
         }
 
     @property
-    def extra_cache_validation_keys(self) -> List[str]:
+    def extra_cache_validation_keys(self) -> list[str]:
         keys = super().extra_cache_validation_keys
         if self.extra_cache_info["pyext_md5"]:
             keys.append("pyext_md5")
         return keys
 
     @abc.abstractmethod
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         pass
 
     def build_extension_module(
         self,
-        pyext_sources: Dict[str, Any],
-        pyext_build_opts: Dict[str, str],
+        pyext_sources: dict[str, Any],
+        pyext_build_opts: dict[str, str],
         *,
         uses_cuda: bool = False,
-    ) -> Tuple[str, str]:
+    ) -> None:
         # Build extension module
         pyext_build_path = pathlib.Path(
             os.path.relpath(self.pyext_build_dir_path, pathlib.Path.cwd())
@@ -397,7 +298,7 @@ class BasePyExtBackend(BaseBackend):
         pyext_target_file_path = self.builder.pkg_path
         qualified_pyext_name = self.pyext_module_path
 
-        pyext_build_args: Dict[str, Any] = dict(
+        pyext_build_args: dict[str, Any] = dict(
             name=qualified_pyext_name,
             sources=sources,
             build_path=str(pyext_build_path),
@@ -416,4 +317,26 @@ class BasePyExtBackend(BaseBackend):
             {"pyext_module_name": module_name, "pyext_file_path": file_path}
         )
 
-        return module_name, file_path
+
+def disabled(message: str, *, enabled_env_var: str) -> Callable[[type[Backend]], type[Backend]]:
+    # We push for hard deprecation here by raising by default and warning if enabling has been forced.
+    enabled = bool(int(os.environ.get(enabled_env_var, "0")))
+    if enabled:
+        return deprecated(message)
+
+    def _decorator(cls: type[Backend]) -> type[Backend]:
+        def _no_generate(obj) -> type[StencilObject]:
+            raise NotImplementedError(
+                f"Disabled '{cls.name}' backend: 'f{message}'\n",
+                f"You can still enable the backend by hand using the environment variable '{enabled_env_var}=1'",
+            )
+
+        # Replace generate method with raise
+        if not hasattr(cls, "generate"):
+            raise ValueError(f"Coding error. Expected a generate method on {cls}")
+        # Flag that it got disabled for register lookup
+        cls.disabled = True  # type: ignore
+        cls.generate = _no_generate  # type: ignore
+        return cls
+
+    return _decorator
