@@ -1,53 +1,47 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
-from typing import TYPE_CHECKING, Any, Dict, Optional, Tuple, Type
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, ClassVar
 
 from gt4py import storage as gt_storage
-from gt4py.cartesian.backend.base import CLIBackendMixin, register
+from gt4py.cartesian.backend.base import disabled, register
 from gt4py.cartesian.backend.gtc_common import (
     BackendCodegen,
+    BaseGTBackend,
+    CUDAPyExtModuleGenerator,
+    GTBackendOptions,
     bindings_main_template,
     pybuffer_to_sid,
 )
-from gt4py.cartesian.gtc import gtir
 from gt4py.cartesian.gtc.common import DataType
 from gt4py.cartesian.gtc.cuir import cuir, cuir_codegen, extent_analysis, kernel_fusion
 from gt4py.cartesian.gtc.cuir.oir_to_cuir import OIRToCUIR
 from gt4py.cartesian.gtc.gtir_to_oir import GTIRToOIR
-from gt4py.cartesian.gtc.passes.gtir_pipeline import GtirPipeline
 from gt4py.cartesian.gtc.passes.oir_optimizations.caches import FillFlushToLocalKCaches
 from gt4py.cartesian.gtc.passes.oir_optimizations.pruning import NoFieldAccessPruning
 from gt4py.cartesian.gtc.passes.oir_pipeline import DefaultPipeline
 from gt4py.eve import codegen
 
-from .gtc_common import BaseGTBackend, CUDAPyExtModuleGenerator
-
 
 if TYPE_CHECKING:
     from gt4py.cartesian.stencil_object import StencilObject
 
 
 class CudaExtGenerator(BackendCodegen):
-    def __init__(self, class_name, module_name, backend):
+    def __init__(self, class_name: str, module_name: str, backend: CudaBackend) -> None:
         self.class_name = class_name
         self.module_name = module_name
         self.backend = backend
 
-    def __call__(self, stencil_ir: gtir.Stencil) -> Dict[str, Dict[str, str]]:
-        stencil_ir = GtirPipeline(stencil_ir, self.backend.builder.stencil_id).full()
-        base_oir = GTIRToOIR().visit(stencil_ir)
+    def __call__(self) -> dict[str, dict[str, str]]:
+        base_oir = GTIRToOIR().visit(self.backend.builder.gtir)
         oir_pipeline = self.backend.builder.options.backend_opts.get(
             "oir_pipeline",
             DefaultPipeline(skip=[NoFieldAccessPruning], add_steps=[FillFlushToLocalKCaches]),
@@ -89,8 +83,7 @@ class CudaBindingsCodegen(codegen.TemplatedGenerator):
             sid_ndim = domain_ndim + data_ndim
             if kwargs["external_arg"]:
                 return "py::object {name}, std::array<gt::int_t,{sid_ndim}> {name}_origin".format(
-                    name=node.name,
-                    sid_ndim=sid_ndim,
+                    name=node.name, sid_ndim=sid_ndim
                 )
             else:
                 return pybuffer_to_sid(
@@ -113,12 +106,7 @@ class CudaBindingsCodegen(codegen.TemplatedGenerator):
         assert "module_name" in kwargs
         entry_params = self.visit(node.params, external_arg=True, **kwargs)
         sid_params = self.visit(node.params, external_arg=False, **kwargs)
-        return self.generic_visit(
-            node,
-            entry_params=entry_params,
-            sid_params=sid_params,
-            **kwargs,
-        )
+        return self.generic_visit(node, entry_params=entry_params, sid_params=sid_params, **kwargs)
 
     Program = bindings_main_template()
 
@@ -131,32 +119,33 @@ class CudaBindingsCodegen(codegen.TemplatedGenerator):
         return generated_code
 
 
+@disabled(
+    message="CUDA backend is deprecated. New features developed after February 2024 are not available.",
+    enabled_env_var="GT4PY_GTC_ENABLE_CUDA",
+)
 @register
-class CudaBackend(BaseGTBackend, CLIBackendMixin):
+class CudaBackend(BaseGTBackend):
     """CUDA backend using gtc."""
 
     name = "cuda"
-    options = {**BaseGTBackend.GT_BACKEND_OPTS, "device_sync": {"versioning": True, "type": bool}}
-    languages = {"computation": "cuda", "bindings": ["python"]}
+    options: ClassVar[GTBackendOptions] = {
+        **BaseGTBackend.GT_BACKEND_OPTS,
+        "device_sync": {"versioning": True, "type": bool},
+    }
+    languages: ClassVar[dict] = {"computation": "cuda", "bindings": ["python"]}
     storage_info = gt_storage.layout.CUDALayout
-    PYEXT_GENERATOR_CLASS = CudaExtGenerator  # type: ignore
+    PYEXT_GENERATOR_CLASS = CudaExtGenerator
     MODULE_GENERATOR_CLASS = CUDAPyExtModuleGenerator
     GT_BACKEND_T = "gpu"
 
-    def generate_extension(self, **kwargs: Any) -> Tuple[str, str]:
-        return self.make_extension(stencil_ir=self.builder.gtir, uses_cuda=True)
+    def generate_extension(self) -> None:
+        return self.make_extension(uses_cuda=True)
 
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         self.check_options(self.builder.options)
 
-        pyext_module_name: Optional[str]
-        pyext_file_path: Optional[str]
-
         # TODO(havogt) add bypass if computation has no effect
-        pyext_module_name, pyext_file_path = self.generate_extension()
+        self.generate_extension()
 
         # Generate and return the Python wrapper class
-        return self.make_module(
-            pyext_module_name=pyext_module_name,
-            pyext_file_path=pyext_file_path,
-        )
+        return self.make_module()
