@@ -1,45 +1,41 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import copy
 import os
 import pathlib
 import re
-import textwrap
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Type
+from typing import TYPE_CHECKING, ClassVar
 
-import dace
-import dace.data
+from dace import SDFG, Memlet, SDFGState, config, data, dtypes, nodes, subsets, symbolic
+from dace.codegen import codeobject
 from dace.sdfg.utils import inline_sdfgs
-from dace.serialize import dumps
 
-from gt4py import storage as gt_storage
-from gt4py.cartesian.backend.base import CLIBackendMixin, register
+from gt4py._core import definitions as core_defs
+from gt4py.cartesian import config as gt_config, definitions
+from gt4py.cartesian.backend.base import register
 from gt4py.cartesian.backend.gtc_common import (
     BackendCodegen,
     BaseGTBackend,
     CUDAPyExtModuleGenerator,
+    GTBackendOptions,
     PyExtModuleGenerator,
     bindings_main_template,
     pybuffer_to_sid,
 )
 from gt4py.cartesian.backend.module_generator import make_args_data_from_gtir
 from gt4py.cartesian.gtc import common, gtir
+from gt4py.cartesian.gtc.dace import daceir as dcir
 from gt4py.cartesian.gtc.dace.nodes import StencilComputation
 from gt4py.cartesian.gtc.dace.oir_to_dace import OirSDFGBuilder
 from gt4py.cartesian.gtc.dace.transformations import (
-    InlineThreadLocalTransients,
     NoEmptyEdgeTrivialMapElimination,
     nest_sequential_map_scopes,
 )
@@ -47,12 +43,12 @@ from gt4py.cartesian.gtc.dace.utils import array_dimensions, replace_strides
 from gt4py.cartesian.gtc.gtir_to_oir import GTIRToOIR
 from gt4py.cartesian.gtc.passes.gtir_k_boundary import compute_k_boundary
 from gt4py.cartesian.gtc.passes.gtir_pipeline import GtirPipeline
-from gt4py.cartesian.gtc.passes.oir_optimizations.inlining import MaskInlining
 from gt4py.cartesian.gtc.passes.oir_optimizations.utils import compute_fields_extents
 from gt4py.cartesian.gtc.passes.oir_pipeline import DefaultPipeline
 from gt4py.cartesian.utils import shash
 from gt4py.eve import codegen
 from gt4py.eve.codegen import MakoTemplate as as_mako
+from gt4py.storage.cartesian import layout
 
 
 if TYPE_CHECKING:
@@ -60,23 +56,18 @@ if TYPE_CHECKING:
     from gt4py.cartesian.stencil_object import StencilObject
 
 
-def _serialize_sdfg(sdfg: dace.SDFG):
-    return dumps(sdfg)
-
-
-def _specialize_transient_strides(sdfg: dace.SDFG, layout_map):
-    repldict = replace_strides(
-        [array for array in sdfg.arrays.values() if array.transient],
-        layout_map,
+def _specialize_transient_strides(sdfg: SDFG, layout_info: layout.LayoutInfo) -> None:
+    replacement_dictionary = replace_strides(
+        [array for array in sdfg.arrays.values() if array.transient], layout_info["layout_map"]
     )
-    sdfg.replace_dict(repldict)
+    sdfg.replace_dict(replacement_dictionary)
     for state in sdfg.nodes():
         for node in state.nodes():
-            if isinstance(node, dace.nodes.NestedSDFG):
-                for k, v in repldict.items():
+            if isinstance(node, nodes.NestedSDFG):
+                for k, v in replacement_dictionary.items():
                     if k in node.symbol_mapping:
                         node.symbol_mapping[k] = v
-    for k in repldict.keys():
+    for k in replacement_dictionary.keys():
         if k in sdfg.symbols:
             sdfg.remove_symbol(k)
 
@@ -108,11 +99,11 @@ def _get_expansion_priority_gpu(node: StencilComputation):
     return expansion_priority
 
 
-def _set_expansion_orders(sdfg: dace.SDFG):
+def _set_expansion_orders(sdfg: SDFG):
     for node, _ in filter(
         lambda n: isinstance(n[0], StencilComputation), sdfg.all_nodes_recursive()
     ):
-        if node.device == dace.DeviceType.GPU:
+        if node.device == dtypes.DeviceType.GPU:
             expansion_priority = _get_expansion_priority_gpu(node)
         else:
             expansion_priority = _get_expansion_priority_cpu(node)
@@ -129,13 +120,11 @@ def _set_expansion_orders(sdfg: dace.SDFG):
             raise ValueError("No expansion compatible")
 
 
-def _set_tile_sizes(sdfg: dace.SDFG):
-    import gt4py.cartesian.gtc.daceir as dcir  # avoid circular import
-
+def _set_tile_sizes(sdfg: SDFG):
     for node, _ in filter(
         lambda n: isinstance(n[0], StencilComputation), sdfg.all_nodes_recursive()
     ):
-        if node.device == dace.DeviceType.GPU:
+        if node.device == dtypes.DeviceType.GPU:
             node.tile_sizes = {dcir.Axis.I: 64, dcir.Axis.J: 8, dcir.Axis.K: 8}
             node.tile_sizes_interpretation = "shape"
         else:
@@ -143,39 +132,36 @@ def _set_tile_sizes(sdfg: dace.SDFG):
             node.tile_sizes_interpretation = "strides"
 
 
-def _to_device(sdfg: dace.SDFG, device: str) -> None:
+def _to_device(sdfg: SDFG, device: str) -> None:
     """Update sdfg in place."""
     if device == "gpu":
         for array in sdfg.arrays.values():
-            array.storage = dace.StorageType.GPU_Global
+            array.storage = dtypes.StorageType.GPU_Global
         for node, _ in sdfg.all_nodes_recursive():
             if isinstance(node, StencilComputation):
-                node.device = dace.DeviceType.GPU
+                node.device = dtypes.DeviceType.GPU
 
 
-def _pre_expand_trafos(gtir_pipeline: GtirPipeline, sdfg: dace.SDFG, layout_map):
-
+def _pre_expand_transformations(
+    gtir_pipeline: GtirPipeline, sdfg: SDFG, layout_info: layout.LayoutInfo
+):
     args_data = make_args_data_from_gtir(gtir_pipeline)
 
     # stencils without effect
     if all(info is None for info in args_data.field_info.values()):
-        sdfg = dace.SDFG(gtir_pipeline.gtir.name)
+        sdfg = SDFG(gtir_pipeline.gtir.name)
         sdfg.add_state(gtir_pipeline.gtir.name)
         return sdfg
 
-    for array in sdfg.arrays.values():
-        if array.transient:
-            array.lifetime = dace.AllocationLifetime.Persistent
-
     sdfg.simplify(validate=False)
 
     _set_expansion_orders(sdfg)
     _set_tile_sizes(sdfg)
-    _specialize_transient_strides(sdfg, layout_map=layout_map)
+    _specialize_transient_strides(sdfg, layout_info)
     return sdfg
 
 
-def _post_expand_trafos(sdfg: dace.SDFG):
+def _post_expand_transformations(sdfg: SDFG):
     # DaCe "standard" clean-up transformations
     sdfg.simplify(validate=False)
 
@@ -183,14 +169,13 @@ def _post_expand_trafos(sdfg: dace.SDFG):
 
     # Control the `#pragma omp parallel` statements: Fully collapse parallel loops,
     # but set 1D maps to be sequential. (Typical domains are too small to benefit from parallelism)
-    for node, _ in filter(
-        lambda n: isinstance(n[0], dace.nodes.MapEntry), sdfg.all_nodes_recursive()
-    ):
+    for node, _ in filter(lambda n: isinstance(n[0], nodes.MapEntry), sdfg.all_nodes_recursive()):
         node.collapse = len(node.range)
-        if node.schedule == dace.ScheduleType.CPU_Multicore and len(node.range) <= 1:
-            node.schedule = dace.ScheduleType.Sequential
+        if node.schedule == dtypes.ScheduleType.CPU_Multicore and len(node.range) <= 1:
+            node.schedule = dtypes.ScheduleType.Sequential
 
-    sdfg.apply_transformations_repeated(InlineThreadLocalTransients, validate=False)
+    # To be re-evaluated with https://github.com/GridTools/gt4py/issues/1896
+    # sdfg.apply_transformations_repeated(InlineThreadLocalTransients, validate=False) # noqa: ERA001
     sdfg.simplify(validate=False)
     nest_sequential_map_scopes(sdfg)
     for sd in sdfg.all_sdfgs_recursive():
@@ -198,11 +183,17 @@ def _post_expand_trafos(sdfg: dace.SDFG):
 
 
 def _sdfg_add_arrays_and_edges(
-    field_info, wrapper_sdfg, state, inner_sdfg, nsdfg, inputs, outputs, origins
-):
-
+    field_info: dict[str, definitions.FieldInfo],
+    wrapper_sdfg: SDFG,
+    state: SDFGState,
+    inner_sdfg: SDFG,
+    nsdfg: nodes.NestedSDFG,
+    inputs: set[str] | dict[str, dtypes.typeclass],
+    outputs: set[str] | dict[str, dtypes.typeclass],
+    origins,
+) -> None:
     for name, array in inner_sdfg.arrays.items():
-        if isinstance(array, dace.data.Array) and not array.transient:
+        if isinstance(array, data.Array) and not array.transient:
             axes = field_info[name].axes
 
             shape = [f"__{name}_{axis}_size" for axis in axes] + [
@@ -222,9 +213,7 @@ def _sdfg_add_arrays_and_edges(
             ranges = [
                 (o - max(0, e), o - max(0, e) + s - 1, 1)
                 for o, e, s in zip(
-                    origin,
-                    field_info[name].boundary.lower_indices,
-                    inner_sdfg.arrays[name].shape,
+                    origin, field_info[name].boundary.lower_indices, inner_sdfg.arrays[name].shape
                 )
             ]
             ranges += [(0, d, 1) for d in field_info[name].data_dims]
@@ -234,7 +223,7 @@ def _sdfg_add_arrays_and_edges(
                     None,
                     nsdfg,
                     name,
-                    dace.Memlet(name, subset=dace.subsets.Range(ranges)),
+                    Memlet(name, subset=subsets.Range(ranges)),
                 )
             if name in outputs:
                 state.add_edge(
@@ -242,11 +231,11 @@ def _sdfg_add_arrays_and_edges(
                     name,
                     state.add_write(name),
                     None,
-                    dace.Memlet(name, subset=dace.subsets.Range(ranges)),
+                    Memlet(name, subset=subsets.Range(ranges)),
                 )
 
 
-def _sdfg_specialize_symbols(wrapper_sdfg, domain: Tuple[int, ...]):
+def _sdfg_specialize_symbols(wrapper_sdfg: SDFG, domain: tuple[int, ...]) -> None:
     ival, jval, kval = domain[0], domain[1], domain[2]
     for sdfg in wrapper_sdfg.all_sdfgs_recursive():
         if sdfg.parent_nsdfg_node is not None:
@@ -271,7 +260,7 @@ def _sdfg_specialize_symbols(wrapper_sdfg, domain: Tuple[int, ...]):
             sdfg.remove_symbol("__K")
 
         for val in ival, jval, kval:
-            sym = dace.symbolic.pystr_to_symbolic(val)
+            sym = symbolic.pystr_to_symbolic(val)
             for fsym in sym.free_symbols:
                 if sdfg.parent_nsdfg_node is not None:
                     sdfg.parent_nsdfg_node.symbol_mapping[str(fsym)] = fsym
@@ -279,21 +268,25 @@ def _sdfg_specialize_symbols(wrapper_sdfg, domain: Tuple[int, ...]):
                     if str(fsym) in sdfg.parent_sdfg.symbols:
                         sdfg.add_symbol(str(fsym), stype=sdfg.parent_sdfg.symbols[str(fsym)])
                     else:
-                        sdfg.add_symbol(str(fsym), stype=dace.dtypes.int32)
+                        sdfg.add_symbol(str(fsym), stype=dtypes.int32)
 
 
-def freeze_origin_domain_sdfg(inner_sdfg, arg_names, field_info, *, origin, domain):
-    wrapper_sdfg = dace.SDFG("frozen_" + inner_sdfg.name)
+def freeze_origin_domain_sdfg(
+    inner_sdfg: SDFG,
+    arg_names: list[str],
+    field_info: dict[str, definitions.FieldInfo],
+    *,
+    origin: dict[str, tuple[int, ...]],
+    domain: tuple[int, ...],
+) -> SDFG:
+    wrapper_sdfg = SDFG("frozen_" + inner_sdfg.name)
     state = wrapper_sdfg.add_state("frozen_" + inner_sdfg.name + "_state")
 
     inputs = set()
     outputs = set()
     for inner_state in inner_sdfg.nodes():
         for node in inner_state.nodes():
-            if (
-                not isinstance(node, dace.nodes.AccessNode)
-                or inner_sdfg.arrays[node.data].transient
-            ):
+            if not isinstance(node, nodes.AccessNode) or inner_sdfg.arrays[node.data].transient:
                 continue
             if node.has_reads(inner_state):
                 inputs.add(node.data)
@@ -313,7 +306,7 @@ def freeze_origin_domain_sdfg(inner_sdfg, arg_names, field_info, *, origin, doma
         for node in states[0].nodes():
             state.remove_node(node)
 
-    # make sure that symbols are passed throught o inner sdfg
+    # make sure that symbols are passed through to inner sdfg
     for symbol in nsdfg.sdfg.free_symbols:
         if symbol not in wrapper_sdfg.symbols:
             wrapper_sdfg.add_symbol(symbol, nsdfg.sdfg.symbols[symbol])
@@ -325,7 +318,7 @@ def freeze_origin_domain_sdfg(inner_sdfg, arg_names, field_info, *, origin, doma
 
     for _, _, array in wrapper_sdfg.arrays_recursive():
         if array.transient:
-            array.lifetime = dace.dtypes.AllocationLifetime.SDFG
+            array.lifetime = dtypes.AllocationLifetime.SDFG
 
     wrapper_sdfg.arg_names = arg_names
 
@@ -333,21 +326,21 @@ def freeze_origin_domain_sdfg(inner_sdfg, arg_names, field_info, *, origin, doma
 
 
 class SDFGManager:
+    # Cache loaded SDFGs across all instances
+    _loaded_sdfgs: ClassVar[dict[str | pathlib.Path, SDFG]] = dict()
 
-    _loaded_sdfgs: Dict[str, dace.SDFG] = dict()
-
-    def __init__(self, builder):
+    def __init__(self, builder: StencilBuilder) -> None:
         self.builder = builder
 
     @staticmethod
-    def _strip_history(sdfg):
+    def _strip_history(sdfg: SDFG) -> None:
         # strip history from SDFG for faster save/load
         for tmp_sdfg in sdfg.all_sdfgs_recursive():
             tmp_sdfg.transformation_hist = []
             tmp_sdfg.orig_sdfg = None
 
     @staticmethod
-    def _save_sdfg(sdfg, path):
+    def _save_sdfg(sdfg: SDFG, path: str) -> None:
         SDFGManager._strip_history(sdfg)
         sdfg.save(path)
 
@@ -359,26 +352,22 @@ class SDFGManager:
         )
 
         if path not in SDFGManager._loaded_sdfgs:
-
             try:
-                sdfg = dace.SDFG.from_file(path)
+                sdfg = SDFG.from_file(path)
             except FileNotFoundError:
-
                 base_oir = GTIRToOIR().visit(self.builder.gtir)
                 oir_pipeline = self.builder.options.backend_opts.get(
-                    "oir_pipeline", DefaultPipeline(skip=[MaskInlining])
+                    "oir_pipeline", DefaultPipeline()
                 )
                 oir_node = oir_pipeline.run(base_oir)
                 sdfg = OirSDFGBuilder().visit(oir_node)
 
                 _to_device(sdfg, self.builder.backend.storage_info["device"])
-                _pre_expand_trafos(
-                    self.builder.gtir_pipeline,
-                    sdfg,
-                    self.builder.backend.storage_info["layout_map"],
+                _pre_expand_transformations(
+                    self.builder.gtir_pipeline, sdfg, self.builder.backend.storage_info
                 )
                 self._save_sdfg(sdfg, path)
-            self._loaded_sdfgs[path] = sdfg
+            SDFGManager._loaded_sdfgs[path] = sdfg
 
         return SDFGManager._loaded_sdfgs[path]
 
@@ -388,97 +377,94 @@ class SDFGManager:
     def _expanded_sdfg(self):
         sdfg = self._unexpanded_sdfg()
         sdfg.expand_library_nodes()
-        _post_expand_trafos(sdfg)
+        _post_expand_transformations(sdfg)
         return sdfg
 
     def expanded_sdfg(self):
         return copy.deepcopy(self._expanded_sdfg())
 
-    def _frozen_sdfg(self, *, origin: Dict[str, Tuple[int, ...]], domain: Tuple[int, ...]):
-        frozen_hash = shash(origin, domain)
+    def _frozen_sdfg(self, *, origin: dict[str, tuple[int, ...]], domain: tuple[int, ...]) -> SDFG:
+        basename = self.builder.module_path.with_suffix("")
+        path = f"{basename}_{shash(origin, domain)}.sdfg"
+
         # check if same sdfg already cached on disk
-        path = self.builder.module_path
-        basename = os.path.splitext(path)[0]
-        path = basename + "_" + str(frozen_hash) + ".sdfg"
-        if path not in self._loaded_sdfgs:
-            try:
-                sdfg = dace.SDFG.from_file(path)
-            except FileNotFoundError:
-                # otherwise, wrap and save sdfg from scratch
-                inner_sdfg = self.unexpanded_sdfg()
+        if path in SDFGManager._loaded_sdfgs:
+            return SDFGManager._loaded_sdfgs[path]
 
-                sdfg = freeze_origin_domain_sdfg(
-                    inner_sdfg,
-                    arg_names=[arg.name for arg in self.builder.gtir.api_signature],
-                    field_info=make_args_data_from_gtir(self.builder.gtir_pipeline).field_info,
-                    origin=origin,
-                    domain=domain,
-                )
-                self._save_sdfg(sdfg, path)
-            self._loaded_sdfgs[path] = sdfg
-        return self._loaded_sdfgs[path]
+        # otherwise, wrap and save sdfg from scratch
+        inner_sdfg = self.unexpanded_sdfg()
 
-    def frozen_sdfg(self, *, origin: Dict[str, Tuple[int, ...]], domain: Tuple[int, ...]):
+        sdfg = freeze_origin_domain_sdfg(
+            inner_sdfg,
+            arg_names=[arg.name for arg in self.builder.gtir.api_signature],
+            field_info=make_args_data_from_gtir(self.builder.gtir_pipeline).field_info,
+            origin=origin,
+            domain=domain,
+        )
+        SDFGManager._loaded_sdfgs[path] = sdfg
+        self._save_sdfg(sdfg, path)
+
+        return SDFGManager._loaded_sdfgs[path]
+
+    def frozen_sdfg(self, *, origin: dict[str, tuple[int, ...]], domain: tuple[int, ...]) -> SDFG:
         return copy.deepcopy(self._frozen_sdfg(origin=origin, domain=domain))
 
 
 class DaCeExtGenerator(BackendCodegen):
-    def __init__(self, class_name, module_name, backend):
+    def __init__(self, class_name: str, module_name: str, backend: BaseDaceBackend) -> None:
         self.class_name = class_name
         self.module_name = module_name
         self.backend = backend
 
-    def __call__(self, stencil_ir: gtir.Stencil) -> Dict[str, Dict[str, str]]:
-
+    def __call__(self) -> dict[str, dict[str, str]]:
         manager = SDFGManager(self.backend.builder)
         sdfg = manager.expanded_sdfg()
 
-        sources: Dict[str, Dict[str, str]]
-        implementation = DaCeComputationCodegen.apply(stencil_ir, self.backend.builder, sdfg)
+        implementation = DaCeComputationCodegen.apply(self.backend.builder, sdfg)
 
         bindings = DaCeBindingsCodegen.apply(
-            stencil_ir, sdfg, module_name=self.module_name, backend=self.backend
+            sdfg, module_name=self.module_name, backend=self.backend
         )
 
         bindings_ext = "cu" if self.backend.storage_info["device"] == "gpu" else "cpp"
-        sources = {
+        return {
             "computation": {"computation.hpp": implementation},
             "bindings": {f"bindings.{bindings_ext}": bindings},
         }
-        return sources
 
 
 class DaCeComputationCodegen:
-
     template = as_mako(
-        """
-        auto ${name}(const std::array<gt::uint_t, 3>& domain) {
-            return [domain](${",".join(functor_args)}) {
-                const int __I = domain[0];
-                const int __J = domain[1];
-                const int __K = domain[2];
-                ${name}_t dace_handle;
-                ${backend_specifics}
-                auto allocator = gt::sid::cached_allocator(&${allocator}<char[]>);
-                ${"\\n".join(tmp_allocs)}
-                __program_${name}(${",".join(["&dace_handle", *dace_args])});
-            };
-        }
-        """
+        """\
+auto ${name}(const std::array<gt::uint_t, 3>& domain) {
+    return [domain](${",".join(functor_args)}) {
+        const int __I = domain[0];
+        const int __J = domain[1];
+        const int __K = domain[2];
+        ${name}${state_suffix} dace_handle;
+        ${backend_specifics}
+        auto allocator = gt::sid::cached_allocator(&${allocator}<char[]>);
+        ${"\\n".join(tmp_allocs)}
+        __program_${name}(${",".join(["&dace_handle", *dace_args])});
+    };
+}
+"""
     )
 
-    def generate_tmp_allocs(self, sdfg):
-
-        global_fmt = "dace_handle.__{sdfg_id}_{name} = allocate(allocator, gt::meta::lazy::id<{dtype}>(), {size})();"
+    def generate_tmp_allocs(self, sdfg: SDFG) -> list[str]:
+        global_fmt = (
+            "__{sdfg_id}_{name} = allocate(allocator, gt::meta::lazy::id<{dtype}>(), {size})();"
+        )
         threadlocal_fmt = (
-            "{name} = dace_handle.__{sdfg_id}_{name} + omp_get_thread_num() * ({local_size});"
+            "__{sdfg_id}_{name} = __full__{sdfg_id}_{name} + omp_get_thread_num() * ({local_size});"
         )
         res = []
         for array_sdfg, name, array in sdfg.arrays_recursive():
-            if array.transient and array.lifetime == dace.AllocationLifetime.Persistent:
-                if array.storage != dace.StorageType.CPU_ThreadLocal:
+            if array.transient and array.lifetime == dtypes.AllocationLifetime.Persistent:
+                if array.storage != dtypes.StorageType.CPU_ThreadLocal:
+                    fmt = "dace_handle." + global_fmt
                     res.append(
-                        global_fmt.format(
+                        fmt.format(
                             name=name,
                             sdfg_id=array_sdfg.sdfg_id,
                             dtype=array.dtype.ctype,
@@ -486,7 +472,13 @@ class DaCeComputationCodegen:
                         )
                     )
                 else:
-                    fmts = [global_fmt, "#pragma omp parallel", "{{", threadlocal_fmt, "}}"]
+                    fmts = [
+                        "{dtype} *__full" + global_fmt,
+                        "#pragma omp parallel",
+                        "{{",
+                        threadlocal_fmt,
+                        "}}",
+                    ]
                     res.extend(
                         [
                             fmt.format(
@@ -502,7 +494,7 @@ class DaCeComputationCodegen:
         return res
 
     @staticmethod
-    def _postprocess_dace_code(code_objects, is_gpu, builder):
+    def _postprocess_dace_code(code_objects: codeobject.CodeObject, is_gpu: bool) -> str:
         lines = code_objects[[co.title for co in code_objects].index("Frame")].clean_code.split(
             "\n"
         )
@@ -518,11 +510,11 @@ class DaCeComputationCodegen:
                     break
             for i, line in enumerate(lines):
                 if "#include <dace/dace.h>" in line:
-                    cuda_code = [co.clean_code for co in code_objects if co.title == "CUDA"][0]
+                    cuda_code = next(co.clean_code for co in code_objects if co.title == "CUDA")
                     lines = lines[0:i] + cuda_code.split("\n") + lines[i + 1 :]
                     break
 
-        def keep_line(line):
+        def keep_line(line: str) -> bool:
             line = line.strip()
             if line == '#include "../../include/hash.h"':
                 return False
@@ -532,24 +524,27 @@ class DaCeComputationCodegen:
                 return False
             return True
 
-        lines = filter(keep_line, lines)
-        generated_code = "\n".join(lines)
-        if builder.options.format_source:
-            generated_code = codegen.format_source("cpp", generated_code, style="LLVM")
-        return generated_code
+        return "\n".join(filter(keep_line, lines))
 
     @classmethod
-    def apply(cls, stencil_ir: gtir.Stencil, builder: "StencilBuilder", sdfg: dace.SDFG):
+    def apply(cls, builder: StencilBuilder, sdfg: SDFG) -> str:
         self = cls()
-        with dace.config.temporary_config():
-            dace.config.Config.set("compiler", "cuda", "max_concurrent_streams", value=-1)
-            dace.config.Config.set("compiler", "cpu", "openmp_sections", value=False)
+        with config.temporary_config():
+            # To prevent conflict with 3rd party usage of DaCe config always make sure that any
+            #  changes be under the temporary_config manager
+            if core_defs.CUPY_DEVICE_TYPE == core_defs.DeviceType.ROCM:
+                config.Config.set("compiler", "cuda", "backend", value="hip")
+            config.Config.set("compiler", "cuda", "max_concurrent_streams", value=-1)
+            config.Config.set(
+                "compiler", "cuda", "default_block_size", value=gt_config.DACE_DEFAULT_BLOCK_SIZE
+            )
+            config.Config.set("compiler", "cpu", "openmp_sections", value=False)
             code_objects = sdfg.generate_code()
         is_gpu = "CUDA" in {co.title for co in code_objects}
 
-        computations = cls._postprocess_dace_code(code_objects, is_gpu, builder)
+        computations = cls._postprocess_dace_code(code_objects, is_gpu)
         if not is_gpu and any(
-            array.transient and array.lifetime == dace.AllocationLifetime.Persistent
+            array.transient and array.lifetime == dtypes.AllocationLifetime.Persistent
             for *_, array in sdfg.arrays_recursive()
         ):
             omp_threads = "int omp_max_threads = omp_get_max_threads();"
@@ -557,31 +552,34 @@ class DaCeComputationCodegen:
         else:
             omp_threads = ""
             omp_header = ""
+
         interface = cls.template.definition.render(
             name=sdfg.name,
             backend_specifics=omp_threads,
-            dace_args=self.generate_dace_args(stencil_ir, sdfg),
+            dace_args=self.generate_dace_args(builder.gtir, sdfg),
             functor_args=self.generate_functor_args(sdfg),
             tmp_allocs=self.generate_tmp_allocs(sdfg),
             allocator="gt::cuda_util::cuda_malloc" if is_gpu else "std::make_unique",
+            state_suffix=config.Config.get("compiler.codegen_state_struct_suffix"),
         )
-        generated_code = textwrap.dedent(
-            f"""#include <gridtools/sid/sid_shift_origin.hpp>
-                             #include <gridtools/sid/allocator.hpp>
-                             #include <gridtools/stencil/cartesian.hpp>
-                             {"#include <gridtools/common/cuda_util.hpp>" if is_gpu else omp_header}
-                             namespace gt = gridtools;
-                             {computations}
+        generated_code = f"""\
+#include <gridtools/sid/sid_shift_origin.hpp>
+#include <gridtools/sid/allocator.hpp>
+#include <gridtools/stencil/cartesian.hpp>
+{"#include <gridtools/common/cuda_util.hpp>" if is_gpu else omp_header}
+namespace gt = gridtools;
+
+{computations}
+
+{interface}
+"""
 
-                             {interface}
-                             """
-        )
         if builder.options.format_source:
             generated_code = codegen.format_source("cpp", generated_code, style="LLVM")
 
         return generated_code
 
-    def generate_dace_args(self, stencil_ir: gtir.Stencil, sdfg: dace.SDFG) -> List[str]:
+    def generate_dace_args(self, stencil_ir: gtir.Stencil, sdfg: SDFG) -> list[str]:
         oir = GTIRToOIR().visit(stencil_ir)
         field_extents = compute_fields_extents(oir, add_k=True)
 
@@ -589,7 +587,7 @@ class DaCeComputationCodegen:
             field_name: max(boundary[0], 0)
             for field_name, boundary in compute_k_boundary(stencil_ir).items()
         }
-        offset_dict: Dict[str, Tuple[int, int, int]] = {
+        offset_dict: dict[str, tuple[int, int, int]] = {
             k: (max(-v[0][0], 0), max(-v[1][0], 0), k_origins[k] if k in k_origins else 0)
             for k, v in field_extents.items()
         }
@@ -633,7 +631,7 @@ class DaCeComputationCodegen:
                 -offset_dict[name][idx]
                 for idx, var in enumerate("IJK")
                 if any(
-                    dace.symbolic.pystr_to_symbolic(f"__{var}") in s.free_symbols
+                    symbolic.pystr_to_symbolic(f"__{var}") in s.free_symbols
                     for s in array.shape
                     if hasattr(s, "free_symbols")
                 )
@@ -641,6 +639,7 @@ class DaCeComputationCodegen:
             symbols[name] = fmt.format(
                 name=name, ndim=len(array.shape), origin=",".join(str(o) for o in origin)
             )
+
         # the remaining arguments are variables and can be passed by name
         for sym in sdfg.signature_arglist(with_types=False, for_call=True):
             if sym not in symbols:
@@ -649,19 +648,19 @@ class DaCeComputationCodegen:
         # return strings in order of sdfg signature
         return [symbols[s] for s in sdfg.signature_arglist(with_types=False, for_call=True)]
 
-    def generate_functor_args(self, sdfg: dace.SDFG):
-        res = []
+    def generate_functor_args(self, sdfg: SDFG) -> list[str]:
+        arguments = []
         for name, array in sdfg.arrays.items():
             if array.transient:
                 continue
-            res.append(f"auto && __{name}_sid")
+            arguments.append(f"auto && __{name}_sid")
         for name, dtype in ((n, d) for n, d in sdfg.symbols.items() if not n.startswith("__")):
-            res.append(dtype.as_arg(name))
-        return res
+            arguments.append(dtype.as_arg(name))
+        return arguments
 
 
 class DaCeBindingsCodegen:
-    def __init__(self, backend):
+    def __init__(self, backend: BaseDaceBackend):
         self.backend = backend
         self._unique_index: int = 0
 
@@ -671,34 +670,37 @@ class DaCeBindingsCodegen:
 
     mako_template = bindings_main_template()
 
-    def generate_entry_params(self, stencil_ir: gtir.Stencil, sdfg: dace.SDFG) -> List[str]:
-        res: Dict[str, str] = {}
+    def generate_entry_params(self, sdfg: SDFG) -> list[str]:
+        res: dict[str, str] = {}
 
         for name in sdfg.signature_arglist(with_types=False, for_call=True):
             if name in sdfg.arrays:
-                data = sdfg.arrays[name]
-                assert isinstance(data, dace.data.Array)
-                res[
-                    name
-                ] = "py::{pybind_type} {name}, std::array<gt::int_t,{ndim}> {name}_origin".format(
-                    pybind_type="object"
-                    if self.backend.storage_info["device"] == "gpu"
-                    else "buffer",
-                    name=name,
-                    ndim=len(data.shape),
+                container = sdfg.arrays[name]
+                assert isinstance(container, data.Array)
+                res[name] = (
+                    "py::{pybind_type} {name}, std::array<gt::int_t,{ndim}> {name}_origin".format(
+                        pybind_type=(
+                            "object" if self.backend.storage_info["device"] == "gpu" else "buffer"
+                        ),
+                        name=name,
+                        ndim=len(container.shape),
+                    )
                 )
             elif name in sdfg.symbols and not name.startswith("__"):
-                assert name in sdfg.symbols
-                res[name] = "{dtype} {name}".format(dtype=sdfg.symbols[name].ctype, name=name)
-        return list(res[node.name] for node in stencil_ir.params if node.name in res)
+                res[name] = f"{sdfg.symbols[name].ctype} {name}"
+        return list(res[node.name] for node in self.backend.builder.gtir.params if node.name in res)
 
-    def generate_sid_params(self, sdfg: dace.SDFG) -> List[str]:
-        res: List[str] = []
+    def generate_sid_params(self, sdfg: SDFG) -> list[str]:
+        res: list[str] = []
 
         for name, array in sdfg.arrays.items():
             if array.transient:
                 continue
-            domain_dim_flags = array_dimensions(array)
+
+            domain_dim_flags = tuple(array_dimensions(array))
+            if len(domain_dim_flags) != 3:
+                raise RuntimeError("Expected 3 cartesian array dimensions. Codegen error.")
+
             data_ndim = len(array.shape) - sum(domain_dim_flags)
             sid_def = pybuffer_to_sid(
                 name=name,
@@ -715,27 +717,27 @@ class DaCeBindingsCodegen:
             res.append(name)
         return res
 
-    def generate_sdfg_bindings(self, stencil_ir: gtir.Stencil, sdfg, module_name) -> str:
-
+    def generate_sdfg_bindings(self, sdfg, module_name) -> str:
         return self.mako_template.render_values(
             name=sdfg.name,
             module_name=module_name,
-            entry_params=self.generate_entry_params(stencil_ir, sdfg),
+            entry_params=self.generate_entry_params(sdfg),
             sid_params=self.generate_sid_params(sdfg),
         )
 
     @classmethod
-    def apply(cls, stencil_ir: gtir.Stencil, sdfg: dace.SDFG, module_name: str, *, backend) -> str:
-        generated_code = cls(backend).generate_sdfg_bindings(
-            stencil_ir, sdfg, module_name=module_name
-        )
+    def apply(cls, sdfg: SDFG, module_name: str, *, backend: BaseDaceBackend) -> str:
+        generated_code = cls(backend).generate_sdfg_bindings(sdfg, module_name=module_name)
         if backend.builder.options.format_source:
             generated_code = codegen.format_source("cpp", generated_code, style="LLVM")
         return generated_code
 
 
 class DaCePyExtModuleGenerator(PyExtModuleGenerator):
-    def generate_imports(self):
+    def __init__(self, builder: StencilBuilder) -> None:
+        super().__init__(builder)
+
+    def generate_imports(self) -> str:
         return "\n".join(
             [
                 *super().generate_imports().splitlines(),
@@ -745,10 +747,10 @@ class DaCePyExtModuleGenerator(PyExtModuleGenerator):
             ]
         )
 
-    def generate_base_class_name(self):
+    def generate_base_class_name(self) -> str:
         return "DaCeStencilObject"
 
-    def generate_class_members(self):
+    def generate_class_members(self) -> str:
         res = super().generate_class_members()
         filepath = self.builder.module_path.joinpath(
             os.path.dirname(self.builder.module_path), self.builder.module_name + ".sdfg"
@@ -761,46 +763,36 @@ class DaCeCUDAPyExtModuleGenerator(DaCePyExtModuleGenerator, CUDAPyExtModuleGene
     pass
 
 
-class BaseDaceBackend(BaseGTBackend, CLIBackendMixin):
-
+class BaseDaceBackend(BaseGTBackend):
     GT_BACKEND_T = "dace"
-    PYEXT_GENERATOR_CLASS = DaCeExtGenerator  # type: ignore
+    PYEXT_GENERATOR_CLASS = DaCeExtGenerator
 
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         self.check_options(self.builder.options)
 
-        pyext_module_name: Optional[str]
-        pyext_file_path: Optional[str]
-
         # TODO(havogt) add bypass if computation has no effect
-        pyext_module_name, pyext_file_path = self.generate_extension()
+        self.generate_extension()
 
         # Generate and return the Python wrapper class
-        return self.make_module(
-            pyext_module_name=pyext_module_name,
-            pyext_file_path=pyext_file_path,
-        )
+        return self.make_module()
 
 
 @register
 class DaceCPUBackend(BaseDaceBackend):
-
     name = "dace:cpu"
-    languages = {"computation": "c++", "bindings": ["python"]}
-    storage_info = {
+    languages: ClassVar[dict] = {"computation": "c++", "bindings": ["python"]}
+    storage_info: ClassVar[layout.LayoutInfo] = {
         "alignment": 1,
         "device": "cpu",
-        "layout_map": gt_storage.layout.layout_maker_factory((0, 1, 2)),
-        "is_optimal_layout": gt_storage.layout.layout_checker_factory(
-            gt_storage.layout.layout_maker_factory((0, 1, 2))
-        ),
+        "layout_map": layout.layout_maker_factory((0, 1, 2)),
+        "is_optimal_layout": layout.layout_checker_factory(layout.layout_maker_factory((0, 1, 2))),
     }
     MODULE_GENERATOR_CLASS = DaCePyExtModuleGenerator
 
     options = BaseGTBackend.GT_BACKEND_OPTS
 
-    def generate_extension(self, **kwargs: Any) -> Tuple[str, str]:
-        return self.make_extension(stencil_ir=self.builder.gtir, uses_cuda=False)
+    def generate_extension(self) -> None:
+        return self.make_extension(uses_cuda=False)
 
 
 @register
@@ -808,20 +800,18 @@ class DaceGPUBackend(BaseDaceBackend):
     """DaCe python backend using gt4py.cartesian.gtc."""
 
     name = "dace:gpu"
-    languages = {"computation": "cuda", "bindings": ["python"]}
-    storage_info = {
+    languages: ClassVar[dict] = {"computation": "cuda", "bindings": ["python"]}
+    storage_info: ClassVar[layout.LayoutInfo] = {
         "alignment": 32,
         "device": "gpu",
-        "layout_map": gt_storage.layout.layout_maker_factory((2, 1, 0)),
-        "is_optimal_layout": gt_storage.layout.layout_checker_factory(
-            gt_storage.layout.layout_maker_factory((2, 1, 0))
-        ),
+        "layout_map": layout.layout_maker_factory((2, 1, 0)),
+        "is_optimal_layout": layout.layout_checker_factory(layout.layout_maker_factory((2, 1, 0))),
     }
     MODULE_GENERATOR_CLASS = DaCeCUDAPyExtModuleGenerator
-    options = {
+    options: ClassVar[GTBackendOptions] = {
         **BaseGTBackend.GT_BACKEND_OPTS,
         "device_sync": {"versioning": True, "type": bool},
     }
 
-    def generate_extension(self, **kwargs: Any) -> Tuple[str, str]:
-        return self.make_extension(stencil_ir=self.builder.gtir, uses_cuda=True)
+    def generate_extension(self) -> None:
+        return self.make_extension(uses_cuda=True)
