@@ -1,16 +1,12 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import copy
 import inspect
@@ -23,7 +19,7 @@ import dace.data
 import dace.frontend.python.common
 from dace.frontend.python.common import SDFGClosure, SDFGConvertible
 
-from gt4py import cartesian as gt4pyc
+from gt4py.cartesian import backend as gt_backend
 from gt4py.cartesian.backend.dace_backend import freeze_origin_domain_sdfg
 from gt4py.cartesian.definitions import AccessKind, DomainInfo, FieldInfo
 from gt4py.cartesian.stencil_object import ArgsInfo, FrozenStencil, StencilObject
@@ -50,16 +46,13 @@ def add_optional_fields(
         if info.access == AccessKind.NONE and name in kwargs and name not in sdfg.arrays:
             outer_array = kwargs[name]
             sdfg.add_array(
-                name,
-                shape=outer_array.shape,
-                dtype=outer_array.dtype,
-                strides=outer_array.strides,
+                name, shape=outer_array.shape, dtype=outer_array.dtype, strides=outer_array.strides
             )
 
     for name, info in parameter_info.items():
         if info.access == AccessKind.NONE and name in kwargs and name not in sdfg.symbols:
             if isinstance(kwargs[name], dace.data.Scalar):
-                sdfg.add_symbol(name, stype=kwargs[name].dtype)
+                sdfg.add_scalar(name, dtype=kwargs[name].dtype)
             else:
                 sdfg.add_symbol(name, stype=dace.typeclass(type(kwargs[name])))
     return sdfg
@@ -67,8 +60,7 @@ def add_optional_fields(
 
 @dataclass(frozen=True)
 class DaCeFrozenStencil(FrozenStencil, SDFGConvertible):
-
-    stencil_object: "DaCeStencilObject"
+    stencil_object: DaCeStencilObject
     origin: Dict[str, Tuple[int, ...]]
     domain: Tuple[int, ...]
     sdfg: dace.SDFG
@@ -89,7 +81,6 @@ class DaCeFrozenStencil(FrozenStencil, SDFGConvertible):
 
 
 class DaCeStencilObject(StencilObject, SDFGConvertible):
-
     _sdfg = None
     SDFG_PATH: str
 
@@ -106,39 +97,32 @@ class DaCeStencilObject(StencilObject, SDFGConvertible):
         return domain, origins_tuple
 
     def freeze(
-        self: "DaCeStencilObject", *, origin: Dict[str, Tuple[int, ...]], domain: Tuple[int, ...]
+        self: DaCeStencilObject, *, origin: Dict[str, Tuple[int, ...]], domain: Tuple[int, ...]
     ) -> DaCeFrozenStencil:
-
         key = DaCeStencilObject._get_domain_origin_key(domain, origin)
+
+        # check if same sdfg already cached on disk
         if key in self._frozen_cache:
             return self._frozen_cache[key]
 
-        frozen_hash = shash(origin, domain)
-
-        # check if same sdfg already cached on disk
-        basename = os.path.splitext(self.SDFG_PATH)[0]
-        filename = basename + "_" + str(frozen_hash) + ".sdfg"
-        try:
-            frozen_sdfg = dace.SDFG.from_file(filename)
-        except FileNotFoundError:
-            # otherwise, wrap and save sdfg from scratch
-            inner_sdfg = self.sdfg()
-
-            frozen_sdfg = freeze_origin_domain_sdfg(
-                inner_sdfg,
-                arg_names=self.__sdfg_signature__()[0],
-                field_info=self.field_info,
-                origin=origin,
-                domain=domain,
-            )
-            frozen_sdfg.save(filename)
-
+        # otherwise, wrap and save sdfg from scratch
+        frozen_sdfg = freeze_origin_domain_sdfg(
+            self.sdfg(),
+            arg_names=list(self.__sdfg_signature__()[0]),
+            field_info=self.field_info,
+            origin=origin,
+            domain=domain,
+        )
         self._frozen_cache[key] = DaCeFrozenStencil(self, origin, domain, frozen_sdfg)
+
+        basename = os.path.splitext(self.SDFG_PATH)[0]
+        filename = f"{basename}_{shash(origin, domain)}.sdfg"
+        frozen_sdfg.save(filename)
+
         return self._frozen_cache[key]
 
     @classmethod
     def sdfg(cls) -> dace.SDFG:
-
         if getattr(cls, "_sdfg", None) is None:
             cls._sdfg = dace.SDFG.from_file(cls.SDFG_PATH)
 
@@ -148,8 +132,8 @@ class DaCeStencilObject(StencilObject, SDFGConvertible):
         self,
         constant_args: Dict[str, Any],
         given_args: Set[str],
-        parent_closure: Optional["dace.frontend.python.common.SDFGClosure"] = None,
-    ) -> "dace.frontend.python.common.SDFGClosure":
+        parent_closure: Optional[dace.frontend.python.common.SDFGClosure] = None,
+    ) -> dace.frontend.python.common.SDFGClosure:
         return dace.frontend.python.common.SDFGClosure()
 
     def __sdfg__(self, *args, **kwargs) -> dace.SDFG:
@@ -191,15 +175,13 @@ class DaCeStencilObject(StencilObject, SDFGConvertible):
         origin: Optional[Dict[str, Tuple[int, ...]]] = None,
         **kwargs,
     ):
-        backend_cls = gt4pyc.backend.from_name(backend)
-        assert backend_cls is not None
+        backend_cls = gt_backend.from_name(backend)
         args_iter = iter(args)
         args_as_kwargs = {
             name: (kwargs[name] if name in kwargs else next(args_iter)) for name in arg_names
         }
         arg_infos = _extract_array_infos(
-            field_args=args_as_kwargs,
-            device=backend_cls.storage_info["device"],
+            field_args=args_as_kwargs, device=backend_cls.storage_info["device"]
         )
 
         origin = DaCeStencilObject._normalize_origins(arg_infos, field_info, origin)
