@@ -1,22 +1,18 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import abc
+import importlib.resources as importlib_resources
 import numbers
-import os
 from dataclasses import dataclass, field
-from typing import TYPE_CHECKING, Any, Dict, List, Literal, Optional, Set, cast
+from typing import TYPE_CHECKING, Any, Literal, cast
 
 import jinja2
 import numpy
@@ -38,30 +34,28 @@ if TYPE_CHECKING:
 
 @dataclass
 class ModuleData:
-    domain_info: Optional[DomainInfo] = None
-    field_info: Dict[str, FieldInfo] = field(default_factory=dict)
-    parameter_info: Dict[str, ParameterInfo] = field(default_factory=dict)
-    unreferenced: List[str] = field(default_factory=list)
+    domain_info: DomainInfo | None = None
+    field_info: dict[str, FieldInfo] = field(default_factory=dict)
+    parameter_info: dict[str, ParameterInfo] = field(default_factory=dict)
+    unreferenced: list[str] = field(default_factory=list)
 
     @property
-    def field_names(self) -> Set[str]:
+    def field_names(self) -> set[str]:
         """Set of all field names."""
         return set(self.field_info.keys())
 
     @property
-    def parameter_names(self) -> Set[str]:
+    def parameter_names(self) -> set[str]:
         """Set of all parameter names."""
         return set(self.parameter_info.keys())
 
 
-_args_data_cache: Dict[StencilID, ModuleData] = {}
+_args_data_cache: dict[StencilID, ModuleData] = {}
 
 
 def make_args_data_from_gtir(pipeline: GtirPipeline) -> ModuleData:
     """
     Compute module data containing information about stencil arguments from gtir.
-
-    This is no longer compatible with the legacy backends.
     """
     if pipeline.stencil_id in _args_data_cache:
         return _args_data_cache[pipeline.stencil_id]
@@ -113,39 +107,25 @@ def make_args_data_from_gtir(pipeline: GtirPipeline) -> ModuleData:
 
 
 class BaseModuleGenerator(abc.ABC):
-
     SOURCE_LINE_LENGTH = 120
     TEMPLATE_INDENT_SIZE = 4
-    DOMAIN_ARG_NAME = "_domain_"
-    ORIGIN_ARG_NAME = "_origin_"
-    SPLITTERS_NAME = "_splitters_"
+    TEMPLATE_RESOURCE = "stencil_module.py.in"
 
-    TEMPLATE_PATH = os.path.join(os.path.dirname(__file__), "templates", "stencil_module.py.in")
-
-    _builder: Optional["StencilBuilder"]
+    builder: StencilBuilder
     args_data: ModuleData
     template: jinja2.Template
 
-    def __init__(self, builder: Optional["StencilBuilder"] = None):
-        self._builder = builder
+    def __init__(self, builder: StencilBuilder):
+        self.builder = builder
         self.args_data = ModuleData()
-        with open(self.TEMPLATE_PATH, "r") as f:
-            self.template = jinja2.Template(f.read())
+        self.template = jinja2.Template(
+            importlib_resources.files("gt4py.cartesian.backend.templates")
+            .joinpath(self.TEMPLATE_RESOURCE)
+            .read_text()
+        )
 
-    def __call__(
-        self,
-        args_data: ModuleData,
-        builder: Optional["StencilBuilder"] = None,
-        **kwargs: Any,
-    ) -> str:
-        """
-        Generate source code for a Python module containing a StencilObject.
-
-        A possible reaosn for extending is processing additional kwargs,
-        using a different template might require completely overriding.
-        """
-        if builder:
-            self._builder = builder
+    def __call__(self, args_data: ModuleData) -> str:
+        """Generate source code for a Python module containing a StencilObject."""
         self.args_data = args_data
 
         module_source = self.template.render(
@@ -176,19 +156,6 @@ class BaseModuleGenerator(abc.ABC):
 
         return module_source
 
-    @property
-    def builder(self) -> "StencilBuilder":
-        """
-        Expose the builder reference.
-
-        Raises a runtime error if the builder reference is not initialized.
-        This is necessary because other parts of the public API depend on it before it is
-        guaranteed to be initialized.
-        """
-        if not self._builder:
-            raise RuntimeError("Builder attribute not initialized!")
-        return self._builder
-
     @property
     def backend_name(self) -> str:
         return self.builder.backend.name
@@ -206,7 +173,7 @@ class BaseModuleGenerator(abc.ABC):
         """
         Generate the name of the stencil class.
 
-        This should ususally be deferred to the chosen caching strategy via
+        This should usually be deferred to the chosen caching strategy via
         the builder object (see default implementation).
         """
         return self.builder.class_name
@@ -233,7 +200,7 @@ class BaseModuleGenerator(abc.ABC):
         """
         return self.backend_name
 
-    def generate_sources(self) -> Dict[str, str]:
+    def generate_sources(self) -> dict[str, str]:
         """
         Return the source code of the stencil definition in string format.
 
@@ -246,7 +213,7 @@ class BaseModuleGenerator(abc.ABC):
             }
         return {}
 
-    def generate_constants(self) -> Dict[str, str]:
+    def generate_constants(self) -> dict[str, str]:
         """
         Return a mapping of named numeric constants passed as externals.
 
@@ -260,7 +227,7 @@ class BaseModuleGenerator(abc.ABC):
             }
         return {}
 
-    def generate_options(self) -> Dict[str, Any]:
+    def generate_options(self) -> dict[str, Any]:
         """
         Return dictionary of build options.
 
