@@ -1,22 +1,18 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import pathlib
-from typing import TYPE_CHECKING, Any, ClassVar, Dict, Type, Union, cast
+from typing import TYPE_CHECKING, Any, ClassVar, cast
 
 from gt4py import storage as gt_storage
-from gt4py.cartesian.backend.base import BaseBackend, CLIBackendMixin, register
+from gt4py.cartesian.backend.base import BaseBackend, register
 from gt4py.cartesian.backend.module_generator import BaseModuleGenerator
 from gt4py.cartesian.gtc.gtir_to_oir import GTIRToOIR
 from gt4py.cartesian.gtc.numpy import npir
@@ -30,6 +26,7 @@ from gt4py.cartesian.gtc.passes.oir_optimizations.caches import (
     PruneKCacheFlushes,
 )
 from gt4py.cartesian.gtc.passes.oir_pipeline import DefaultPipeline, OirPipeline
+from gt4py.cartesian.stencil_builder import StencilBuilder
 from gt4py.eve.codegen import format_source
 
 
@@ -38,6 +35,9 @@ if TYPE_CHECKING:
 
 
 class ModuleGenerator(BaseModuleGenerator):
+    def __init__(self, builder: StencilBuilder) -> None:
+        super().__init__(builder)
+
     def generate_imports(self) -> str:
         comp_pkg = (
             self.builder.caching.module_prefix + "computation" + self.builder.caching.module_postfix
@@ -57,11 +57,11 @@ class ModuleGenerator(BaseModuleGenerator):
         return f"computation.run({', '.join(params)})"
 
     @property
-    def backend(self) -> "NumpyBackend":
+    def backend(self) -> NumpyBackend:
         return cast(NumpyBackend, self.builder.backend)
 
 
-def recursive_write(root_path: pathlib.Path, tree: Dict[str, Union[str, dict]]):
+def recursive_write(root_path: pathlib.Path, tree: dict[str, str | dict]):
     root_path.mkdir(parents=True, exist_ok=True)
     for key, value in tree.items():
         if isinstance(value, dict):
@@ -72,20 +72,20 @@ def recursive_write(root_path: pathlib.Path, tree: Dict[str, Union[str, dict]]):
 
 
 @register
-class NumpyBackend(BaseBackend, CLIBackendMixin):
+class NumpyBackend(BaseBackend):
     """NumPy backend using gtc."""
 
     name = "numpy"
-    options: ClassVar[Dict[str, Any]] = {
+    options: ClassVar[dict[str, Any]] = {
         "oir_pipeline": {"versioning": True, "type": OirPipeline},
         # TODO: Implement this option in source code
         "ignore_np_errstate": {"versioning": True, "type": bool},
     }
     storage_info = gt_storage.layout.NaiveCPULayout
-    languages = {"computation": "python", "bindings": ["python"]}
+    languages: ClassVar[dict] = {"computation": "python", "bindings": ["python"]}
     MODULE_GENERATOR_CLASS = ModuleGenerator
 
-    def generate_computation(self) -> Dict[str, Union[str, Dict]]:
+    def generate_computation(self) -> dict[str, str | dict]:
         computation_name = (
             self.builder.caching.module_prefix
             + "computation"
@@ -100,11 +100,7 @@ class NumpyBackend(BaseBackend, CLIBackendMixin):
 
         return {computation_name: source}
 
-    def generate_bindings(self, language_name: str) -> Dict[str, Union[str, Dict]]:
-        super().generate_bindings(language_name)
-        return {self.builder.module_path.name: self.make_module_source()}
-
-    def generate(self) -> Type["StencilObject"]:
+    def generate(self) -> type[StencilObject]:
         self.check_options(self.builder.options)
         src_dir = self.builder.module_path.parent
         if not self.builder.options._impl_opts.get("disable-code-generation", False):
@@ -117,12 +113,7 @@ class NumpyBackend(BaseBackend, CLIBackendMixin):
         oir_pipeline = self.builder.options.backend_opts.get(
             "oir_pipeline",
             DefaultPipeline(
-                skip=[
-                    IJCacheDetection,
-                    KCacheDetection,
-                    PruneKCacheFills,
-                    PruneKCacheFlushes,
-                ]
+                skip=[IJCacheDetection, KCacheDetection, PruneKCacheFills, PruneKCacheFlushes]
             ),
         )
         oir_node = oir_pipeline.run(base_oir)
