@@ -1,33 +1,33 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import contextlib
 import copy
-import distutils
-import distutils.sysconfig
 import io
 import os
 import shutil
-from typing import Any, Dict, List, Optional, Tuple, Type, Union, cast, overload
+from typing import Any, Dict, List, Literal, Optional, Tuple, Type, TypedDict, Union
 
 import pybind11
 import setuptools
+from setuptools import distutils
 from setuptools.command.build_ext import build_ext
 
+from gt4py._core import definitions as core_defs
 from gt4py.cartesian import config as gt_config
 
 
+class SetuptoolsArgs(TypedDict):
+    name: str
+    ext_modules: list[setuptools.Extension]
+    script_args: list[str]
+
+
 def get_dace_module_path() -> Optional[str]:
     try:
         import dace
@@ -49,13 +49,15 @@ def get_cuda_compute_capability():
 def get_gt_pyext_build_opts(
     *,
     debug_mode: bool = False,
+    opt_level: Literal["0", "1", "2", "3", "s"] = "3",
+    extra_opt_flags: str = "",
     add_profile_info: bool = False,
     uses_openmp: bool = True,
     uses_cuda: bool = False,
-    gt_version: int = 1,
 ) -> Dict[str, Union[str, List[str], Dict[str, Any]]]:
-    include_dirs = [gt_config.build_settings["boost_include_path"]]
+    include_dirs: list[str] = []
     extra_compile_args_from_config = gt_config.build_settings["extra_compile_args"]
+    is_rocm_gpu = core_defs.CUPY_DEVICE_TYPE == core_defs.DeviceType.ROCM
 
     if uses_cuda:
         compute_capability = get_cuda_compute_capability()
@@ -73,55 +75,67 @@ def get_gt_pyext_build_opts(
 
     gt_include_path = gt_config.build_settings["gt_include_path"]
 
-    import os
-
     extra_compile_args = dict(
         cxx=[
             "-std=c++17",
             "-ftemplate-depth={}".format(gt_config.build_settings["cpp_template_depth"]),
             "-fvisibility=hidden",
             "-fPIC",
+            # A compiler is allowed to choose if `char` is signed or unsigned. We force the signed behavior
+            # because `char` is used to represent the `int8` type in GT4Py programs.
+            "-fsigned-char",
             "-isystem{}".format(gt_include_path),
-            "-isystem{}".format(gt_config.build_settings["boost_include_path"]),
-            "-DBOOST_PP_VARIADICS",
             *extra_compile_args_from_config["cxx"],
-        ],
-        nvcc=[
-            "-std=c++17",
-            "-ftemplate-depth={}".format(gt_config.build_settings["cpp_template_depth"]),
-            "-arch=sm_{}".format(cuda_arch),
+        ]
+    )
+    extra_compile_args["cuda"] = [
+        "-std=c++17",
+        "-ftemplate-depth={}".format(gt_config.build_settings["cpp_template_depth"]),
+        *extra_compile_args_from_config["cuda"],
+    ]
+    if is_rocm_gpu:
+        extra_compile_args["cuda"] += [
+            "-isystem{}".format(gt_include_path),
+            "-fvisibility=hidden",
+            "-fPIC",
+        ]
+    else:
+        extra_compile_args["cuda"] += [
             "-isystem={}".format(gt_include_path),
-            "-isystem={}".format(gt_config.build_settings["boost_include_path"]),
-            "-DBOOST_PP_VARIADICS",
-            "-DBOOST_OPTIONAL_CONFIG_USE_OLD_IMPLEMENTATION_OF_OPTIONAL",
-            "-DBOOST_OPTIONAL_USE_OLD_DEFINITION_OF_NONE",
+            "-arch=sm_{}".format(cuda_arch),
             "--expt-relaxed-constexpr",
             "--compiler-options",
             "-fvisibility=hidden",
             "--compiler-options",
             "-fPIC",
-            *extra_compile_args_from_config["nvcc"],
-        ],
+        ]
+    extra_link_args = copy.deepcopy(gt_config.build_settings["extra_link_args"])
+
+    mode_flags = (
+        ["-O0", "-ggdb"] if debug_mode else [f"-O{opt_level}", "-DNDEBUG", *extra_opt_flags.split()]
     )
-    extra_link_args = gt_config.build_settings["extra_link_args"]
 
-    mode_flags = ["-O0", "-ggdb"] if debug_mode else ["-O3", "-DNDEBUG"]
     extra_compile_args["cxx"].extend(mode_flags)
-    extra_compile_args["nvcc"].extend(mode_flags)
+    extra_compile_args["cuda"].extend(mode_flags)
     extra_link_args.extend(mode_flags)
 
     if dace_path := get_dace_module_path():
         extra_compile_args["cxx"].append(
             "-isystem{}".format(os.path.join(dace_path, "runtime/include"))
         )
-        extra_compile_args["nvcc"].append(
-            "-isystem={}".format(os.path.join(dace_path, "runtime/include"))
-        )
+        if is_rocm_gpu:
+            extra_compile_args["cuda"].append(
+                "-isystem{}".format(os.path.join(dace_path, "runtime/include"))
+            )
+        else:
+            extra_compile_args["cuda"].append(
+                "-isystem={}".format(os.path.join(dace_path, "runtime/include"))
+            )
 
     if add_profile_info:
         profile_flags = ["-pg"]
         extra_compile_args["cxx"].extend(profile_flags)
-        extra_compile_args["nvcc"].extend(profile_flags)
+        extra_compile_args["cuda"].extend(profile_flags)
         extra_link_args.extend(profile_flags)
 
     if uses_cuda:
@@ -142,8 +156,11 @@ def get_gt_pyext_build_opts(
         if uses_cuda:
             cuda_flags = []
             for cpp_flag in cpp_flags:
-                cuda_flags.extend(["--compiler-options", cpp_flag])
-            build_opts["extra_compile_args"]["nvcc"].extend(cuda_flags)
+                if is_rocm_gpu:
+                    cuda_flags.extend([cpp_flag])
+                else:
+                    cuda_flags.extend(["--compiler-options", cpp_flag])
+            build_opts["extra_compile_args"]["cuda"].extend(cuda_flags)
         elif cpp_flags:
             build_opts["extra_compile_args"].extend(cpp_flags)
 
@@ -154,35 +171,23 @@ def get_gt_pyext_build_opts(
     return build_opts
 
 
-# The following tells mypy to accept unpacking kwargs
-@overload
-def build_pybind_ext(
-    name: str,
-    sources: list,
-    build_path: str,
-    target_path: str,
-    **kwargs: str,
-) -> Tuple[str, str]:
-    ...
+def setuptools_setup(*, build_ext_class: type[build_ext] | None, **kwargs) -> None:
+    """
+    Calls setuptools.setup() with 'cmdclass' set to 'build_ext_class'.
 
+    This is a workaround because any config file that sets an element of
+    'cmdclass' will override (instead of extend) the 'cmdclass' dict passed
+    as argument to 'setuptools.setup()'.
 
-@overload
-def build_pybind_ext(
-    name: str,
-    sources: list,
-    build_path: str,
-    target_path: str,
-    *,
-    include_dirs: Optional[List[str]] = None,
-    library_dirs: Optional[List[str]] = None,
-    libraries: Optional[List[str]] = None,
-    extra_compile_args: Optional[Union[List[str], Dict[str, List[str]]]] = None,
-    extra_link_args: Optional[List[str]] = None,
-    build_ext_class: Type = None,
-    verbose: bool = False,
-    clean: bool = False,
-) -> Tuple[str, str]:
-    ...
+    Note: This is NOT thread-safe.
+    """
+    old_setup_stop_after = setuptools.distutils.core._setup_stop_after
+    setuptools.distutils.core._setup_stop_after = "commandline"
+    dist = setuptools.setup(**kwargs)
+    if build_ext_class is not None:
+        dist.cmdclass.update({"build_ext": build_ext_class})
+    setuptools.distutils.core._setup_stop_after = old_setup_stop_after
+    setuptools.distutils.core.run_commands(dist)
 
 
 def build_pybind_ext(
@@ -196,7 +201,7 @@ def build_pybind_ext(
     libraries: Optional[List[str]] = None,
     extra_compile_args: Optional[Union[List[str], Dict[str, List[str]]]] = None,
     extra_link_args: Optional[List[str]] = None,
-    build_ext_class: Type = None,
+    build_ext_class: Optional[Type] = None,
     verbose: bool = False,
     clean: bool = False,
 ) -> Tuple[str, str]:
@@ -214,7 +219,11 @@ def build_pybind_ext(
     py_extension = setuptools.Extension(
         name,
         sources,
-        include_dirs=[pybind11.get_include(), pybind11.get_include(user=True), *include_dirs],
+        include_dirs=[
+            pybind11.get_include(),
+            pybind11.get_include(user=True),
+            *include_dirs,
+        ],
         library_dirs=[*library_dirs],
         libraries=[*libraries],
         language="c++",
@@ -222,29 +231,25 @@ def build_pybind_ext(
         extra_link_args=extra_link_args,
     )
 
-    setuptools_args = dict(
+    setuptools_args = SetuptoolsArgs(
         name=name,
         ext_modules=[py_extension],
         script_args=[
             "build_ext",
-            # "--parallel={}".format(gt_config.build_settings["parallel_jobs"]),
             "--build-temp={}".format(build_path),
             "--build-lib={}".format(build_path),
             "--force",
         ],
     )
-    if build_ext_class is not None:
-        setuptools_args["cmdclass"] = {"build_ext": build_ext_class}
 
     if verbose:
-        script_args = cast(List[str], setuptools_args["script_args"])
-        script_args.append("-v")
-        setuptools.setup(**setuptools_args)
+        setuptools_args["script_args"].append("-v")
+        setuptools_setup(**setuptools_args, build_ext_class=build_ext_class)
     else:
         setuptools_args["script_args"].append("-q")
         io_out, io_err = io.StringIO(), io.StringIO()
         with contextlib.redirect_stdout(io_out), contextlib.redirect_stderr(io_err):
-            setuptools.setup(**setuptools_args)
+            setuptools_setup(**setuptools_args, build_ext_class=build_ext_class)
 
     # Copy extension in target path
     module_name = py_extension._full_name
@@ -266,18 +271,6 @@ def build_pybind_ext(
     return module_name, dest_path
 
 
-# The following tells mypy to accept unpacking kwargs
-@overload
-def build_pybind_cuda_ext(
-    name: str,
-    sources: list,
-    build_path: str,
-    target_path: str,
-    **kwargs: str,
-) -> Tuple[str, str]:
-    pass
-
-
 def build_pybind_cuda_ext(
     name: str,
     sources: list,
@@ -297,7 +290,10 @@ def build_pybind_cuda_ext(
     library_dirs = library_dirs or []
     library_dirs = [*library_dirs, gt_config.build_settings["cuda_library_path"]]
     libraries = libraries or []
-    libraries = [*libraries, "cudart"]
+    if core_defs.CUPY_DEVICE_TYPE == core_defs.DeviceType.ROCM:
+        libraries = [*libraries, "hiprtc"]
+    else:
+        libraries = [*libraries, "cudart"]
     extra_compile_args = extra_compile_args or []
 
     return build_pybind_ext(
@@ -318,7 +314,7 @@ def build_pybind_cuda_ext(
 
 def _clean_build_flags(config_vars: Dict[str, str]) -> None:
     for key, value in config_vars.items():
-        if type(value) == str:
+        if isinstance(value, str):
             value = " " + value + " "
             for s in value.split(" "):
                 if (
@@ -342,16 +338,19 @@ class CUDABuildExtension(build_ext, object):
         # Save references to the original methods
         original_compile = self.compiler._compile
 
-        def nvcc_compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
+        def cuda_compile(obj, src, ext, cc_args, extra_postargs, pp_opts):
             original_compiler_so = self.compiler.compiler_so
             # Copy before we make any modifications.
             cflags = copy.deepcopy(extra_postargs)
             try:
                 if os.path.splitext(src)[-1] == ".cu":
-                    nvcc_exec = os.path.join(gt_config.build_settings["cuda_bin_path"], "nvcc")
-                    self.compiler.set_executable("compiler_so", [nvcc_exec])
+                    if core_defs.CUPY_DEVICE_TYPE == core_defs.DeviceType.ROCM:
+                        cuda_exec = os.path.join(gt_config.build_settings["cuda_bin_path"], "hipcc")
+                    else:
+                        cuda_exec = os.path.join(gt_config.build_settings["cuda_bin_path"], "nvcc")
+                    self.compiler.set_executable("compiler_so", [cuda_exec])
                     if isinstance(cflags, dict):
-                        cflags = cflags["nvcc"]
+                        cflags = cflags["cuda"]
                 elif isinstance(cflags, dict):
                     cflags = cflags["cxx"]
 
@@ -360,6 +359,6 @@ class CUDABuildExtension(build_ext, object):
                 # Put the original compiler back in place.
                 self.compiler.set_executable("compiler_so", original_compiler_so)
 
-        self.compiler._compile = nvcc_compile
+        self.compiler._compile = cuda_compile
         build_ext.build_extensions(self)
         self.compiler._compile = original_compile
