@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import enum
 import functools
@@ -111,17 +105,14 @@ class BuildOptions(AttributeClassLike):
 
     @property
     def qualified_name(self):
-        name = ".".join([self.module, self.name])
-        return name
+        return ".".join([self.module, self.name])
 
     @property
     def shashed_id(self):
-        result = gt_utils.shashed_id(
+        return gt_utils.shashed_id(
             self.name, self.module, self.format_source, *tuple(sorted(self.backend_opts.items()))
         )
 
-        return result
-
 
 @attribclass(frozen=True)
 class StencilID(AttributeClassLike):
