@@ -1,42 +1,23 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import abc
-from typing import Any, Dict, Optional, Type, Union
+from typing import Any
 
 from gt4py.cartesian import utils as gt_utils
 from gt4py.cartesian.definitions import BuildOptions, StencilID
 from gt4py.cartesian.gtc import gtir
-from gt4py.cartesian.type_hints import AnnotatedStencilFunc, StencilFunc
-
-
-REGISTRY = gt_utils.Registry()
-AnyStencilFunc = Union[StencilFunc, AnnotatedStencilFunc]
-
-
-def from_name(name: str) -> Optional[Type["Frontend"]]:
-    """Return frontend by name."""
-    return REGISTRY.get(name, None)
-
-
-def register(frontend_cls: Type["Frontend"]) -> None:
-    """Register a new frontend."""
-    return REGISTRY.register(frontend_cls.name, frontend_cls)
+from gt4py.cartesian.type_hints import AnnotatedStencilFunc, AnyStencilFunc
 
 
 class Frontend(abc.ABC):
-
     name: str
     """Frontend name."""
 
@@ -46,7 +27,7 @@ class Frontend(abc.ABC):
         cls,
         qualified_name: str,
         definition: AnyStencilFunc,
-        externals: Dict[str, Any],
+        externals: dict[str, Any],
         options_id: str,
     ) -> StencilID:
         """
@@ -76,9 +57,10 @@ class Frontend(abc.ABC):
     def generate(
         cls,
         definition: AnyStencilFunc,
-        externals: Dict[str, Any],
-        dtypes: Dict[Type, Type],
+        externals: dict[str, Any],
+        dtypes: dict[type, type],
         options: BuildOptions,
+        backend_name: str,
     ) -> gtir.Stencil:
         """
         Generate a StencilDefinition from a stencil Python function.
@@ -99,7 +81,7 @@ class Frontend(abc.ABC):
     @classmethod
     @abc.abstractmethod
     def prepare_stencil_definition(
-        cls, definition: AnyStencilFunc, externals: Dict[str, Any]
+        cls, definition: AnyStencilFunc, externals: dict[str, Any]
     ) -> AnnotatedStencilFunc:
         """
         Annotate the stencil function if not already done so.
@@ -113,3 +95,21 @@ class Frontend(abc.ABC):
             If there is a error resolving external types.
         """
         pass
+
+
+REGISTRY = gt_utils.Registry[type[Frontend]]()
+
+
+def from_name(name: str) -> type[Frontend]:
+    """Return frontend by name."""
+    frontend_cls = REGISTRY.get(name, None)
+    if frontend_cls is None:
+        raise NotImplementedError(
+            f"Frontend '{name} is not implemented. Options are: {REGISTRY.names}."
+        )
+    return frontend_cls
+
+
+def register(frontend_cls: type[Frontend]) -> type[Frontend]:
+    """Register a new frontend."""
+    return REGISTRY.register(frontend_cls.name, frontend_cls)
