@@ -1,22 +1,16 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import copy
 import functools
 import itertools
 import numbers
-from typing import Any, Dict, List, Optional, Tuple, Union, cast
+from typing import Any, Dict, Final, List, Optional, Tuple, Union, cast
 
 import numpy as np
 
@@ -107,7 +101,7 @@ class UnrollVectorAssignments(IRNodeMapper):
     def apply(cls, root, **kwargs):
         return cls().visit(root, **kwargs)
 
-    def _is_vector_assignment(self, stmt: Node, fields_decls: Dict[str, FieldDecl]):
+    def _is_vector_assignment(self, stmt: Node, fields_decls: Dict[str, FieldDecl]) -> bool:
         if not isinstance(stmt, Assign):
             return False
 
@@ -137,7 +131,7 @@ class UnrollVectorAssignments(IRNodeMapper):
     def _nested_list_dim(self, a: List) -> List[int]:
         if not isinstance(a, list):
             return []
-        return [len(a)] + self._nested_list_dim(a[0])
+        return [len(a), *self._nested_list_dim(a[0])]
 
     def visit_Assign(
         self, node: Assign, *, fields_decls: Dict[str, FieldDecl], **kwargs
@@ -181,14 +175,14 @@ class UnrollVectorExpressions(IRNodeMapper):
         # if the expression is just a scalar broadcast to the expected dimensions
         if not isinstance(result, list):
             result = functools.reduce(
-                lambda val, len: [val for _ in range(len)], reversed(expected_dim), result
+                lambda val, len_: [val for _ in range(len_)], reversed(expected_dim), result
             )
         return result
 
     def visit_FieldRef(self, node: FieldRef, *, fields_decls: Dict[str, FieldDecl], **kwargs):
         name = node.name
         if fields_decls[name].data_dims:
-            field_list = []
+            field_list: List[Union[FieldRef, List[FieldRef]]] = []
             # vector
             if len(fields_decls[name].data_dims) == 1:
                 dims = fields_decls[name].data_dims[0]
@@ -196,17 +190,14 @@ class UnrollVectorExpressions(IRNodeMapper):
                     data_type = DataType.INT32
                     data_index = [ScalarLiteral(value=index, data_type=data_type)]
                     element_ref = FieldRef(
-                        name=node.name,
-                        offset=node.offset,
-                        data_index=data_index,
-                        loc=node.loc,
+                        name=node.name, offset=node.offset, data_index=data_index, loc=node.loc
                     )
                     field_list.append(element_ref)
             # matrix
             elif len(fields_decls[name].data_dims) == 2:
                 rows, cols = fields_decls[name].data_dims
                 for row in range(rows):
-                    row_list = []
+                    row_list: List[FieldRef] = []
                     for col in range(cols):
                         data_type = DataType.INT32
                         data_index = [
@@ -255,40 +246,48 @@ class UnrollVectorExpressions(IRNodeMapper):
                     acc = BinOpExpr(op=BinaryOperator.ADD, lhs=acc, rhs=mul, loc=node.loc)
 
                 result.append(acc)
-        else:
-            # vector and vector
-            if isinstance(lhs, list) and isinstance(rhs, list):
-                assert len(lhs) == len(rhs)
-                for lhs_el, rhs_el in zip(lhs, rhs):
-                    result.append(BinOpExpr(op=node.op, lhs=lhs_el, rhs=rhs_el, loc=node.loc))
-            # scalar and vector
-            elif isinstance(lhs, Expr) and isinstance(rhs, list):
-                for rhs_el in rhs:
-                    result.append(BinOpExpr(op=node.op, lhs=lhs, rhs=rhs_el, loc=node.loc))
-            elif isinstance(lhs, list) and isinstance(rhs, Expr):
-                for lhs_el in lhs:
-                    result.append(BinOpExpr(op=node.op, lhs=lhs_el, rhs=rhs, loc=node.loc))
-            # scalar and scalar fallback
-            else:
-                result = self.generic_visit(node, **kwargs)
+            return result
 
-        return result
+        # vector and vector
+        if isinstance(lhs, list) and isinstance(rhs, list):
+            assert len(lhs) == len(rhs)
+            for lhs_el, rhs_el in zip(lhs, rhs):
+                result.append(BinOpExpr(op=node.op, lhs=lhs_el, rhs=rhs_el, loc=node.loc))
+            return result
+
+        # scalar and vector
+        if isinstance(lhs, Expr) and isinstance(rhs, list):
+            for rhs_el in rhs:
+                result.append(BinOpExpr(op=node.op, lhs=lhs, rhs=rhs_el, loc=node.loc))
+            return result
+
+        if isinstance(lhs, list) and isinstance(rhs, Expr):
+            for lhs_el in lhs:
+                result.append(BinOpExpr(op=node.op, lhs=lhs_el, rhs=rhs, loc=node.loc))
+            return result
+
+        # scalar and scalar fallback
+        return self.generic_visit(node, **kwargs)
 
 
 class DefIRToGTIR(IRNodeVisitor):
-
-    GT4PY_ITERATIONORDER_TO_GTIR_LOOPORDER = {
+    GT4PY_ITERATIONORDER_TO_GTIR_LOOPORDER: Final[dict[IterationOrder, common.LoopOrder]] = {
         IterationOrder.BACKWARD: common.LoopOrder.BACKWARD,
         IterationOrder.PARALLEL: common.LoopOrder.PARALLEL,
         IterationOrder.FORWARD: common.LoopOrder.FORWARD,
     }
 
-    GT4PY_LEVELMARKER_TO_GTIR_LEVELMARKER = {
+    GT4PY_LEVELMARKER_TO_GTIR_LEVELMARKER: Final[dict[LevelMarker, common.LevelMarker]] = {
         LevelMarker.START: common.LevelMarker.START,
         LevelMarker.END: common.LevelMarker.END,
     }
 
-    GT4PY_OP_TO_GTIR_OP = {
+    GT4PY_OP_TO_GTIR_OP: Final[
+        dict[
+            BinaryOperator,
+            common.ArithmeticOperator | common.LogicalOperator | common.ComparisonOperator,
+        ]
+    ] = {
         # arithmetic
         BinaryOperator.ADD: common.ArithmeticOperator.ADD,
         BinaryOperator.SUB: common.ArithmeticOperator.SUB,
@@ -306,13 +305,13 @@ class DefIRToGTIR(IRNodeVisitor):
         BinaryOperator.GE: common.ComparisonOperator.GE,
     }
 
-    GT4PY_UNARYOP_TO_GTIR = {
+    GT4PY_UNARYOP_TO_GTIR: Final[dict[UnaryOperator, common.UnaryOperator]] = {
         UnaryOperator.POS: common.UnaryOperator.POS,
         UnaryOperator.NEG: common.UnaryOperator.NEG,
         UnaryOperator.NOT: common.UnaryOperator.NOT,
     }
 
-    GT4PY_NATIVE_FUNC_TO_GTIR = {
+    GT4PY_NATIVE_FUNC_TO_GTIR: Final[dict[NativeFunction, common.NativeFunction]] = {
         NativeFunction.ABS: common.NativeFunction.ABS,
         NativeFunction.MIN: common.NativeFunction.MIN,
         NativeFunction.MAX: common.NativeFunction.MAX,
@@ -332,6 +331,7 @@ class DefIRToGTIR(IRNodeVisitor):
         NativeFunction.SQRT: common.NativeFunction.SQRT,
         NativeFunction.EXP: common.NativeFunction.EXP,
         NativeFunction.LOG: common.NativeFunction.LOG,
+        NativeFunction.LOG10: common.NativeFunction.LOG10,
         NativeFunction.GAMMA: common.NativeFunction.GAMMA,
         NativeFunction.CBRT: common.NativeFunction.CBRT,
         NativeFunction.ISFINITE: common.NativeFunction.ISFINITE,
@@ -342,7 +342,7 @@ class DefIRToGTIR(IRNodeVisitor):
         NativeFunction.TRUNC: common.NativeFunction.TRUNC,
     }
 
-    GT4PY_BUILTIN_TO_GTIR = {
+    GT4PY_BUILTIN_TO_GTIR: Final[dict[Builtin, common.BuiltInLiteral]] = {
         Builtin.TRUE: common.BuiltInLiteral.TRUE,
         Builtin.FALSE: common.BuiltInLiteral.FALSE,
     }
@@ -355,14 +355,12 @@ class DefIRToGTIR(IRNodeVisitor):
         field_params = {f.name: self.visit(f) for f in node.api_fields}
         scalar_params = {p.name: self.visit(p) for p in node.parameters}
         vertical_loops = [self.visit(c) for c in node.computations if c.body.stmts]
-        if node.externals is not None:
-            externals = {
-                name: _make_literal(value)
-                for name, value in node.externals.items()
-                if isinstance(value, numbers.Number)
-            }
-        else:
-            externals = {}
+        externals = {
+            name: _make_literal(value)
+            for name, value in (node.externals or {}).items()
+            if isinstance(value, numbers.Number)
+        }
+
         return gtir.Stencil(
             name=node.name,
             api_signature=[
@@ -398,9 +396,7 @@ class DefIRToGTIR(IRNodeVisitor):
                 stmts.append(decl_or_stmt)
         start, end = self.visit(node.interval)
         interval = gtir.Interval(
-            start=start,
-            end=end,
-            loc=location_to_source_location(node.interval.loc),
+            start=start, end=end, loc=location_to_source_location(node.interval.loc)
         )
         return gtir.VerticalLoop(
             interval=interval,
@@ -453,7 +449,7 @@ class DefIRToGTIR(IRNodeVisitor):
             loc=location_to_source_location(node.loc),
         )
 
-    def visit_BuiltinLiteral(self, node: BuiltinLiteral) -> gtir.Literal:  # type: ignore[return]
+    def visit_BuiltinLiteral(self, node: BuiltinLiteral) -> gtir.Literal:
         # currently deals only with boolean literals
         if node.value in self.GT4PY_BUILTIN_TO_GTIR.keys():
             return gtir.Literal(
@@ -489,32 +485,32 @@ class DefIRToGTIR(IRNodeVisitor):
             return gtir.FieldIfStmt(
                 cond=cond,
                 true_branch=gtir.BlockStmt(body=self.visit(node.main_body)),
-                false_branch=gtir.BlockStmt(body=self.visit(node.else_body))
-                if node.else_body
-                else None,
-                loc=location_to_source_location(node.loc),
-            )
-        else:
-            return gtir.ScalarIfStmt(
-                cond=cond,
-                true_branch=gtir.BlockStmt(body=self.visit(node.main_body)),
-                false_branch=gtir.BlockStmt(body=self.visit(node.else_body))
-                if node.else_body
-                else None,
+                false_branch=(
+                    gtir.BlockStmt(body=self.visit(node.else_body)) if node.else_body else None
+                ),
                 loc=location_to_source_location(node.loc),
             )
 
+        return gtir.ScalarIfStmt(
+            cond=cond,
+            true_branch=gtir.BlockStmt(body=self.visit(node.main_body)),
+            false_branch=(
+                gtir.BlockStmt(body=self.visit(node.else_body)) if node.else_body else None
+            ),
+            loc=location_to_source_location(node.loc),
+        )
+
     def visit_HorizontalIf(self, node: HorizontalIf) -> gtir.FieldIfStmt:
         def make_bound_or_level(bound: AxisBound, level) -> Optional[common.AxisBound]:
             if (level == LevelMarker.START and bound.offset <= -10000) or (
                 level == LevelMarker.END and bound.offset >= 10000
             ):
                 return None
-            else:
-                return common.AxisBound(
-                    level=self.GT4PY_LEVELMARKER_TO_GTIR_LEVELMARKER[bound.level],
-                    offset=bound.offset,
-                )
+
+            return common.AxisBound(
+                level=self.GT4PY_LEVELMARKER_TO_GTIR_LEVELMARKER[bound.level],
+                offset=bound.offset,
+            )
 
         axes = {
             axis.lower(): common.HorizontalInterval(
@@ -525,8 +521,7 @@ class DefIRToGTIR(IRNodeVisitor):
         }
 
         return gtir.HorizontalRestriction(
-            mask=common.HorizontalMask(**axes),
-            body=self.visit(node.body),
+            mask=common.HorizontalMask(**axes), body=self.visit(node.body)
         )
 
     def visit_While(self, node: While) -> gtir.While:
@@ -536,7 +531,7 @@ class DefIRToGTIR(IRNodeVisitor):
             loc=location_to_source_location(node.loc),
         )
 
-    def visit_VarRef(self, node: VarRef, **kwargs):
+    def visit_VarRef(self, node: VarRef, **kwargs) -> gtir.ScalarAccess:
         return gtir.ScalarAccess(name=node.name, loc=location_to_source_location(node.loc))
 
     def visit_AxisInterval(self, node: AxisInterval) -> Tuple[gtir.AxisBound, gtir.AxisBound]:
@@ -574,7 +569,7 @@ class DefIRToGTIR(IRNodeVisitor):
         k_val = offset.get("K", 0)
         if isinstance(k_val, numbers.Integral):
             return common.CartesianOffset(i=offset.get("I", 0), j=offset.get("J", 0), k=k_val)
-        elif isinstance(k_val, Expr):
+        if isinstance(k_val, Expr):
             return gtir.VariableKOffset(k=self.visit(k_val, **kwargs))
-        else:
-            raise TypeError("Unrecognized vertical offset type")
+
+        raise TypeError("Unrecognized vertical offset type")
