@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import ast
 import copy
@@ -21,19 +15,15 @@ import numbers
 import textwrap
 import time
 import types
-import warnings
 from typing import Any, Dict, Final, List, Literal, Optional, Sequence, Set, Tuple, Type, Union
 
 import numpy as np
 
 from gt4py.cartesian import definitions as gt_definitions, gtscript, utils as gt_utils
 from gt4py.cartesian.frontend import node_util, nodes
+from gt4py.cartesian.frontend.base import Frontend, register
 from gt4py.cartesian.frontend.defir_to_gtir import DefIRToGTIR, UnrollVectorAssignments
-from gt4py.cartesian.gtc import utils as gtc_utils
-from gt4py.cartesian.utils import NOTHING, meta as gt_meta
-
-from .base import Frontend, register
-from .exceptions import (
+from gt4py.cartesian.frontend.exceptions import (
     GTScriptAssertionError,
     GTScriptDataTypeError,
     GTScriptDefinitionError,
@@ -41,9 +31,10 @@ from .exceptions import (
     GTScriptSyntaxError,
     GTScriptValueError,
 )
+from gt4py.cartesian.utils import meta as gt_meta
 
 
-PYTHON_AST_VERSION: Final = (3, 8)
+PYTHON_AST_VERSION: Final = (3, 10)
 
 
 class AssertionChecker(ast.NodeTransformer):
@@ -59,8 +50,8 @@ class AssertionChecker(ast.NodeTransformer):
         self.source = source
 
     def _process_assertion(self, expr_node: ast.Expr) -> None:
-        condition_value = gt_utils.meta.ast_eval(expr_node, self.context, default=NOTHING)
-        if condition_value is not NOTHING:
+        condition_value = gt_meta.ast_eval(expr_node, self.context, default=gt_utils.NOTHING)
+        if condition_value is not gt_utils.NOTHING:
             if not condition_value:
                 source_lines = textwrap.dedent(self.source).split("\n")
                 loc = nodes.Location.from_ast_node(expr_node)
@@ -95,7 +86,6 @@ class AxisIntervalParser(gt_meta.ASTPass):
 
     Corner cases: `ast.Ellipsis` refers to the entire interval, and
     if an `ast.Subscript` is passed, this parses its slice attribute.
-
     """
 
     @classmethod
@@ -211,15 +201,15 @@ class AxisIntervalParser(gt_meta.ASTPass):
         right = self.visit(node.right)
 
         if isinstance(node.op, ast.Add):
-            bin_op = lambda x, y: x + y  # noqa: E731
-            u_op = lambda x: x  # noqa: E731
+            bin_op = lambda x, y: x + y  # noqa: E731 [lambda-assignment]
+            u_op = lambda x: x  # noqa: E731 [lambda-assignment]
         elif isinstance(node.op, ast.Sub):
-            bin_op = lambda x, y: x - y  # noqa: E731
-            u_op = lambda x: -x  # noqa: E731
+            bin_op = lambda x, y: x - y  # noqa: E731 [lambda-assignment]
+            u_op = lambda x: -x  # noqa: E731 [lambda-assignment]
         elif isinstance(node.op, ast.Mult):
             if left.level != right.level or not isinstance(left.level, nodes.LevelMarker):
                 raise self.interval_error
-            bin_op = lambda x, y: x * y  # noqa: E731
+            bin_op = lambda x, y: x * y  # noqa: E731 [lambda-assignment]
             u_op = None
         else:
             raise GTScriptSyntaxError("Unexpected binary operator found in interval expression")
@@ -249,7 +239,7 @@ class AxisIntervalParser(gt_meta.ASTPass):
 
     def visit_UnaryOp(self, node: ast.UnaryOp) -> nodes.AxisBound:
         if isinstance(node.op, ast.USub):
-            op = lambda x: -x  # noqa: E731
+            op = lambda x: -x  # noqa: E731 [lambda-assignment]
         else:
             raise self.interval_error
 
@@ -313,7 +303,7 @@ class ValueInliner(ast.NodeTransformer):
         return node
 
 
-class ReturnReplacer(gt_utils.meta.ASTTransformPass):
+class ReturnReplacer(gt_meta.ASTTransformPass):
     @classmethod
     def apply(cls, ast_object: ast.AST, target_node: ast.AST) -> None:
         """Ensure that there is only a single return statement (can still return a tuple)."""
@@ -404,6 +394,10 @@ class CallInliner(ast.NodeTransformer):
         # Assertions are removed in the AssertionChecker later.
         return node
 
+    def visit_While(self, node: ast.While):
+        node.body = self._process_stmts(node.body)
+        return node
+
     def visit_Assign(self, node: ast.Assign):
         if (
             isinstance(node.value, ast.Call)
@@ -417,9 +411,7 @@ class CallInliner(ast.NodeTransformer):
         else:
             return self.generic_visit(node)
 
-    def visit_Call(  # noqa: C901 # Cyclomatic complexity too high
-        self, node: ast.Call, *, target_node=None
-    ):
+    def visit_Call(self, node: ast.Call, *, target_node=None):  # Cyclomatic complexity too high
         call_name = gt_meta.get_qualified_name_from_node(node.func)
 
         if call_name in self.call_stack:
@@ -461,10 +453,11 @@ class CallInliner(ast.NodeTransformer):
                 if name not in call_args:
                     assert arg_infos[name] != nodes.Empty
                     call_args[name] = ast.Constant(value=arg_infos[name])
-        except Exception:
+        except Exception as ex:
             raise GTScriptSyntaxError(
-                message="Invalid call signature", loc=nodes.Location.from_ast_node(node)
-            )
+                message=f"Invalid call signature when calling {call_name}",
+                loc=nodes.Location.from_ast_node(node),
+            ) from ex
 
         # Rename local names in subroutine to avoid conflicts with caller context names
         try:
@@ -487,7 +480,7 @@ class CallInliner(ast.NodeTransformer):
             if isinstance(value, ast.Name) and name not in assigned_symbols
         }
 
-        call_id = gt_utils.shashed_id(call_name)[:3]
+        call_id = gt_utils.shashed_id(call_name, length=3)
         call_id_suffix = f"{call_id}_{node.lineno}_{node.col_offset}"
         template_fmt = "{name}__" + call_id_suffix
 
@@ -585,12 +578,11 @@ class CallInliner(ast.NodeTransformer):
 
 class CompiledIfInliner(ast.NodeTransformer):
     @classmethod
-    def apply(cls, ast_object: ast.AST, context: Dict[str, Any], stencil_name: str):
-        cls(context, stencil_name).visit(ast_object)
+    def apply(cls, ast_object: ast.AST, context: Dict[str, Any]):
+        cls(context).visit(ast_object)
 
-    def __init__(self, context: Dict[str, Any], stencil_name: str):
+    def __init__(self, context: Dict[str, Any]):
         self.context = context
-        self.stencil_name = stencil_name
 
     def visit_If(self, node: ast.If):
         # Compile-time evaluation of "if" conditions
@@ -601,13 +593,9 @@ class CompiledIfInliner(ast.NodeTransformer):
             and node.test.func.id == "__INLINED"
             and len(node.test.args) == 1
         ):
-            warnings.warn(
-                f"stencil {self.stencil_name}, line {node.lineno}, column {node.col_offset}: compile-time if condition via __INLINED deprecated",
-                category=DeprecationWarning,
-            )
             eval_node = node.test.args[0]
-            condition_value = gt_utils.meta.ast_eval(eval_node, self.context, default=NOTHING)
-            if condition_value is not NOTHING:
+            condition_value = gt_meta.ast_eval(eval_node, self.context, default=gt_utils.NOTHING)
+            if condition_value is not gt_utils.NOTHING:
                 node = node.body if condition_value else node.orelse
             else:
                 raise GTScriptSyntaxError(
@@ -618,7 +606,7 @@ class CompiledIfInliner(ast.NodeTransformer):
 
 
 def _make_temp_decls(
-    descriptors: Dict[str, gtscript._FieldDescriptor]
+    descriptors: Dict[str, gtscript._FieldDescriptor],
 ) -> Dict[str, nodes.FieldDecl]:
     return {
         name: nodes.FieldDecl(
@@ -660,8 +648,7 @@ def _make_init_computations(
         else:
             stmts.append(
                 nodes.Assign(
-                    target=nodes.FieldRef.at_center(name, axes=decl.axes),
-                    value=init_values[name],
+                    target=nodes.FieldRef.at_center(name, axes=decl.axes), value=init_values[name]
                 )
             )
 
@@ -692,12 +679,32 @@ class ParsingContext(enum.Enum):
     COMPUTATION = 2
 
 
+_DATADIMS_INDEXER = "A"
+
+
+def _is_datadims_indexing_name(name: str):
+    return name.endswith(f".{_DATADIMS_INDEXER}")
+
+
+def _trim_indexing_symbol(name: str):
+    return name[: -1 * (len(_DATADIMS_INDEXER) + 1)]
+
+
+def _is_datadims_indexing_node(node):
+    return (
+        isinstance(node.value, ast.Attribute)
+        and node.value.attr == _DATADIMS_INDEXER
+        and isinstance(node.value.value, ast.Name)
+    )
+
+
 class IRMaker(ast.NodeVisitor):
     def __init__(
         self,
         fields: dict,
         parameters: dict,
         local_symbols: dict,
+        backend_name: str,
         *,
         domain: nodes.Domain,
         temp_decls: Optional[Dict[str, nodes.FieldDecl]] = None,
@@ -711,6 +718,7 @@ class IRMaker(ast.NodeVisitor):
             isinstance(value, (type, np.dtype)) for value in local_symbols.values()
         )
 
+        self.backend_name = backend_name
         self.fields = fields
         self.parameters = parameters
         self.local_symbols = local_symbols
@@ -742,6 +750,7 @@ class IRMaker(ast.NodeVisitor):
             "sqrt": nodes.NativeFunction.SQRT,
             "exp": nodes.NativeFunction.EXP,
             "log": nodes.NativeFunction.LOG,
+            "log10": nodes.NativeFunction.LOG10,
             "gamma": nodes.NativeFunction.GAMMA,
             "cbrt": nodes.NativeFunction.CBRT,
             "isfinite": nodes.NativeFunction.ISFINITE,
@@ -808,25 +817,9 @@ class IRMaker(ast.NodeVisitor):
         # if sorting didn't change anything it was already sorted
         return compute_blocks == compute_blocks_sorted
 
-    def _parse_region_intervals(
-        self, node: Union[ast.ExtSlice, ast.Index, ast.Tuple], loc: nodes.Location = None
-    ) -> Dict[str, nodes.AxisInterval]:
-        if isinstance(node, ast.Index):
-            # Python 3.8 wraps a Tuple in an Index for region[0, 1]
-            tuple_node = node.value
-            list_of_exprs = tuple_node.elts
-        elif isinstance(node, ast.ExtSlice) or isinstance(node, ast.Tuple):
-            # Python 3.8 returns an ExtSlice for region[0, :]
-            # Python 3.9 directly returns a Tuple for region[0, 1]
-            node_list = node.dims if isinstance(node, ast.ExtSlice) else node.elts
-            list_of_exprs = [
-                axis_node.value if isinstance(axis_node, ast.Index) else axis_node
-                for axis_node in node_list
-            ]
-        else:
-            raise GTScriptSyntaxError(
-                f"Invalid 'region' index at line {loc.line} (column {loc.column})", loc=loc
-            )
+    def _parse_region_intervals(self, node: ast.Tuple) -> Dict[str, nodes.AxisInterval]:
+        # Since Python 3.9: directly returns a Tuple for region[0, 1]
+        list_of_exprs = [axis_node for axis_node in node.elts]
         axes_names = [axis.name for axis in self.domain.parallel_axes]
         return {
             name: AxisIntervalParser.apply(axis_node, name)
@@ -846,7 +839,7 @@ class IRMaker(ast.NodeVisitor):
         ):
             raise syntax_error
 
-        return [self._parse_region_intervals(arg.slice, loc) for arg in call_args]
+        return [self._parse_region_intervals(arg.slice) for arg in call_args]
 
     def _are_intervals_nonoverlapping(self, compute_blocks: List[nodes.ComputationBlock]):
         for i, block in enumerate(compute_blocks[1:]):
@@ -958,14 +951,13 @@ class IRMaker(ast.NodeVisitor):
         self.parsing_context = ParsingContext.COMPUTATION
         stmts = []
         for stmt in node.body:
-            stmts.extend(gtc_utils.listify(self.visit(stmt)))
+            stmts.extend(gt_utils.listify(self.visit(stmt)))
         self.parsing_context = ParsingContext.CONTROL_FLOW
 
         if intervals_dicts:
             stmts = [
                 nodes.HorizontalIf(
-                    intervals=intervals_dict,
-                    body=nodes.BlockStmt(stmts=stmts, loc=loc),
+                    intervals=intervals_dict, body=nodes.BlockStmt(stmts=stmts, loc=loc)
                 )
                 for intervals_dict in intervals_dicts
             ]
@@ -992,9 +984,7 @@ class IRMaker(ast.NodeVisitor):
         elif isinstance(value, bool):
             return nodes.Cast(
                 data_type=nodes.DataType.BOOL,
-                expr=nodes.BuiltinLiteral(
-                    value=nodes.Builtin.from_value(value),
-                ),
+                expr=nodes.BuiltinLiteral(value=nodes.Builtin.from_value(value)),
                 loc=nodes.Location.from_ast_node(node),
             )
         elif isinstance(value, numbers.Number):
@@ -1038,6 +1028,10 @@ class IRMaker(ast.NodeVisitor):
             result = nodes.VarRef(name=symbol, loc=nodes.Location.from_ast_node(node))
         elif self._is_local_symbol(symbol):
             raise AssertionError("Logic error")
+        elif _is_datadims_indexing_name(symbol):
+            result = nodes.FieldRef.datadims_index(
+                name=_trim_indexing_symbol(symbol), loc=nodes.Location.from_ast_node(node)
+            )
         else:
             raise AssertionError(f"Missing '{symbol}' symbol definition")
 
@@ -1058,10 +1052,10 @@ class IRMaker(ast.NodeVisitor):
         for index_node in index_nodes:
             try:
                 value = gt_meta.ast_eval(index_node, axis_context)
-            except Exception:
+            except Exception as ex:
                 raise GTScriptSyntaxError(
                     message="Could not evaluate axis shift expression.", loc=index_node
-                )
+                ) from ex
             if not isinstance(value, (gtscript.ShiftedAxis, gtscript.Axis)):
                 raise GTScriptSyntaxError(
                     message=f"Axis shift expression evaluated to unrecognized type {type(value)}.",
@@ -1095,7 +1089,7 @@ class IRMaker(ast.NodeVisitor):
         self, node: ast.Subscript, field_axes: Optional[Set[Literal["I", "J", "K"]]] = None
     ) -> Optional[List[int]]:
         tuple_or_expr = node.slice.value if isinstance(node.slice, ast.Index) else node.slice
-        index_nodes = gtc_utils.listify(
+        index_nodes = gt_utils.listify(
             tuple_or_expr.elts if isinstance(tuple_or_expr, ast.Tuple) else tuple_or_expr
         )
 
@@ -1146,16 +1140,22 @@ class IRMaker(ast.NodeVisitor):
                 field_axes = self.fields[result.name].axes
                 if index is not None:
                     if len(field_axes) != len(index):
+                        ro_field_message = ""
+                        if len(field_axes) == 0:
+                            ro_field_message = f"Did you mean .A{index}?"
                         raise GTScriptSyntaxError(
                             f"Incorrect offset specification detected. Found {index}, "
-                            f"but the field has dimensions ({', '.join(field_axes)})"
+                            f"but the field has dimensions ({', '.join(field_axes)}). "
+                            f"{ro_field_message}"
                         )
                     result.offset = {axis: value for axis, value in zip(field_axes, index)}
-            elif isinstance(node.value, ast.Subscript):
+            elif isinstance(node.value, ast.Subscript) or _is_datadims_indexing_node(node):
                 result.data_index = [
-                    nodes.ScalarLiteral(value=value, data_type=nodes.DataType.INT32)
-                    if isinstance(value, numbers.Integral)
-                    else value
+                    (
+                        nodes.ScalarLiteral(value=value, data_type=nodes.DataType.INT32)
+                        if isinstance(value, numbers.Integral)
+                        else value
+                    )
                     for value in index
                 ]
                 if len(result.data_index) != len(self.fields[result.name].data_dims):
@@ -1299,13 +1299,13 @@ class IRMaker(ast.NodeVisitor):
 
         main_stmts = []
         for stmt in node.body:
-            main_stmts.extend(gtc_utils.listify(self.visit(stmt)))
+            main_stmts.extend(gt_utils.listify(self.visit(stmt)))
         assert all(isinstance(item, nodes.Statement) for item in main_stmts)
 
         else_stmts = []
         if node.orelse:
             for stmt in node.orelse:
-                else_stmts.extend(gtc_utils.listify(self.visit(stmt)))
+                else_stmts.extend(gt_utils.listify(self.visit(stmt)))
             assert all(isinstance(item, nodes.Statement) for item in else_stmts)
 
         result = []
@@ -1315,14 +1315,26 @@ class IRMaker(ast.NodeVisitor):
             self.decls_stack[-2].extend(self.decls_stack[-1])
             self.decls_stack.pop()
 
+        try:
+            condition = self.visit(node.test)
+        except KeyError as e:
+            raise GTScriptSyntaxError(
+                message=(
+                    "Using function calls in the condition of an if is not allowed,"
+                    " the function needs to be assigned to a variable outside the condition."
+                ),
+                loc=nodes.Location.from_ast_node(node),
+            ) from e
         result.append(
             nodes.If(
-                condition=self.visit(node.test),
+                condition=condition,
                 loc=nodes.Location.from_ast_node(node),
                 main_body=nodes.BlockStmt(stmts=main_stmts, loc=nodes.Location.from_ast_node(node)),
-                else_body=nodes.BlockStmt(stmts=else_stmts, loc=nodes.Location.from_ast_node(node))
-                if else_stmts
-                else None,
+                else_body=(
+                    nodes.BlockStmt(stmts=else_stmts, loc=nodes.Location.from_ast_node(node))
+                    if else_stmts
+                    else None
+                ),
             )
         )
 
@@ -1412,11 +1424,18 @@ class IRMaker(ast.NodeVisitor):
         for t in node.targets[0].elts if isinstance(node.targets[0], ast.Tuple) else node.targets:
             name, spatial_offset, data_index = self._parse_assign_target(t)
             if spatial_offset:
-                if any(offset != 0 for offset in spatial_offset):
+                if spatial_offset[0] != 0 or spatial_offset[1] != 0:
                     raise GTScriptSyntaxError(
-                        message="Assignment to non-zero offsets is not supported.",
+                        message="Assignment to non-zero offsets is not supported in IJ.",
                         loc=nodes.Location.from_ast_node(t),
                     )
+                # Case of K-offset
+                if len(spatial_offset) == 3 and spatial_offset[2] != 0:
+                    if self.iteration_order == nodes.IterationOrder.PARALLEL:
+                        raise GTScriptSyntaxError(
+                            message="Assignment to non-zero offsets in K is not available in PARALLEL. Choose FORWARD or BACKWARD.",
+                            loc=nodes.Location.from_ast_node(t),
+                        )
 
             if not self._is_known(name):
                 if name in self.temp_decls:
@@ -1456,7 +1475,7 @@ class IRMaker(ast.NodeVisitor):
 
             target.append(self.visit(t))
 
-        value = gtc_utils.listify(self.visit(node.value))
+        value = gt_utils.listify(self.visit(node.value))
 
         assert len(target) == len(value)
         for left, right in zip(target, value):
@@ -1490,9 +1509,7 @@ class IRMaker(ast.NodeVisitor):
 
             self.parsing_horizontal_region = True
             intervals_dicts = self._visit_with_horizontal(node.items[0], loc)
-            all_stmts = gt_utils.flatten(
-                [gtc_utils.listify(self.visit(stmt)) for stmt in node.body]
-            )
+            all_stmts = gt_utils.flatten([gt_utils.listify(self.visit(stmt)) for stmt in node.body])
             self.parsing_horizontal_region = False
             stmts = list(filter(lambda stmt: isinstance(stmt, nodes.Decl), all_stmts))
             body_block = nodes.BlockStmt(
@@ -1558,7 +1575,7 @@ class IRMaker(ast.NodeVisitor):
 
                 return compute_blocks
             elif self.parsing_context == ParsingContext.CONTROL_FLOW:
-                return gtc_utils.listify(self._visit_computation_node(node))
+                return gt_utils.listify(self._visit_computation_node(node))
             else:
                 # Mixing nested `with` blocks with stmts not allowed
                 raise syntax_error
@@ -1600,6 +1617,11 @@ class CollectLocalSymbolsAstVisitor(ast.NodeVisitor):
                 elif isinstance(t, ast.Subscript):
                     if isinstance(t.value, ast.Name):
                         name_node = t.value
+                    elif _is_datadims_indexing_node(t):
+                        raise GTScriptSyntaxError(
+                            message="writing to an GlobalTable ('A' global indexation) is forbidden",
+                            loc=nodes.Location.from_ast_node(node),
+                        )
                     elif isinstance(t.value, ast.Subscript) and isinstance(t.value.value, ast.Name):
                         name_node = t.value.value
                     else:
@@ -1610,7 +1632,6 @@ class CollectLocalSymbolsAstVisitor(ast.NodeVisitor):
 
 
 class GTScriptParser(ast.NodeVisitor):
-
     CONST_VALUE_TYPES = (
         *gtscript._VALID_DATA_TYPES,
         types.FunctionType,
@@ -1623,7 +1644,7 @@ class GTScriptParser(ast.NodeVisitor):
         assert isinstance(definition, types.FunctionType)
         self.definition = definition
         self.filename = inspect.getfile(definition)
-        self.source, decorators_source = gt_meta.split_def_decorators(self.definition)
+        self.source, _decorators_source = gt_meta.split_def_decorators(self.definition)
         self.ast_root = ast.parse(self.source, feature_version=PYTHON_AST_VERSION)
         self.options = options
         self.build_info = options.build_info
@@ -1634,7 +1655,7 @@ class GTScriptParser(ast.NodeVisitor):
         self.block = None
         self.dtypes = dtypes
 
-    def __str__(self):
+    def __str__(self) -> str:
         result = "<GT4Py.GTScriptParser> {\n"
         result += "\n".join("\t{}: {}".format(name, getattr(self, name)) for name in vars(self))
         result += "\n}"
@@ -1726,7 +1747,7 @@ class GTScriptParser(ast.NodeVisitor):
             assert isinstance(ann_assign.target, ast.Name)
             name = ann_assign.target.id
 
-            source = gt_meta.ast_unparse(ann_assign.annotation)
+            source = ast.unparse(ann_assign.annotation)
             descriptor = eval(source, ann_assign_context)
             temp_annotations[name] = descriptor
             if descriptor.axes != gtscript.IJK:
@@ -1973,7 +1994,7 @@ class GTScriptParser(ast.NodeVisitor):
 
         return api_signature, fields_decls, parameter_decls
 
-    def run(self):
+    def run(self, backend_name: str):
         assert (
             isinstance(self.ast_root, ast.Module)
             and "body" in self.ast_root._fields
@@ -2014,7 +2035,7 @@ class GTScriptParser(ast.NodeVisitor):
         CallInliner.apply(main_func_node, context=local_context)
 
         # Evaluate and inline compile-time conditionals
-        CompiledIfInliner.apply(main_func_node, context=local_context, stencil_name=self.main_name)
+        CompiledIfInliner.apply(main_func_node, context=local_context)
 
         AssertionChecker.apply(main_func_node, context=local_context, source=self.source)
 
@@ -2031,6 +2052,7 @@ class GTScriptParser(ast.NodeVisitor):
             fields=fields_decls,
             parameters=parameter_decls,
             local_symbols={},  # Not used
+            backend_name=backend_name,
             domain=domain,
             temp_decls=temp_decls,
             dtypes=self.dtypes,
@@ -2086,14 +2108,14 @@ class GTScriptFrontend(Frontend):
         return GTScriptParser.annotate_definition(definition, externals)
 
     @classmethod
-    def generate(cls, definition, externals, dtypes, options):
+    def generate(cls, definition, externals, dtypes, options, backend_name):
         if options.build_info is not None:
             start_time = time.perf_counter()
 
         if not hasattr(definition, "_gtscript_"):
             cls.prepare_stencil_definition(definition, externals)
         translator = GTScriptParser(definition, externals=externals, dtypes=dtypes, options=options)
-        definition_ir = translator.run()
+        definition_ir = translator.run(backend_name)
 
         # GTIR only supports LatLonGrids
         if definition_ir.domain != nodes.Domain.LatLonGrid():
