@@ -1,30 +1,20 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 import collections
-import copy
 import operator
 from typing import Generator, Optional, Type
 
 import boltons.typeutils
 
-import gt4py.cartesian.gtc.utils as gtc_utils
 from gt4py import eve
 from gt4py.cartesian import utils as gt_utils
-from gt4py.cartesian.gtc import common
-
-from .nodes import Location, Node
+from gt4py.cartesian.frontend.nodes import Location, Node
 
 
 #: Marker value used to avoid confusion with `None`
@@ -72,7 +62,7 @@ class IRNodeVisitor:
         else:
             pass
 
-        for key, value in items:
+        for _, value in items:
             self._visit(value, **kwargs)
 
 
@@ -125,14 +115,14 @@ def iter_nodes_of_type(root_node: Node, node_type: Type) -> Generator[Node, None
     """Yield an iterator over the nodes of node_type inside root_node in DFS order."""
 
     def recurse(node: Node) -> Generator[Node, None, None]:
-        for key, value in iter_attributes(node):
+        for _, value in iter_attributes(node):
             if isinstance(node, collections.abc.Iterable):
                 if isinstance(node, collections.abc.Mapping):
                     children = node.values()
                 else:
                     children = node
             else:
-                children = gtc_utils.listify(value)
+                children = gt_utils.listify(value)
 
             for value in children:
                 if isinstance(value, Node):
@@ -147,4 +137,4 @@ def iter_nodes_of_type(root_node: Node, node_type: Type) -> Generator[Node, None
 def location_to_source_location(loc: Optional[Location]) -> Optional[eve.SourceLocation]:
     if loc is None or loc.line <= 0 or loc.column <= 0:
         return None
-    return eve.SourceLocation(line=loc.line, column=loc.column, source=loc.scope)
+    return eve.SourceLocation(line=loc.line, column=loc.column, filename=loc.scope)
