@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """
 Implementation of the intermediate representations used in GT4Py.
@@ -46,7 +40,7 @@ BinaryOperator enumeration (:class:`BinaryOperator`)
 NativeFunction enumeration (:class:`NativeFunction`)
     Native function identifier
     [`ABS`, `MAX`, `MIN, `MOD`, `SIN`, `COS`, `TAN`, `ARCSIN`, `ARCCOS`, `ARCTAN`,
-    `SQRT`, `EXP`, `LOG`, `ISFINITE`, `ISINF`, `ISNAN`, `FLOOR`, `CEIL`, `TRUNC`]
+    `SQRT`, `EXP`, `LOG`, `LOG10`, `ISFINITE`, `ISINF`, `ISNAN`, `FLOOR`, `CEIL`, `TRUNC`]
 
 LevelMarker enumeration (:class:`LevelMarker`)
     Special axis levels
@@ -136,28 +130,25 @@ storing a reference to the piece of source code which originated the node.
                       parameters: List[VarDecl],
                       computations: List[ComputationBlock],
                       [externals: Dict[str, Any], sources: Dict[str, str]])
-
 """
 
+from __future__ import annotations
+
 import enum
 import operator
 import sys
-from typing import Generator, List, Optional, Sequence, Type
+from typing import List, Optional, Sequence
 
 import numpy as np
 
-from gt4py.cartesian.definitions import AccessKind, CartesianSpace
-from gt4py.cartesian.gtc.definitions import Extent, Index
+from gt4py.cartesian.definitions import CartesianSpace
 from gt4py.cartesian.utils.attrib import (
     Any as Any,
     Dict as DictOf,
     List as ListOf,
-    Optional as OptionalOf,
-    Tuple as TupleOf,
     Union as UnionOf,
     attribkwclass as attribclass,
     attribute,
-    attributes_of,
 )
 
 
@@ -194,7 +185,7 @@ class LevelMarker(enum.Enum):
     START = 0
     END = -1
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
 
@@ -245,17 +236,15 @@ class Builtin(enum.Enum):
     TRUE = 1
 
     @classmethod
-    def from_value(cls, value):
+    def from_value(cls, value: bool | None) -> Builtin:
         if value is None:
-            result = cls.NONE
-        elif value is True:
-            result = cls.TRUE
-        elif value is False:
-            result = cls.FALSE
+            return cls.NONE
+        if value is True:
+            return cls.TRUE
+        if value is False:
+            return cls.FALSE
 
-        return result
-
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
 
@@ -272,7 +261,7 @@ class DataType(enum.Enum):
     FLOAT32 = 104
     FLOAT64 = 108
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
     @property
@@ -326,15 +315,6 @@ class ScalarLiteral(Literal):
     loc = attribute(of=Location, optional=True)
 
 
-# @attribclass
-# class TupleLiteral(Node):
-#     items = attribute(of=TupleOf[Expr])
-#
-#     @property
-#     def length(self):
-#         return len(self.items)
-
-
 @attribclass
 class BuiltinLiteral(Literal):
     value = attribute(of=Builtin)
@@ -367,6 +347,10 @@ class FieldRef(Ref):
             name=name, offset={axis: 0 for axis in axes}, data_index=data_index or [], loc=loc
         )
 
+    @classmethod
+    def datadims_index(cls, name: str, loc=None):
+        return cls(name=name, offset={}, data_index=[], loc=loc)
+
 
 @attribclass
 class Cast(Expr):
@@ -413,6 +397,7 @@ class NativeFunction(enum.Enum):
     SQRT = enum.auto()
     EXP = enum.auto()
     LOG = enum.auto()
+    LOG10 = enum.auto()
     GAMMA = enum.auto()
     CBRT = enum.auto()
 
@@ -448,6 +433,7 @@ NativeFunction.IR_OP_TO_NUM_ARGS = {
     NativeFunction.SQRT: 1,
     NativeFunction.EXP: 1,
     NativeFunction.LOG: 1,
+    NativeFunction.LOG10: 1,
     NativeFunction.GAMMA: 1,
     NativeFunction.CBRT: 1,
     NativeFunction.ISFINITE: 1,
@@ -595,12 +581,6 @@ class Statement(Node):
     pass
 
 
-# @attribclass
-# class ExprStmt(Statement):
-#     expr = attribute(of=Expr)
-#     loc = attribute(of=Location, optional=True)
-
-
 class Decl(Statement):
     pass
 
@@ -673,12 +653,12 @@ class IterationOrder(enum.Enum):
     def symbol(self):
         if self == self.BACKWARD:
             return "<-"
-        elif self == self.PARALLEL:
+        if self == self.PARALLEL:
             return "||"
-        elif self == self.FORWARD:
+        if self == self.FORWARD:
             return "->"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
     def __lshift__(self, steps: int):
@@ -702,19 +682,17 @@ class AxisInterval(Node):
     loc = attribute(of=Location, optional=True)
 
     @classmethod
-    def full_interval(cls, order=IterationOrder.PARALLEL):
+    def full_interval(cls, order=IterationOrder.PARALLEL) -> AxisInterval:
         if order != IterationOrder.BACKWARD:
-            interval = cls(
+            return cls(
                 start=AxisBound(level=LevelMarker.START, offset=0),
                 end=AxisBound(level=LevelMarker.END, offset=0),
             )
-        else:
-            interval = cls(
-                start=AxisBound(level=LevelMarker.END, offset=-1),
-                end=AxisBound(level=LevelMarker.START, offset=-1),
-            )
 
-        return interval
+        return cls(
+            start=AxisBound(level=LevelMarker.END, offset=-1),
+            end=AxisBound(level=LevelMarker.START, offset=-1),
+        )
 
     @property
     def is_single_index(self) -> bool:
@@ -723,10 +701,7 @@ class AxisInterval(Node):
 
         return self.start.level == self.end.level and self.start.offset == self.end.offset - 1
 
-    def disjoint_from(self, other: "AxisInterval") -> bool:
-        # This made-up constant must be larger than any LevelMarker.offset used
-        DOMAIN_SIZE: int = 1000
-
+    def disjoint_from(self, other: AxisInterval) -> bool:
         def get_offset(bound: AxisBound) -> int:
             return (
                 0 + bound.offset if bound.level == LevelMarker.START else sys.maxsize + bound.offset
