@@ -1,29 +1,37 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from __future__ import annotations
 
 import enum
 import functools
 import typing
-from typing import Any, ClassVar, Dict, Generic, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    Any,
+    ClassVar,
+    Dict,
+    Final,
+    Generic,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
-import scipy.special
 
 from gt4py import eve
-from gt4py.cartesian.gtc.utils import dimension_flags_to_names, flatten_list
+from gt4py.cartesian import utils
+from gt4py.cartesian.gtc import ufuncs
+from gt4py.cartesian.gtc.utils import dimension_flags_to_names
 from gt4py.eve import datamodels
 
 
@@ -31,14 +39,14 @@ class GTCPreconditionError(eve.exceptions.EveError, RuntimeError):
     message_template = "GTC pass precondition error: [{info}]"
 
     def __init__(self, *, expected: str, **kwargs: Any) -> None:
-        super().__init__(expected=expected, **kwargs)  # type: ignore
+        super().__init__(expected=expected, **kwargs)
 
 
 class GTCPostconditionError(eve.exceptions.EveError, RuntimeError):
     message_template = "GTC pass postcondition error: [{info}]"
 
     def __init__(self, *, expected: str, **kwargs: Any) -> None:
-        super().__init__(expected=expected, **kwargs)  # type: ignore
+        super().__init__(expected=expected, **kwargs)
 
 
 class AssignmentKind(eve.StrEnum):
@@ -53,7 +61,7 @@ class AssignmentKind(eve.StrEnum):
 
 @enum.unique
 class UnaryOperator(eve.StrEnum):
-    """Unary operator indentifier."""
+    """Unary operator identifier."""
 
     POS = "+"
     NEG = "-"
@@ -111,7 +119,7 @@ class DataType(eve.IntEnum):
         return self == self.BOOL
 
     def isinteger(self):
-        return self in (self.INT8, self.INT32, self.INT64)
+        return self in (self.INT8, self.INT16, self.INT32, self.INT64)
 
     def isfloat(self):
         return self in (self.FLOAT32, self.FLOAT64)
@@ -160,6 +168,7 @@ class NativeFunction(eve.StrEnum):
     POW = "pow"
     EXP = "exp"
     LOG = "log"
+    LOG10 = "log10"
     GAMMA = "gamma"
     CBRT = "cbrt"
 
@@ -170,7 +179,7 @@ class NativeFunction(eve.StrEnum):
     CEIL = "ceil"
     TRUNC = "trunc"
 
-    IR_OP_TO_NUM_ARGS: ClassVar[Dict["NativeFunction", int]]
+    IR_OP_TO_NUM_ARGS: ClassVar[Dict[NativeFunction, int]]
 
     @property
     def arity(self) -> int:
@@ -200,6 +209,7 @@ NativeFunction.IR_OP_TO_NUM_ARGS = {
         NativeFunction.POW: 2,
         NativeFunction.EXP: 1,
         NativeFunction.LOG: 1,
+        NativeFunction.LOG10: 1,
         NativeFunction.GAMMA: 1,
         NativeFunction.CBRT: 1,
         NativeFunction.ISFINITE: 1,
@@ -220,8 +230,8 @@ class LevelMarker(eve.StrEnum):
 
 @enum.unique
 class ExprKind(eve.IntEnum):
-    SCALAR: "ExprKind" = typing.cast("ExprKind", enum.auto())
-    FIELD: "ExprKind" = typing.cast("ExprKind", enum.auto())
+    SCALAR = typing.cast("ExprKind", enum.auto())
+    FIELD = typing.cast("ExprKind", enum.auto())
 
 
 class LocNode(eve.Node):
@@ -258,7 +268,7 @@ def verify_and_get_common_dtype(
 ) -> Optional[DataType]:
     assert len(exprs) > 0
     if all(e.dtype is not DataType.AUTO for e in exprs):
-        dtypes: List[DataType] = [e.dtype for e in exprs]  # type: ignore # guaranteed to be not None
+        dtypes: List[DataType] = [e.dtype for e in exprs]  # guaranteed to be not None
         dtype = dtypes[0]
         if strict:
             if all(dt == dtype for dt in dtypes):
@@ -302,7 +312,7 @@ class CartesianOffset(eve.Node):
     k: int
 
     @classmethod
-    def zero(cls) -> "CartesianOffset":
+    def zero(cls) -> CartesianOffset:
         return cls(i=0, j=0, k=0)
 
     def to_dict(self) -> Dict[str, int]:
@@ -334,7 +344,7 @@ class FieldAccess(eve.GenericNode, Generic[ExprT, VariableKOffsetT]):
     kind: ExprKind = ExprKind.FIELD
 
     @classmethod
-    def centered(cls, *, name: str, loc: Optional[eve.SourceLocation] = None) -> "FieldAccess":
+    def centered(cls, *, name: str, loc: Optional[eve.SourceLocation] = None) -> FieldAccess:
         return cls(name=name, loc=loc, offset=CartesianOffset.zero())
 
     @datamodels.validator("data_index")
@@ -391,10 +401,7 @@ def _make_root_validator(impl: datamodels.RootValidator) -> datamodels.RootValid
 
 
 def assign_stmt_dtype_validation(*, strict: bool) -> datamodels.RootValidator:
-    def _impl(
-        cls: Type[datamodels.DataModel],
-        instance: datamodels.DataModel,
-    ) -> None:
+    def _impl(cls: Type[datamodels.DataModel], instance: datamodels.DataModel) -> None:
         assert isinstance(instance, AssignStmt)
         verify_and_get_common_dtype(cls, [instance.left, instance.right], strict=strict)
 
@@ -557,7 +564,7 @@ def native_func_call_dtype_propagation(*, strict: bool = True) -> datamodels.Roo
 def validate_dtype_is_set() -> datamodels.RootValidator:
     def _impl(cls: Type[ExprT], instance: ExprT) -> None:
         dtype_nodes: List[ExprT] = []
-        for v in flatten_list(datamodels.astuple(instance)):
+        for v in utils.flatten(datamodels.astuple(instance)):
             if isinstance(v, eve.Node):
                 dtype_nodes.extend(v.walk_values().if_hasattr("dtype"))
 
@@ -574,7 +581,7 @@ def validate_dtype_is_set() -> datamodels.RootValidator:
 
 class _LvalueDimsValidator(eve.VisitorWithSymbolTableTrait):
     def __init__(self, vertical_loop_type: Type[eve.Node], decl_type: Type[eve.Node]) -> None:
-        if not vertical_loop_type.__annotations__.get("loop_order") is LoopOrder:
+        if vertical_loop_type.__annotations__.get("loop_order") is not LoopOrder:
             raise ValueError(
                 f"Vertical loop type {vertical_loop_type} has no `loop_order` attribute"
             )
@@ -682,24 +689,24 @@ class AxisBound(eve.Node):
 
     def __lt__(self, other: AxisBound) -> bool:
         if not isinstance(other, AxisBound):
-            return NotImplemented
+            raise TypeError
         return (self.level == LevelMarker.START and other.level == LevelMarker.END) or (
             self.level == other.level and self.offset < other.offset
         )
 
     def __le__(self, other: AxisBound) -> bool:
         if not isinstance(other, AxisBound):
-            return NotImplemented
+            raise TypeError
         return self < other or self == other
 
     def __gt__(self, other: AxisBound) -> bool:
         if not isinstance(other, AxisBound):
-            return NotImplemented
+            raise TypeError
         return not self < other and not self == other
 
     def __ge__(self, other: AxisBound) -> bool:
         if not isinstance(other, AxisBound):
-            return NotImplemented
+            raise TypeError
         return not self < other
 
 
@@ -708,14 +715,13 @@ class HorizontalInterval(eve.Node):
 
     This is separate from `gtir.Interval` because the endpoints may
     be outside the compute domain.
-
     """
 
     start: Optional[AxisBound]
     end: Optional[AxisBound]
 
     @classmethod
-    def compute_domain(cls, start_offset: int = 0, end_offset: int = 0) -> "HorizontalInterval":
+    def compute_domain(cls, start_offset: int = 0, end_offset: int = 0) -> HorizontalInterval:
         return cls(start=AxisBound.start(start_offset), end=AxisBound.end(end_offset))
 
     @classmethod
@@ -725,7 +731,7 @@ class HorizontalInterval(eve.Node):
     @classmethod
     def at_endpt(
         cls, level: LevelMarker, start_offset: int, end_offset: Optional[int] = None
-    ) -> "HorizontalInterval":
+    ) -> HorizontalInterval:
         if end_offset is None:
             end_offset = start_offset + 1
         return cls(
@@ -791,7 +797,6 @@ class HorizontalRestriction(eve.GenericNode, Generic[StmtT]):
 
 
 def data_type_to_typestr(dtype: DataType) -> str:
-
     table = {
         DataType.BOOL: "bool",
         DataType.INT8: "int8",
@@ -809,88 +814,101 @@ def data_type_to_typestr(dtype: DataType) -> str:
     return np.dtype(table[dtype]).str
 
 
-@functools.lru_cache(maxsize=None, typed=True)  # typed since uniqueness is only guaranteed per enum
+# This is a mapping of mappings instead of a flat mapping because
+# different operators use the same key: UnaryOperator.POS == BinaryOperator.ADD
+OP_TO_UFUNC_NAME: Final[
+    Mapping[
+        Union[
+            Type[UnaryOperator],
+            Type[ArithmeticOperator],
+            Type[ComparisonOperator],
+            Type[LogicalOperator],
+            Type[NativeFunction],
+        ],
+        Mapping[
+            Union[
+                UnaryOperator,
+                ArithmeticOperator,
+                ComparisonOperator,
+                LogicalOperator,
+                NativeFunction,
+            ],
+            str,
+        ],
+    ]
+] = {
+    UnaryOperator: {
+        UnaryOperator.POS: "positive",
+        UnaryOperator.NEG: "negative",
+        UnaryOperator.NOT: "logical_not",
+    },
+    ArithmeticOperator: {
+        ArithmeticOperator.ADD: "add",
+        ArithmeticOperator.SUB: "subtract",
+        ArithmeticOperator.MUL: "multiply",
+        ArithmeticOperator.DIV: "true_divide",
+    },
+    ComparisonOperator: {
+        ComparisonOperator.GT: "greater",
+        ComparisonOperator.LT: "less",
+        ComparisonOperator.GE: "greater_equal",
+        ComparisonOperator.LE: "less_equal",
+        ComparisonOperator.EQ: "equal",
+        ComparisonOperator.NE: "not_equal",
+    },
+    LogicalOperator: {LogicalOperator.AND: "logical_and", LogicalOperator.OR: "logical_or"},
+    NativeFunction: {
+        NativeFunction.ABS: "abs",
+        NativeFunction.MIN: "minimum",
+        NativeFunction.MAX: "maximum",
+        NativeFunction.MOD: "remainder",
+        NativeFunction.SIN: "sin",
+        NativeFunction.COS: "cos",
+        NativeFunction.TAN: "tan",
+        NativeFunction.ARCSIN: "arcsin",
+        NativeFunction.ARCCOS: "arccos",
+        NativeFunction.ARCTAN: "arctan",
+        NativeFunction.SINH: "sinh",
+        NativeFunction.COSH: "cosh",
+        NativeFunction.TANH: "tanh",
+        NativeFunction.ARCSINH: "arcsinh",
+        NativeFunction.ARCCOSH: "arccosh",
+        NativeFunction.ARCTANH: "arctanh",
+        NativeFunction.SQRT: "sqrt",
+        NativeFunction.POW: "power",
+        NativeFunction.EXP: "exp",
+        NativeFunction.LOG: "log",
+        NativeFunction.LOG10: "log10",
+        NativeFunction.GAMMA: "gamma",
+        NativeFunction.CBRT: "cbrt",
+        NativeFunction.ISFINITE: "isfinite",
+        NativeFunction.ISINF: "isinf",
+        NativeFunction.ISNAN: "isnan",
+        NativeFunction.FLOOR: "floor",
+        NativeFunction.CEIL: "ceil",
+        NativeFunction.TRUNC: "trunc",
+    },
+}
+
+
 def op_to_ufunc(
     op: Union[
         UnaryOperator, ArithmeticOperator, ComparisonOperator, LogicalOperator, NativeFunction
-    ]
+    ],
 ) -> np.ufunc:
-    table: Dict[
-        Union[
-            UnaryOperator, ArithmeticOperator, ComparisonOperator, LogicalOperator, NativeFunction
-        ],
-        np.ufunc,
-    ]
-    # Can't put all in single table since UnaryOperator.POS == BinaryOperator.ADD
-    if isinstance(op, UnaryOperator):
-        table = {
-            UnaryOperator.POS: np.positive,
-            UnaryOperator.NEG: np.negative,
-            UnaryOperator.NOT: np.logical_not,
-        }
-    elif isinstance(op, ArithmeticOperator):
-        table = {
-            ArithmeticOperator.ADD: np.add,
-            ArithmeticOperator.SUB: np.subtract,
-            ArithmeticOperator.MUL: np.multiply,
-            ArithmeticOperator.DIV: np.true_divide,
-        }
-    elif isinstance(op, ComparisonOperator):
-        table = {
-            ComparisonOperator.GT: np.greater,
-            ComparisonOperator.LT: np.less,
-            ComparisonOperator.GE: np.greater_equal,
-            ComparisonOperator.LE: np.less_equal,
-            ComparisonOperator.EQ: np.equal,
-            ComparisonOperator.NE: np.not_equal,
-        }
-    elif isinstance(op, LogicalOperator):
-        table = {
-            LogicalOperator.AND: np.logical_and,
-            LogicalOperator.OR: np.logical_or,
-        }
-    elif isinstance(op, NativeFunction):
-        table = {
-            NativeFunction.ABS: np.abs,
-            NativeFunction.MIN: np.minimum,
-            NativeFunction.MAX: np.maximum,
-            NativeFunction.MOD: np.remainder,
-            NativeFunction.SIN: np.sin,
-            NativeFunction.COS: np.cos,
-            NativeFunction.TAN: np.tan,
-            NativeFunction.ARCSIN: np.arcsin,
-            NativeFunction.ARCCOS: np.arccos,
-            NativeFunction.ARCTAN: np.arctan,
-            NativeFunction.SINH: np.sinh,
-            NativeFunction.COSH: np.cosh,
-            NativeFunction.TANH: np.tanh,
-            NativeFunction.ARCSINH: np.arcsinh,
-            NativeFunction.ARCCOSH: np.arccosh,
-            NativeFunction.ARCTANH: np.arctanh,
-            NativeFunction.SQRT: np.sqrt,
-            NativeFunction.POW: np.power,
-            NativeFunction.EXP: np.exp,
-            NativeFunction.LOG: np.log,
-            NativeFunction.GAMMA: scipy.special.gamma,
-            NativeFunction.CBRT: np.cbrt,
-            NativeFunction.ISFINITE: np.isfinite,
-            NativeFunction.ISINF: np.isinf,
-            NativeFunction.ISNAN: np.isnan,
-            NativeFunction.FLOOR: np.floor,
-            NativeFunction.CEIL: np.ceil,
-            NativeFunction.TRUNC: np.trunc,
-        }
-    else:
+    if not isinstance(
+        op, (UnaryOperator, ArithmeticOperator, ComparisonOperator, LogicalOperator, NativeFunction)
+    ):
         raise TypeError(
             "Can only convert instances of GTC operators and supported native functions to typestr."
         )
-    return table[op]
+    return getattr(ufuncs, OP_TO_UFUNC_NAME[type(op)][op])
 
 
 @functools.lru_cache(maxsize=None)
 def typestr_to_data_type(typestr: str) -> DataType:
     if not isinstance(typestr, str) or len(typestr) < 3 or not typestr[2:].isnumeric():
-        return DataType.INVALID  # type: ignore
+        return DataType.INVALID
     table = {
         ("b", 1): DataType.BOOL,
         ("i", 1): DataType.INT8,
@@ -901,4 +919,4 @@ def typestr_to_data_type(typestr: str) -> DataType:
         ("f", 8): DataType.FLOAT64,
     }
     key = (typestr[1], int(typestr[2:]))
-    return table.get(key, DataType.INVALID)  # type: ignore
+    return table.get(key, DataType.INVALID)
