@@ -1,18 +1,12 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
-from typing import Any, Collection, Dict, List, Set, Union
+from typing import Any, Collection, Dict, Final, List, Set, Union
 
 import numpy as np
 
@@ -31,7 +25,6 @@ from gt4py.eve.concepts import LeafNode
 
 
 class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
-
     LocalScalar = as_fmt("{dtype} {name};")
 
     FieldDecl = as_fmt("{name}")
@@ -77,7 +70,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
         decl = symtable[node.name]
         if isinstance(decl, cuir.Temporary) and decl.data_dims:
             data_index_str = "+".join(
-                f"{index}*{int(np.prod(decl.data_dims[i + 1:], initial=1))}"
+                f"{index}*{int(np.prod(decl.data_dims[i + 1 :], initial=1))}"
                 for i, index in enumerate(data_index)
             )
             return f"{name}({offset})[{data_index_str}]"
@@ -116,7 +109,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
 
     BinaryOp = as_fmt("({left} {op} {right})")
 
-    UNARY_OPERATOR_TO_CODE = {
+    UNARY_OPERATOR_TO_CODE: Final[dict[UnaryOperator, str]] = {
         UnaryOperator.NOT: "!",
         UnaryOperator.NEG: "-",
         UnaryOperator.POS: "+",
@@ -128,7 +121,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
 
     Cast = as_fmt("static_cast<{dtype}>({expr})")
 
-    BUILTIN_LITERAL_TO_CODE = {
+    BUILTIN_LITERAL_TO_CODE: Final[dict[BuiltInLiteral, str]] = {
         BuiltInLiteral.TRUE: "true",
         BuiltInLiteral.FALSE: "false",
     }
@@ -149,7 +142,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
             dtype = self.visit(node.dtype, **kwargs)
             return f"static_cast<{dtype}>({value})"
 
-    NATIVE_FUNCTION_TO_CODE = {
+    NATIVE_FUNCTION_TO_CODE: Final[dict[NativeFunction, str]] = {
         NativeFunction.ABS: "std::abs",
         NativeFunction.MIN: "std::min",
         NativeFunction.MAX: "std::max",
@@ -170,6 +163,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
         NativeFunction.POW: "std::pow",
         NativeFunction.EXP: "std::exp",
         NativeFunction.LOG: "std::log",
+        NativeFunction.LOG10: "std::log10",
         NativeFunction.GAMMA: "std::tgamma",
         NativeFunction.CBRT: "std::cbrt",
         NativeFunction.ISFINITE: "std::isfinite",
@@ -190,7 +184,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
 
     NativeFuncCall = as_mako("${func}(${','.join(args)})")
 
-    DATA_TYPE_TO_CODE = {
+    DATA_TYPE_TO_CODE: Final[dict[DataType, str]] = {
         DataType.BOOL: "bool",
         DataType.INT8: "std::int8_t",
         DataType.INT16: "std::int16_t",
@@ -303,7 +297,6 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
     def visit_VerticalLoop(
         self, node: cuir.VerticalLoop, *, symtable: Dict[str, Any], **kwargs: Any
     ) -> Union[str, Collection[str]]:
-
         fields = {
             name: data_dims
             for name, data_dims in node.walk_values()
@@ -602,7 +595,7 @@ class CUIRCodegen(codegen.TemplatedGenerator, eve.VisitorWithSymbolTableTrait):
     @classmethod
     def apply(cls, root: LeafNode, **kwargs: Any) -> str:
         if not isinstance(root, cuir.Program):
-            raise ValueError("apply() requires gtcpp.Progam root node")
+            raise ValueError("apply() requires gtcpp.Program root node")
         generated_code = super().apply(root, **kwargs)
         if kwargs.get("format_source", True):
             generated_code = codegen.format_source("cpp", generated_code, style="LLVM")
