@@ -1,32 +1,21 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from collections import defaultdict
 from typing import Any, Dict
 
-from gt4py.eve import NodeTranslator
-
-from . import cuir
+from gt4py import eve
+from gt4py.cartesian.gtc.cuir import cuir
 
 
-class CacheExtents(NodeTranslator):
+class CacheExtents(eve.NodeTranslator):
     def visit_IJCacheDecl(
-        self,
-        node: cuir.IJCacheDecl,
-        *,
-        ij_extents: Dict[str, cuir.KExtent],
-        **kwargs: Any,
+        self, node: cuir.IJCacheDecl, *, ij_extents: Dict[str, cuir.KExtent], **kwargs: Any
     ) -> cuir.IJCacheDecl:
         return cuir.IJCacheDecl(name=node.name, dtype=node.dtype, extent=ij_extents[node.name])
 
