@@ -1,16 +1,12 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import functools
 from dataclasses import dataclass, field
@@ -29,8 +25,7 @@ from gt4py.cartesian.gtc.passes.oir_optimizations.utils import (
 
 
 class SymbolNameCreator(Protocol):
-    def __call__(self, name: str) -> str:
-        ...
+    def __call__(self, name: str) -> str: ...
 
 
 def _make_axis_offset_expr(bound: common.AxisBound, axis_index: int) -> cuir.Expr:
@@ -87,9 +82,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
 
     def visit_BinaryOp(self, node: oir.BinaryOp, **kwargs: Any) -> cuir.BinaryOp:
         return cuir.BinaryOp(
-            op=node.op,
-            left=self.visit(node.left, **kwargs),
-            right=self.visit(node.right, **kwargs),
+            op=node.op, left=self.visit(node.left, **kwargs), right=self.visit(node.right, **kwargs)
         )
 
     def visit_Temporary(self, node: oir.Temporary, **kwargs: Any) -> cuir.Temporary:
@@ -102,7 +95,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
     ) -> cuir.VariableKOffset:
         return cuir.VariableKOffset(k=self.visit(node.k, **kwargs))
 
-    def _mask_to_expr(self, mask: common.HorizontalMask, ctx: "Context") -> cuir.Expr:
+    def _mask_to_expr(self, mask: common.HorizontalMask, ctx: Context) -> cuir.Expr:
         mask_expr: List[cuir.Expr] = []
         for axis_index, interval in enumerate(mask.intervals):
             if interval.is_single_index():
@@ -149,23 +142,13 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
         *,
         ij_caches: Dict[str, cuir.IJCacheDecl],
         k_caches: Dict[str, cuir.KCacheDecl],
-        ctx: "Context",
+        ctx: Context,
         **kwargs: Any,
     ) -> Union[cuir.FieldAccess, cuir.IJCacheAccess, cuir.KCacheAccess]:
         data_index = self.visit(
-            node.data_index,
-            ij_caches=ij_caches,
-            k_caches=k_caches,
-            ctx=ctx,
-            **kwargs,
-        )
-        offset = self.visit(
-            node.offset,
-            ij_caches=ij_caches,
-            k_caches=k_caches,
-            ctx=ctx,
-            **kwargs,
+            node.data_index, ij_caches=ij_caches, k_caches=k_caches, ctx=ctx, **kwargs
         )
+        offset = self.visit(node.offset, ij_caches=ij_caches, k_caches=k_caches, ctx=ctx, **kwargs)
         if node.name in ij_caches:
             return cuir.IJCacheAccess(
                 name=ij_caches[node.name].name,
@@ -182,10 +165,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
             )
         ctx.accessed_fields.add(node.name)
         return cuir.FieldAccess(
-            name=node.name,
-            offset=offset,
-            data_index=data_index,
-            dtype=node.dtype,
+            name=node.name, offset=offset, data_index=data_index, dtype=node.dtype
         )
 
     def visit_ScalarAccess(
@@ -248,12 +228,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
         )
 
     def visit_VerticalLoop(
-        self,
-        node: oir.VerticalLoop,
-        *,
-        symtable: Dict[str, Any],
-        ctx: "Context",
-        **kwargs: Any,
+        self, node: oir.VerticalLoop, *, symtable: Dict[str, Any], ctx: Context, **kwargs: Any
     ) -> cuir.Kernel:
         assert not any(c.fill or c.flush for c in node.caches if isinstance(c, oir.KCache))
         ij_caches = {
@@ -281,7 +256,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
                     ij_caches=list(ij_caches.values()),
                     k_caches=list(k_caches.values()),
                 )
-            ],
+            ]
         )
 
     def visit_Stencil(self, node: oir.Stencil, **kwargs: Any) -> cuir.Program:
@@ -289,12 +264,7 @@ class OIRToCUIR(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
         ctx = self.Context(
             new_symbol_name=cast(SymbolNameCreator, symbol_name_creator(collect_symbol_names(node)))
         )
-        kernels = self.visit(
-            node.vertical_loops,
-            ctx=ctx,
-            block_extents=block_extents,
-            **kwargs,
-        )
+        kernels = self.visit(node.vertical_loops, ctx=ctx, block_extents=block_extents, **kwargs)
         temporaries = [self.visit(d) for d in node.declarations if d.name in ctx.accessed_fields]
         return cuir.Program(
             name=node.name,
