@@ -1,26 +1,18 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 from typing import TYPE_CHECKING, List
 
-import dace
-import dace.data
-import dace.library
-import dace.subsets
-
 from gt4py import eve
-from gt4py.cartesian.gtc import common, daceir as dcir, oir
+from gt4py.cartesian.gtc import common, oir
+from gt4py.cartesian.gtc.dace import daceir as dcir
 from gt4py.cartesian.gtc.definitions import Extent
 
 
@@ -28,22 +20,8 @@ if TYPE_CHECKING:
     from gt4py.cartesian.gtc.dace.nodes import StencilComputation
 
 
-def get_dace_debuginfo(node: common.LocNode):
-
-    if node.loc is not None:
-        return dace.dtypes.DebugInfo(
-            node.loc.line,
-            node.loc.column,
-            node.loc.line,
-            node.loc.column,
-            node.loc.source,
-        )
-    else:
-        return dace.dtypes.DebugInfo(0)
-
-
 class HorizontalIntervalRemover(eve.NodeTranslator):
-    def visit_HorizontalMask(self, node: common.HorizontalMask, *, axis: "dcir.Axis"):
+    def visit_HorizontalMask(self, node: common.HorizontalMask, *, axis: dcir.Axis):
         mask_attrs = dict(i=node.i, j=node.j)
         mask_attrs[axis.lower()] = self.visit(getattr(node, axis.lower()))
         return common.HorizontalMask(**mask_attrs)
@@ -53,8 +31,7 @@ class HorizontalIntervalRemover(eve.NodeTranslator):
 
 
 class HorizontalMaskRemover(eve.NodeTranslator):
-    def visit_Tasklet(self, node: "dcir.Tasklet"):
-
+    def visit_Tasklet(self, node: dcir.Tasklet):
         res_body = []
         for stmt in node.stmts:
             newstmt = self.visit(stmt)
@@ -63,8 +40,8 @@ class HorizontalMaskRemover(eve.NodeTranslator):
             else:
                 res_body.append(newstmt)
         return dcir.Tasklet(
+            label=f"he_remover_{id(node)}",
             stmts=res_body,
-            decls=node.decls,
             read_memlets=node.read_memlets,
             write_memlets=node.write_memlets,
         )
@@ -173,8 +150,7 @@ class HorizontalExecutionSplitter(eve.NodeTranslator):
         return oir.VerticalLoopSection(interval=node.interval, horizontal_executions=res_hes)
 
 
-def split_horizontal_executions_regions(node: "StencilComputation"):
-
+def split_horizontal_executions_regions(node: StencilComputation):
     extents: List[Extent] = []
 
     node.oir_node = HorizontalExecutionSplitter().visit(
