@@ -1,21 +1,17 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import base64
 import pickle
 import typing
-from typing import Dict, List, Optional, Set, Union
+from typing import Dict, Final, List, Optional, Set, Union
 
 import dace.data
 import dace.dtypes
@@ -24,23 +20,24 @@ import dace.subsets
 import numpy as np
 from dace import library
 
-from gt4py.cartesian.gtc import common, daceir as dcir, oir
+from gt4py.cartesian.gtc import common, oir
+from gt4py.cartesian.gtc.dace import daceir as dcir
 from gt4py.cartesian.gtc.dace.expansion.expansion import StencilComputationExpansion
+from gt4py.cartesian.gtc.dace.expansion.utils import HorizontalExecutionSplitter
+from gt4py.cartesian.gtc.dace.expansion_specification import ExpansionItem, make_expansion_order
+from gt4py.cartesian.gtc.dace.utils import get_dace_debuginfo
 from gt4py.cartesian.gtc.definitions import Extent
 from gt4py.cartesian.gtc.oir import Decl, FieldDecl, VerticalLoop, VerticalLoopSection
 
-from .expansion.utils import HorizontalExecutionSplitter, get_dace_debuginfo
-from .expansion_specification import ExpansionItem, make_expansion_order
-
 
 def _set_expansion_order(
-    node: "StencilComputation", expansion_order: Union[List[ExpansionItem], List[str]]
+    node: StencilComputation, expansion_order: Union[List[ExpansionItem], List[str]]
 ):
     res = make_expansion_order(node, expansion_order)
     node._expansion_specification = res
 
 
-def _set_tile_sizes_interpretation(node: "StencilComputation", tile_sizes_interpretation: str):
+def _set_tile_sizes_interpretation(node: StencilComputation, tile_sizes_interpretation: str):
     valid_values = {"shape", "strides"}
     if tile_sizes_interpretation not in valid_values:
         raise ValueError(f"tile_sizes_interpretation must be one in {valid_values}.")
@@ -56,9 +53,17 @@ class PickledProperty:
 
     @classmethod
     def from_json(cls, d, sdfg=None):
-        b64string = d["pickle"]
-        byte_repr = base64.b64decode(b64string)
-        return pickle.loads(byte_repr)
+        # DaCe won't serialize attr with default values by default
+        # which would lead the deserializer to push a default in the
+        # wrong format (non pickle).
+        # Best mitigation is to give back the object plain if it does
+        # not contain any pickling information
+        if isinstance(d, dict) and "pickle" in d.keys():
+            b64string = d["pickle"]
+            byte_repr = base64.b64decode(b64string)
+            return pickle.loads(byte_repr)
+
+        return d
 
 
 class PickledDataclassProperty(PickledProperty, dace.properties.DataclassProperty):
@@ -75,7 +80,7 @@ class PickledDictProperty(PickledProperty, dace.properties.DictProperty):
 
 @library.node
 class StencilComputation(library.LibraryNode):
-    implementations: Dict[str, dace.library.ExpandTransformation] = {
+    implementations: Final[Dict[str, dace.library.ExpandTransformation]] = {
         "default": StencilComputationExpansion
     }
     default_implementation = "default"
@@ -92,14 +97,10 @@ class StencilComputation(library.LibraryNode):
         dtype=dace.DeviceType, default=dace.DeviceType.CPU, allow_none=True
     )
     expansion_specification = PickledListProperty(
-        element_type=ExpansionItem,
-        allow_none=True,
-        setter=_set_expansion_order,
+        element_type=ExpansionItem, allow_none=True, setter=_set_expansion_order
     )
     tile_sizes = PickledDictProperty(
-        key_type=dcir.Axis,
-        value_type=int,
-        default={dcir.Axis.I: 8, dcir.Axis.J: 8, dcir.Axis.K: 8},
+        key_type=dcir.Axis, value_type=int, default={dcir.Axis.I: 8, dcir.Axis.J: 8, dcir.Axis.K: 8}
     )
 
     tile_sizes_interpretation = dace.properties.Property(
@@ -121,7 +122,7 @@ class StencilComputation(library.LibraryNode):
         *args,
         **kwargs,
     ):
-        super().__init__(name=name, *args, **kwargs)
+        super().__init__(*args, name=name, **kwargs)
 
         from gt4py.cartesian.gtc.dace.utils import compute_dcir_access_infos
 
@@ -215,10 +216,8 @@ class StencilComputation(library.LibraryNode):
     def tile_strides(self):
         if self.tile_sizes_interpretation == "strides":
             return self.tile_sizes
-        else:
-            overall_extent: Extent = next(iter(self.extents.values()))
-            for extent in self.extents.values():
-                overall_extent |= extent
-            return {
-                key: value + overall_extent[key.to_idx()] for key, value in self.tile_sizes.items()
-            }
+
+        overall_extent: Extent = next(iter(self.extents.values()))
+        for extent in self.extents.values():
+            overall_extent |= extent
+        return {key: value + overall_extent[key.to_idx()] for key, value in self.tile_sizes.items()}
