@@ -1,49 +1,53 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import re
 from dataclasses import dataclass, field
 from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
 
-import dace
-import dace.data
 import numpy as np
+from dace import data, dtypes, properties, subsets, symbolic
 
 from gt4py import eve
-from gt4py.cartesian.gtc import common, daceir as dcir, oir
-from gt4py.cartesian.gtc.common import CartesianOffset
+from gt4py.cartesian.gtc import common, oir
+from gt4py.cartesian.gtc.common import CartesianOffset, VariableKOffset
+from gt4py.cartesian.gtc.dace import daceir as dcir, prefix
 from gt4py.cartesian.gtc.passes.oir_optimizations.utils import compute_horizontal_block_extents
 
 
-def array_dimensions(array: dace.data.Array):
-    dims = [
+def get_dace_debuginfo(node: common.LocNode) -> dtypes.DebugInfo:
+    if node.loc is None:
+        return dtypes.DebugInfo(0)
+
+    return dtypes.DebugInfo(
+        node.loc.line, node.loc.column, node.loc.line, node.loc.column, node.loc.filename
+    )
+
+
+def array_dimensions(array: data.Array) -> list[bool]:
+    return [
         any(
             re.match(f"__.*_{k}_stride", str(sym))
             for st in array.strides
-            for sym in dace.symbolic.pystr_to_symbolic(st).free_symbols
+            for sym in symbolic.pystr_to_symbolic(st).free_symbols
         )
         or any(
             re.match(f"__{k}", str(sym))
             for sh in array.shape
-            for sym in dace.symbolic.pystr_to_symbolic(sh).free_symbols
+            for sym in symbolic.pystr_to_symbolic(sh).free_symbols
         )
         for k in "IJK"
     ]
-    return dims
 
 
-def replace_strides(arrays, get_layout_map):
+def replace_strides(arrays: list[data.Array], get_layout_map) -> dict[str, str]:
     symbol_mapping = {}
     for array in arrays:
         dims = array_dimensions(array)
@@ -55,26 +59,31 @@ def replace_strides(arrays, get_layout_map):
             for idx in reversed(np.argsort(layout)):
                 symbol = array.strides[idx]
                 if symbol.is_symbol:
-                    symbol_mapping[str(symbol)] = dace.symbolic.pystr_to_symbolic(stride)
+                    symbol_mapping[str(symbol)] = symbolic.pystr_to_symbolic(stride)
                 stride *= array.shape[idx]
     return symbol_mapping
 
 
-def get_tasklet_symbol(name, offset, is_target):
-    if is_target:
-        return f"__{name}"
+def get_tasklet_symbol(
+    name: str,
+    *,
+    offset: Optional[CartesianOffset | VariableKOffset] = None,
+    is_target: bool,
+):
+    access_name = f"{prefix.TASKLET_OUT}{name}" if is_target else f"{prefix.TASKLET_IN}{name}"
+    if offset is None:
+        return access_name
 
-    acc_name = name + "__"
-    if offset is not None:
-        offset_strs = []
-        for axis in dcir.Axis.dims_3d():
-            off = offset.to_dict()[axis.lower()]
-            if off is not None and off != 0:
-                offset_strs.append(axis.lower() + ("m" if off < 0 else "p") + f"{abs(off):d}")
-        suffix = "_".join(offset_strs)
-        if suffix != "":
-            acc_name += suffix
-    return acc_name
+    # add (per axis) offset markers, e.g. gtIN__A_km1 for A[0, 0, -1]
+    offset_strings = []
+    for axis in dcir.Axis.dims_3d():
+        axis_offset = offset.to_dict()[axis.lower()]
+        if axis_offset is not None and axis_offset != 0:
+            offset_strings.append(
+                axis.lower() + ("m" if axis_offset < 0 else "p") + f"{abs(axis_offset):d}"
+            )
+
+    return access_name + "_".join(offset_strings)
 
 
 def axes_list_from_flags(flags):
@@ -89,25 +98,19 @@ class AccessInfoCollector(eve.NodeVisitor):
 
     @dataclass
     class Context:
-        axes: Dict[str, List["dcir.Axis"]]
-        access_infos: Dict[str, "dcir.FieldAccessInfo"] = field(default_factory=dict)
+        axes: Dict[str, List[dcir.Axis]]
+        access_infos: Dict[str, dcir.FieldAccessInfo] = field(default_factory=dict)
 
     def visit_VerticalLoop(
         self, node: oir.VerticalLoop, *, block_extents, ctx, **kwargs: Any
-    ) -> Dict[str, "dcir.FieldAccessInfo"]:
+    ) -> Dict[str, dcir.FieldAccessInfo]:
         for section in reversed(node.sections):
             self.visit(section, block_extents=block_extents, ctx=ctx, **kwargs)
         return ctx.access_infos
 
     def visit_VerticalLoopSection(
-        self,
-        node: oir.VerticalLoopSection,
-        *,
-        block_extents,
-        ctx,
-        grid_subset=None,
-        **kwargs: Any,
-    ) -> Dict[str, "dcir.FieldAccessInfo"]:
+        self, node: oir.VerticalLoopSection, *, block_extents, ctx, grid_subset=None, **kwargs: Any
+    ) -> Dict[str, dcir.FieldAccessInfo]:
         inner_ctx = self.Context(axes=ctx.axes)
 
         if grid_subset is None:
@@ -147,7 +150,7 @@ class AccessInfoCollector(eve.NodeVisitor):
         k_interval,
         grid_subset=None,
         **kwargs,
-    ) -> Dict[str, "dcir.FieldAccessInfo"]:
+    ) -> Dict[str, dcir.FieldAccessInfo]:
         horizontal_extent = block_extents(node)
 
         inner_ctx = self.Context(axes=ctx.axes)
@@ -190,16 +193,16 @@ class AccessInfoCollector(eve.NodeVisitor):
         self.visit(node.body, is_conditional=True, region=node.mask, **kwargs)
 
     def visit_MaskStmt(self, node: oir.MaskStmt, *, is_conditional=False, **kwargs):
-
         self.visit(node.mask, is_conditional=is_conditional, **kwargs)
         self.visit(node.body, is_conditional=True, **kwargs)
 
     def visit_While(self, node: oir.While, *, is_conditional=False, **kwargs):
-        self.generic_visit(node, is_conditional=True, **kwargs)
+        self.visit(node.cond, is_conditional=is_conditional, **kwargs)
+        self.visit(node.body, is_conditional=True, **kwargs)
 
     @staticmethod
     def _global_grid_subset(
-        region: common.HorizontalMask, he_grid: "dcir.GridSubset", offset: List[Optional[int]]
+        region: common.HorizontalMask, he_grid: dcir.GridSubset, offset: List[Optional[int]]
     ):
         res: Dict[
             dcir.Axis, Union[dcir.DomainInterval, dcir.TileInterval, dcir.IndexWithExtent]
@@ -238,12 +241,11 @@ class AccessInfoCollector(eve.NodeVisitor):
         region,
         he_grid,
         grid_subset,
-    ) -> "dcir.FieldAccessInfo":
+        is_write,
+    ) -> dcir.FieldAccessInfo:
+        # Check we have expression offsets in K
         offset = [offset_node.to_dict()[k] for k in "ijk"]
-        if isinstance(offset_node, oir.VariableKOffset):
-            variable_offset_axes = [dcir.Axis.K]
-        else:
-            variable_offset_axes = []
+        variable_offset_axes = [dcir.Axis.K] if isinstance(offset_node, oir.VariableKOffset) else []
 
         global_subset = self._global_grid_subset(region, he_grid, offset)
         intervals = {}
@@ -262,7 +264,6 @@ class AccessInfoCollector(eve.NodeVisitor):
         return dcir.FieldAccessInfo(
             grid_subset=grid_subset,
             global_grid_subset=global_subset,
-            dynamic_access=len(variable_offset_axes) > 0 or is_conditional or region is not None,
             variable_offset_axes=variable_offset_axes,
         )
 
@@ -275,7 +276,7 @@ class AccessInfoCollector(eve.NodeVisitor):
         is_write: bool = False,
         is_conditional: bool = False,
         region=None,
-        ctx: "AccessInfoCollector.Context",
+        ctx: AccessInfoCollector.Context,
         **kwargs,
     ):
         self.visit(
@@ -299,6 +300,7 @@ class AccessInfoCollector(eve.NodeVisitor):
             region=region,
             he_grid=he_grid,
             grid_subset=grid_subset,
+            is_write=is_write,
         )
         ctx.access_infos[node.name] = access_info.union(
             ctx.access_infos.get(node.name, access_info)
@@ -314,7 +316,7 @@ def compute_dcir_access_infos(
     collect_write=True,
     include_full_domain=False,
     **kwargs,
-) -> dace.properties.DictProperty:
+) -> properties.DictProperty:
     if block_extents is None:
         assert isinstance(oir_node, oir.Stencil)
         block_extents = compute_horizontal_block_extents(oir_node)
@@ -337,23 +339,186 @@ def compute_dcir_access_infos(
                     global_grid_subset=access_info.global_grid_subset,
                 )
             )
-    else:
-        res = ctx.access_infos
+        return res
 
-    return res
+    return ctx.access_infos
+
+
+class TaskletAccessInfoCollector(eve.NodeVisitor):
+    @dataclass
+    class Context:
+        axes: dict[str, list[dcir.Axis]]
+        access_infos: dict[str, dcir.FieldAccessInfo] = field(default_factory=dict)
+
+    def __init__(
+        self, collect_read: bool, collect_write: bool, *, horizontal_extent, k_interval, grid_subset
+    ):
+        self.collect_read: bool = collect_read
+        self.collect_write: bool = collect_write
+
+        self.ij_grid = dcir.GridSubset.from_gt4py_extent(horizontal_extent)
+        self.he_grid = self.ij_grid.set_interval(dcir.Axis.K, k_interval)
+        self.grid_subset = grid_subset
+
+    def visit_CodeBlock(self, _node: oir.CodeBlock, **_kwargs):
+        raise RuntimeError("We shouldn't reach code blocks anymore")
+
+    def visit_AssignStmt(self, node: oir.AssignStmt, **kwargs):
+        self.visit(node.right, is_write=False, **kwargs)
+        self.visit(node.left, is_write=True, **kwargs)
+
+    def visit_MaskStmt(self, node: oir.MaskStmt, **kwargs):
+        self.visit(node.mask, is_write=False, **kwargs)
+        self.visit(node.body, **kwargs)
+
+    def visit_While(self, node: oir.While, **kwargs):
+        self.visit(node.cond, is_write=False, **kwargs)
+        self.visit(node.body, **kwargs)
+
+    def visit_HorizontalRestriction(self, node: oir.HorizontalRestriction, **kwargs):
+        self.visit(node.mask, is_write=False, **kwargs)
+        self.visit(node.body, region=node.mask, **kwargs)
+
+    def _global_grid_subset(
+        self,
+        region: Optional[common.HorizontalMask],
+        offset: list[Optional[int]],
+    ):
+        res: dict[dcir.Axis, dcir.DomainInterval | dcir.IndexWithExtent | dcir.TileInterval] = {}
+        if region is not None:
+            for axis, oir_interval in zip(dcir.Axis.dims_horizontal(), region.intervals):
+                he_grid_interval = self.he_grid.intervals[axis]
+                assert isinstance(he_grid_interval, dcir.DomainInterval)
+                start = (
+                    oir_interval.start if oir_interval.start is not None else he_grid_interval.start
+                )
+                end = oir_interval.end if oir_interval.end is not None else he_grid_interval.end
+                dcir_interval = dcir.DomainInterval(
+                    start=dcir.AxisBound.from_common(axis, start),
+                    end=dcir.AxisBound.from_common(axis, end),
+                )
+                res[axis] = dcir.DomainInterval.union(dcir_interval, res.get(axis, dcir_interval))
+        if dcir.Axis.K in self.he_grid.intervals:
+            off = offset[dcir.Axis.K.to_idx()] or 0
+            he_grid_k_interval = self.he_grid.intervals[dcir.Axis.K]
+            assert not isinstance(he_grid_k_interval, dcir.TileInterval)
+            res[dcir.Axis.K] = he_grid_k_interval.shifted(off)
+        for axis in dcir.Axis.dims_horizontal():
+            iteration_interval = self.he_grid.intervals[axis]
+            mask_interval = res.get(axis, iteration_interval)
+            res[axis] = dcir.DomainInterval.intersection(
+                axis, iteration_interval, mask_interval
+            ).shifted(offset[axis.to_idx()])
+        return dcir.GridSubset(intervals=res)
+
+    def _make_access_info(
+        self,
+        offset_node: CartesianOffset | VariableKOffset,
+        axes,
+        region: Optional[common.HorizontalMask],
+    ) -> dcir.FieldAccessInfo:
+        # Check we have expression offsets in K
+        offset = [offset_node.to_dict()[k] for k in "ijk"]
+        variable_offset_axes = [dcir.Axis.K] if isinstance(offset_node, VariableKOffset) else []
+
+        global_subset = self._global_grid_subset(region, offset)
+        intervals = {}
+        for axis in axes:
+            extent = (
+                (0, 0)
+                if axis in variable_offset_axes
+                else (offset[axis.to_idx()], offset[axis.to_idx()])
+            )
+            intervals[axis] = dcir.IndexWithExtent(
+                axis=axis, value=axis.iteration_symbol(), extent=extent
+            )
+
+        return dcir.FieldAccessInfo(
+            grid_subset=dcir.GridSubset(intervals=intervals),
+            global_grid_subset=global_subset,
+            # Field access inside horizontal regions might or might not happen
+            dynamic_access=region is not None,
+            variable_offset_axes=variable_offset_axes,
+        )
+
+    def visit_FieldAccess(
+        self,
+        node: oir.FieldAccess,
+        *,
+        is_write: bool,
+        region: Optional[common.HorizontalMask] = None,
+        ctx: TaskletAccessInfoCollector.Context,
+        **kwargs,
+    ):
+        self.visit(node.offset, ctx=ctx, is_write=False, region=region, **kwargs)
+
+        if (is_write and not self.collect_write) or (not is_write and not self.collect_read):
+            return
+
+        access_info = self._make_access_info(
+            node.offset,
+            axes=ctx.axes[node.name],
+            region=region,
+        )
+        ctx.access_infos[node.name] = access_info.union(
+            ctx.access_infos.get(node.name, access_info)
+        )
+
+
+def compute_tasklet_access_infos(
+    node: oir.CodeBlock | oir.MaskStmt | oir.While,
+    *,
+    collect_read: bool = True,
+    collect_write: bool = True,
+    declarations: dict[str, oir.Decl],
+    horizontal_extent,
+    k_interval,
+    grid_subset,
+):
+    """
+    Compute access information needed to build Memlets for the Tasklet
+    associated with the given `node`.
+    """
+    axes = {
+        name: axes_list_from_flags(declaration.dimensions)
+        for name, declaration in declarations.items()
+        if isinstance(declaration, oir.FieldDecl)
+    }
+    ctx = TaskletAccessInfoCollector.Context(axes=axes, access_infos=dict())
+    collector = TaskletAccessInfoCollector(
+        collect_read=collect_read,
+        collect_write=collect_write,
+        horizontal_extent=horizontal_extent,
+        k_interval=k_interval,
+        grid_subset=grid_subset,
+    )
+    if isinstance(node, oir.CodeBlock):
+        collector.visit(node.body, ctx=ctx)
+    elif isinstance(node, oir.MaskStmt):
+        # node.mask is a simple expression.
+        # Pass `is_write` explicitly since we don't automatically set it in `visit_AssignStmt()`
+        collector.visit(node.mask, ctx=ctx, is_write=False)
+    elif isinstance(node, oir.While):
+        # node.cond is a simple expression.
+        # Pass `is_write` explicitly since we don't automatically set it in `visit_AssignStmt()`
+        collector.visit(node.cond, ctx=ctx, is_write=False)
+    else:
+        raise ValueError("Unexpected node type.")
+
+    return ctx.access_infos
 
 
 def make_dace_subset(
-    context_info: "dcir.FieldAccessInfo",
-    access_info: "dcir.FieldAccessInfo",
+    context_info: dcir.FieldAccessInfo,
+    access_info: dcir.FieldAccessInfo,
     data_dims: Tuple[int, ...],
-) -> dace.subsets.Range:
+) -> subsets.Range:
     clamped_access_info = access_info
     clamped_context_info = context_info
     for axis in access_info.axes():
         if axis in access_info.variable_offset_axes:
             clamped_access_info = clamped_access_info.clamp_full_axis(axis)
-        if axis in clamped_context_info.variable_offset_axes:
+        if axis in context_info.variable_offset_axes:
             clamped_context_info = clamped_context_info.clamp_full_axis(axis)
     res_ranges = []
 
@@ -364,13 +529,11 @@ def make_dace_subset(
         ].to_dace_symbolic()
         res_ranges.append((subset_start - context_start, subset_end - context_start - 1, 1))
     res_ranges.extend((0, dim - 1, 1) for dim in data_dims)
-    return dace.subsets.Range(res_ranges)
+    return subsets.Range(res_ranges)
 
 
-def untile_memlets(
-    memlets: Sequence["dcir.Memlet"], axes: Sequence["dcir.Axis"]
-) -> List["dcir.Memlet"]:
-    res_memlets: List["dcir.Memlet"] = []
+def untile_memlets(memlets: Sequence[dcir.Memlet], axes: Sequence[dcir.Axis]) -> List[dcir.Memlet]:
+    res_memlets: List[dcir.Memlet] = []
     for memlet in memlets:
         res_memlets.append(
             dcir.Memlet(
@@ -395,7 +558,7 @@ def union_node_grid_subsets(nodes: List[eve.Node]):
     return grid_subset
 
 
-def _union_memlets(*memlets: "dcir.Memlet") -> List["dcir.Memlet"]:
+def _union_memlets(*memlets: dcir.Memlet) -> List[dcir.Memlet]:
     res: Dict[str, dcir.Memlet] = {}
     for memlet in memlets:
         res[memlet.field] = memlet.union(res.get(memlet.field, memlet))
@@ -420,8 +583,8 @@ def flatten_list(list_or_node: Union[List[Any], eve.Node]):
 
 
 def collect_toplevel_computation_nodes(
-    list_or_node: Union[List[Any], eve.Node]
-) -> List["dcir.ComputationNode"]:
+    list_or_node: Union[List[Any], eve.Node],
+) -> List[dcir.ComputationNode]:
     class ComputationNodeCollector(eve.NodeVisitor):
         def visit_ComputationNode(self, node: dcir.ComputationNode, *, collection: List):
             collection.append(node)
@@ -432,8 +595,8 @@ def collect_toplevel_computation_nodes(
 
 
 def collect_toplevel_iteration_nodes(
-    list_or_node: Union[List[Any], eve.Node]
-) -> List["dcir.IterationNode"]:
+    list_or_node: Union[List[Any], eve.Node],
+) -> List[dcir.IterationNode]:
     class IterationNodeCollector(eve.NodeVisitor):
         def visit_IterationNode(self, node: dcir.IterationNode, *, collection: List):
             collection.append(node)
