@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """
 GridTools Intermediate Representation.
@@ -25,6 +19,8 @@ Analysis is required to generate valid code (complying with the parallel model)
 - `FieldIfStmt` expansion to comply with the parallel model
 """
 
+from __future__ import annotations
+
 from typing import Any, Dict, List, Set, Tuple, Type
 
 from gt4py import eve
@@ -47,7 +43,7 @@ class BlockStmt(common.BlockStmt[Stmt], Stmt):
     pass
 
 
-class Literal(common.Literal, Expr):  # type: ignore
+class Literal(common.Literal, Expr):
     pass
 
 
@@ -55,11 +51,11 @@ class VariableKOffset(common.VariableKOffset[Expr]):
     pass
 
 
-class ScalarAccess(common.ScalarAccess, Expr):  # type: ignore
+class ScalarAccess(common.ScalarAccess, Expr):
     pass
 
 
-class FieldAccess(common.FieldAccess[Expr, VariableKOffset], Expr):  # type: ignore
+class FieldAccess(common.FieldAccess[Expr, VariableKOffset], Expr):
     pass
 
 
@@ -83,7 +79,7 @@ class ParAssignStmt(common.AssignStmt[FieldAccess, Expr], Stmt):
     @datamodels.root_validator
     @classmethod
     def no_write_and_read_with_offset_of_same_field(
-        cls: Type["ParAssignStmt"], instance: "ParAssignStmt"
+        cls: Type[ParAssignStmt], instance: ParAssignStmt
     ) -> None:
         if isinstance(instance.left, FieldAccess):
             offset_reads = (
@@ -167,7 +163,7 @@ class TernaryOp(common.TernaryOp[Expr], Expr):
     _dtype_propagation = common.ternary_op_dtype_propagation(strict=False)
 
 
-class Cast(common.Cast[Expr], Expr):  # type: ignore
+class Cast(common.Cast[Expr], Expr):
     pass
 
 
@@ -209,7 +205,7 @@ class VerticalLoop(LocNode):
     @datamodels.root_validator
     @classmethod
     def _no_write_and_read_with_horizontal_offset(
-        cls: Type["VerticalLoop"], instance: "VerticalLoop"
+        cls: Type[VerticalLoop], instance: VerticalLoop
     ) -> None:
         """
         In the same VerticalLoop a field must not be written and read with a horizontal offset.
@@ -257,9 +253,10 @@ def _variablek_fieldaccess(node) -> bool:
     return isinstance(node, FieldAccess) and isinstance(node.offset, VariableKOffset)
 
 
+# TODO(havogt): either move to eve or will be removed in the attr-based eve if a List[Node] is represented as a CollectionNode
 def _written_and_read_with_offset(stmts: List[Stmt]) -> Set[str]:
     """Return a list of names that are written to and read with offset."""
-    # TODO(havogt): either move to eve or will be removed in the attr-based eve if a List[Node] is represented as a CollectionNode
+
     def _writes(stmts: List[Stmt]) -> Set[str]:
         result = set()
         for left in eve.walk_values(stmts).if_isinstance(ParAssignStmt).getattr("left"):
