@@ -1,27 +1,21 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from gt4py import eve
-from gt4py.cartesian.gtc import common, oir, utils
+from gt4py.cartesian import utils
+from gt4py.cartesian.gtc import common, oir
 from gt4py.cartesian.gtc.definitions import Extent
+from gt4py.cartesian.gtc.numpy import npir
 from gt4py.cartesian.gtc.passes.horizontal_masks import compute_relative_mask
 from gt4py.cartesian.gtc.passes.oir_optimizations.utils import compute_extents
 
-from . import npir
-
 
 class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
     """Lower from optimizable IR (OIR) to numpy IR (NPIR)."""
@@ -103,9 +97,7 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
         self, node: oir.BinaryOp, **kwargs: Any
     ) -> Union[npir.VectorArithmetic, npir.VectorLogic]:
         args = dict(
-            op=node.op,
-            left=self.visit(node.left, **kwargs),
-            right=self.visit(node.right, **kwargs),
+            op=node.op, left=self.visit(node.left, **kwargs), right=self.visit(node.right, **kwargs)
         )
         if isinstance(node.op, common.LogicalOperator):
             return npir.VectorLogic(**args)
@@ -135,17 +127,13 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
 
     # --- Statements ---
     def visit_MaskStmt(
-        self,
-        node: oir.MaskStmt,
-        *,
-        mask: Optional[npir.Expr] = None,
-        **kwargs: Any,
+        self, node: oir.MaskStmt, *, mask: Optional[npir.Expr] = None, **kwargs: Any
     ) -> List[npir.Stmt]:
         mask_expr = self.visit(node.mask, **kwargs)
         if mask:
             mask_expr = npir.VectorLogic(op=common.LogicalOperator.AND, left=mask, right=mask_expr)
 
-        return utils.flatten_list(self.visit(node.body, mask=mask_expr, **kwargs))
+        return utils.flatten(self.visit(node.body, mask=mask_expr, **kwargs))
 
     def visit_AssignStmt(
         self,
@@ -169,13 +157,12 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
     def visit_While(
         self, node: oir.While, *, mask: Optional[npir.Expr] = None, **kwargs: Any
     ) -> npir.While:
-        cond = self.visit(node.cond, mask=mask, **kwargs)
+        cond_expr = self.visit(node.cond, **kwargs)
         if mask:
-            mask = npir.VectorLogic(op=common.LogicalOperator.AND, left=mask, right=cond)
-        else:
-            mask = cond
+            cond_expr = npir.VectorLogic(op=common.LogicalOperator.AND, left=mask, right=cond_expr)
+
         return npir.While(
-            cond=cond, body=utils.flatten_list(self.visit(node.body, mask=mask, **kwargs))
+            cond=cond_expr, body=utils.flatten(self.visit(node.body, mask=cond_expr, **kwargs))
         )
 
     def visit_HorizontalRestriction(
@@ -187,7 +174,7 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
 
         horizontal_mask = npir.HorizontalMask(i=mask[0], j=mask[1])
 
-        return utils.flatten_list(self.visit(node.body, horizontal_mask=horizontal_mask, **kwargs))
+        return utils.flatten(self.visit(node.body, horizontal_mask=horizontal_mask, **kwargs))
 
     # --- Control Flow ---
     def visit_HorizontalExecution(
@@ -202,7 +189,7 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
         else:
             extent = Extent.zeros(ndims=2)
 
-        stmts = utils.flatten_list(self.visit(node.body, extent=extent, **kwargs))
+        stmts = utils.flatten(self.visit(node.body, extent=extent, **kwargs))
         return npir.HorizontalBlock(
             body=stmts, extent=extent, declarations=self.visit(node.declarations, **kwargs)
         )
@@ -236,12 +223,8 @@ class OirToNpir(eve.NodeTranslator, eve.VisitorWithSymbolTableTrait):
             self.visit(decl, field_extents=field_extents, **kwargs) for decl in node.declarations
         ]
 
-        vertical_passes = utils.flatten_list(
-            self.visit(
-                node.vertical_loops,
-                block_extents=block_extents,
-                **kwargs,
-            )
+        vertical_passes = utils.flatten(
+            self.visit(node.vertical_loops, block_extents=block_extents, **kwargs)
         )
 
         return npir.Computation(
