@@ -1,18 +1,15 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Stencil Object that allows for deferred building."""
+
+from __future__ import annotations
+
 from typing import TYPE_CHECKING, Any, Dict
 
 from cached_property import cached_property
@@ -29,7 +26,7 @@ class LazyStencil:
     A stencil object which defers compilation until it is needed.
 
     Usually obtained using the :py:func:`gt4py.gtscript.lazy_stencil` decorator, not directly
-    instanciated.
+    instantiated.
     This is done by keeping a reference to a :py:class:`gt4py.stencil_builder.StencilBuilder`
     instance.
 
@@ -37,12 +34,12 @@ class LazyStencil:
     Low-level build utilities are accessible through the public :code:`builder` attribute.
     """
 
-    def __init__(self, builder: "StencilBuilder"):
+    def __init__(self, builder: StencilBuilder):
         self.builder = builder
         self.builder.caching.capture_externals()
 
     @cached_property
-    def implementation(self) -> "StencilObject":
+    def implementation(self) -> StencilObject:
         """
         Expose the compiled backend-specific python callable which executes the stencil.
 
@@ -53,7 +50,7 @@ class LazyStencil:
         return impl
 
     @property
-    def backend(self) -> "Backend":
+    def backend(self) -> Backend:
         """Do not trigger a build."""
         return self.builder.backend
 
