@@ -1,29 +1,25 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
+
+from __future__ import annotations
 
 import pathlib
-from typing import TYPE_CHECKING, Any, Dict, Optional, Type, Union
+from typing import TYPE_CHECKING, Any
 
 from gt4py import cartesian as gt4pyc
 from gt4py.cartesian.definitions import BuildOptions, StencilID
 from gt4py.cartesian.gtc import gtir
 from gt4py.cartesian.gtc.passes.gtir_pipeline import GtirPipeline
-from gt4py.cartesian.type_hints import AnnotatedStencilFunc, StencilFunc
+from gt4py.cartesian.type_hints import AnnotatedStencilFunc, AnyStencilFunc
 
 
 if TYPE_CHECKING:
-    from gt4py.cartesian.backend.base import Backend as BackendType, CLIBackendMixin
+    from gt4py.cartesian.backend.base import Backend as BackendType
     from gt4py.cartesian.frontend.base import Frontend as FrontendType
     from gt4py.cartesian.stencil_object import StencilObject
 
@@ -50,38 +46,29 @@ class StencilBuilder:
     -----
     Backends can use the :py:meth:`with_backend_data` method the :py:meth:`backend_data` property
     to set and retrieve backend specific information for use in later steps.
-
     """
 
     def __init__(
         self,
-        definition_func: Union[StencilFunc, AnnotatedStencilFunc],
+        definition_func: AnyStencilFunc,
         *,
-        backend: Optional[Union[str, Type["BackendType"]]] = None,
-        options: Optional[BuildOptions] = None,
-        frontend: Optional[Type["FrontendType"]] = None,
+        backend: str | type[BackendType] | None = None,
+        options: BuildOptions | None = None,
+        frontend: type[FrontendType] | None = None,
     ):
         self._definition = definition_func
-        # type ignore explanation: Attribclass generated init not recognized by mypy
-        self.options = options or BuildOptions(  # type: ignore
-            **self.default_options_dict(definition_func)
-        )
+        self.options = options or BuildOptions(**self.default_options_dict(definition_func))
         backend = backend or "numpy"
         backend = gt4pyc.backend.from_name(backend) if isinstance(backend, str) else backend
-        if backend is None:
-            raise RuntimeError(f"Unknown backend: {backend}")
-
         frontend = frontend or gt4pyc.frontend.from_name("gtscript")
-        if frontend is None:
-            raise RuntimeError(f"Unknown frontend: {frontend}")
 
-        self.backend: "BackendType" = backend(self)
-        self.frontend: Type["FrontendType"] = frontend
+        self.backend: BackendType = backend(self)
+        self.frontend: type[FrontendType] = frontend
         self.with_caching("jit")
-        self._externals: Dict[str, Any] = {}
-        self._dtypes: Dict[Type, Type] = {}
+        self._externals: dict[str, Any] = {}
+        self._dtypes: dict[type, type] = {}
 
-    def build(self) -> Type["StencilObject"]:
+    def build(self) -> type[StencilObject]:
         """Generate, compile and/or load everything necessary to provide a usable stencil class."""
         # load or generate
         stencil_class = None if self.options.rebuild else self.backend.load()
@@ -93,17 +80,9 @@ class StencilBuilder:
             stencil_class = self.backend.generate()
         return stencil_class
 
-    def generate_computation(self) -> Dict[str, Union[str, Dict]]:
-        """Generate the stencil source code, fail if backend does not support CLI."""
-        return self.cli_backend.generate_computation()
-
-    def generate_bindings(self, targe_language: str) -> Dict[str, Union[str, Dict]]:
-        """Generate ``target_language`` bindings source, fail if backend does not support CLI."""
-        return self.cli_backend.generate_bindings(targe_language)
-
     def with_caching(
-        self: "StencilBuilder", caching_strategy_name: str, *args: Any, **kwargs: Any
-    ) -> "StencilBuilder":
+        self: StencilBuilder, caching_strategy_name: str, *args: Any, **kwargs: Any
+    ) -> StencilBuilder:
         """
         Fluidly set the caching strategy from the name.
 
@@ -122,14 +101,14 @@ class StencilBuilder:
         -----
         Resets all cached build data.
         """
-        self._build_data: Dict[str, Any] = {}
+        self._build_data: dict[str, Any] = {}
         kwargs = {**self.options.cache_settings, **kwargs}
         self.caching = gt4pyc.caching.strategy_factory(caching_strategy_name, self, *args, **kwargs)
         return self
 
     def with_options(
-        self: "StencilBuilder", *, name: str, module: str, **kwargs: Any
-    ) -> "StencilBuilder":
+        self: StencilBuilder, *, name: str, module: str, **kwargs: Any
+    ) -> StencilBuilder:
         """
         Fluidly set the build options.
 
@@ -143,21 +122,20 @@ class StencilBuilder:
 
         kwargs:
             passed through to the options instance
-
         """
         self._build_data = {}
         # mypy attribkwclass bug
         self.options = BuildOptions(name=name, module=module, **kwargs)  # type: ignore
         return self
 
-    def with_changed_options(self: "StencilBuilder", **kwargs: Dict[str, Any]) -> "StencilBuilder":
+    def with_changed_options(self: StencilBuilder, **kwargs: dict[str, Any]) -> StencilBuilder:
         old_options = self.options.as_dict()
         # BuildOptions constructor expects ``impl_opts`` keyword
         # but BuildOptions.as_dict outputs ``_impl_opts`` key
         old_options["impl_opts"] = old_options.pop("_impl_opts")
         return self.with_options(**{**old_options, **kwargs})
 
-    def with_backend(self: "StencilBuilder", backend_name: str) -> "StencilBuilder":
+    def with_backend(self: StencilBuilder, backend_name: str) -> StencilBuilder:
         """
         Fluidly set the backend type from backend name.
 
@@ -169,23 +147,18 @@ class StencilBuilder:
         Notes
         -----
         Resets all cached build data.
-
-
         """
         self._build_data = {}
         backend = gt4pyc.backend.from_name(backend_name)
-        assert backend is not None
         self.backend = backend(self)
         return self
 
     @classmethod
-    def default_options_dict(
-        cls, definition_func: Union[StencilFunc, AnnotatedStencilFunc]
-    ) -> Dict[str, Any]:
+    def default_options_dict(cls, definition_func: AnyStencilFunc) -> dict[str, Any]:
         return {"name": definition_func.__name__, "module": definition_func.__module__}
 
     @classmethod
-    def name_to_options_args(cls, name: Optional[str]) -> Dict[str, str]:
+    def name_to_options_args(cls, name: str | None) -> dict[str, str]:
         """Check for qualified name, extract also module option in that case."""
         if not name:
             return {}
@@ -196,7 +169,7 @@ class StencilBuilder:
         return data
 
     @classmethod
-    def nest_impl_options(cls, options_dict: Dict[str, Any]) -> Dict[str, Any]:
+    def nest_impl_options(cls, options_dict: dict[str, Any]) -> dict[str, Any]:
         impl_opts = options_dict.setdefault("impl_opts", {})
         # The following is not a dict comprehension because:
         # The backend-specific options (starting with ``_``) are nested under
@@ -214,18 +187,18 @@ class StencilBuilder:
         )
 
     @property
-    def externals(self) -> Dict[str, Any]:
+    def externals(self) -> dict[str, Any]:
         return self._build_data.get("externals") or self._build_data.setdefault(
             "externals", self._externals.copy()
         )
 
     @property
-    def dtypes(self) -> Dict[Type, Type]:
+    def dtypes(self) -> dict[type, type]:
         return self._build_data.get("dtypes") or self._build_data.setdefault(
             "dtypes", self._dtypes.copy()
         )
 
-    def with_externals(self: "StencilBuilder", externals: Dict[str, Any]) -> "StencilBuilder":
+    def with_externals(self: StencilBuilder, externals: dict[str, Any]) -> StencilBuilder:
         """
         Fluidly set externals for this build.
 
@@ -236,17 +209,17 @@ class StencilBuilder:
         self.with_caching(self.caching.name)
         return self
 
-    def with_dtypes(self: "StencilBuilder", dtypes: Dict[Type, Type]) -> "StencilBuilder":
+    def with_dtypes(self: StencilBuilder, dtypes: dict[type, type]) -> StencilBuilder:
         self._build_data = {}
         self._dtypes = dtypes
         self.with_caching(self.caching.name)
         return self
 
     @property
-    def backend_data(self) -> Dict[str, Any]:
+    def backend_data(self) -> dict[str, Any]:
         return self._build_data.get("backend_data", {}).copy()
 
-    def with_backend_data(self: "StencilBuilder", data: Dict[str, Any]) -> "StencilBuilder":
+    def with_backend_data(self: StencilBuilder, data: dict[str, Any]) -> StencilBuilder:
         self._build_data["backend_data"] = {**self.backend_data, **data}
         return self
 
@@ -260,7 +233,7 @@ class StencilBuilder:
             "root_pkg_name", gt4pyc.config.code_settings["root_package_name"]
         )
 
-    def with_root_pkg_name(self: "StencilBuilder", name: str) -> "StencilBuilder":
+    def with_root_pkg_name(self: StencilBuilder, name: str) -> StencilBuilder:
         self._build_data["root_pkg_name"] = name
         return self
 
@@ -281,7 +254,9 @@ class StencilBuilder:
         return self._build_data.get("gtir_pipeline") or self._build_data.setdefault(
             "gtir_pipeline",
             GtirPipeline(
-                self.frontend.generate(self.definition, self.externals, self.dtypes, self.options),
+                self.frontend.generate(
+                    self.definition, self.externals, self.dtypes, self.options, self.backend.name
+                ),
                 self.stencil_id,
             ),
         )
@@ -324,11 +299,3 @@ class StencilBuilder:
     @property
     def is_build_data_empty(self) -> bool:
         return not bool(self._build_data)
-
-    @property
-    def cli_backend(self) -> "CLIBackendMixin":
-        from gt4py.cartesian.backend.base import CLIBackendMixin
-
-        if not isinstance(self.backend, CLIBackendMixin):
-            raise RuntimeError("backend of StencilBuilder instance is not CLI enabled.")
-        return self.backend
