@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from . import attrib, meta, text
 from .base import (
@@ -30,6 +24,7 @@ from .base import (
     is_iterable_of,
     is_mapping_of,
     jsonify,
+    listify,
     make_dir,
     make_local_dir,
     make_module_from_file,
@@ -43,7 +38,7 @@ from .base import (
 )
 
 
-__all__ = [
+__all__ = [  # noqa: RUF022 `__all__` is not sorted
     # Modules
     "attrib",
     "meta",
@@ -65,6 +60,7 @@ __all__ = [
     "is_iterable_of",
     "is_mapping_of",
     "jsonify",
+    "listify",
     "make_dir",
     "make_local_dir",
     "make_module_from_file",
