@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Utilities for runtime: introspection and bytecode."""
 
@@ -18,13 +12,10 @@ import ast
 import copy
 import inspect
 import operator
-import platform
 import textwrap
-from typing import Callable, Dict, List, Tuple, Type
+from typing import Callable, Dict, Final, List, Tuple, Type
 
-from packaging import version
-
-from .base import shashed_id
+from gt4py.cartesian.utils.base import shashed_id
 
 
 def get_closure(func, *, include_globals=True, included_nonlocals=True, include_builtins=True):
@@ -123,16 +114,6 @@ def ast_dump(
     return dumped_ast
 
 
-def ast_unparse(ast_node):
-    """Call ast.unparse, but use astunparse for Python prior to 3.9."""
-    if version.parse(platform.python_version()) < version.parse("3.9"):
-        import astunparse
-
-        return astunparse.unparse(ast_node)
-    else:
-        return ast.unparse(ast_node)
-
-
 def ast_shash(ast_node, *, skip_decorators=True):
     return shashed_id(ast_dump(ast_node, skip_decorators=skip_decorators))
 
@@ -210,7 +191,7 @@ def get_qualified_name_from_node(name_or_attribute, *, as_list=False):
     else:
         assert isinstance(node, ast.Attribute)
         components = get_qualified_name_from_node(node.value, as_list=True)
-        components = components + [node.attr]
+        components = [*components, node.attr]
 
     return components if as_list else ".".join(components)
 
@@ -262,7 +243,7 @@ class ASTTransformPass(ASTPass):
 
 
 class ASTEvaluator(ASTPass):
-    AST_OP_TO_OP: Dict[Type, Callable] = {
+    AST_OP_TO_OP: Final[Dict[Type, Callable]] = {
         # Arithmetic operations
         ast.UAdd: operator.pos,
         ast.USub: operator.neg,
@@ -431,7 +412,7 @@ class QualifiedNameCollector(ASTPass):
             valid = self.prefixes is None or node.id in self.prefixes
         elif isinstance(node, ast.Attribute):
             components, valid = self._get_name_components(node.value)
-            components = components + [node.attr]
+            components = [*components, node.attr]
             valid = valid or ".".join(components) in self.prefixes
         else:
             components = [None]
