@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Eve framework with general utils for development of DSL toolchains in Python.
 
@@ -27,11 +21,9 @@ on some of the previous ones):
   7. visitors
   8. traits
   9. codegen
-
 """
 
-from __future__ import annotations  # isort:skip
-
+from __future__ import annotations
 
 from .concepts import (
     AnnexManager,
@@ -58,7 +50,12 @@ from .datamodels import (
     field,
     frozenmodel,
 )
-from .traits import SymbolTableTrait, ValidatedSymbolTableTrait, VisitorWithSymbolTableTrait
+from .traits import (
+    PreserveLocationVisitor,
+    SymbolTableTrait,
+    ValidatedSymbolTableTrait,
+    VisitorWithSymbolTableTrait,
+)
 from .trees import (
     bfs_walk_items,
     bfs_walk_values,
@@ -73,10 +70,7 @@ from .type_definitions import NOTHING, ConstrainedStr, Enum, IntEnum, NothingTyp
 from .visitors import NodeTranslator, NodeVisitor
 
 
-__all__ = [
-    # version
-    "__version__",
-    "__version_info__",
+__all__ = [  # noqa: RUF022 `__all__` is not sorted
     # concepts
     "AnnexManager",
     "AnySourceLocation",
@@ -90,15 +84,6 @@ __all__ = [
     "SymbolRef",
     "VType",
     "register_annex_user",
-    "# datamodels" "Coerced",
-    "DataModel",
-    "FrozenModel",
-    "GenericDataModel",
-    "Unchecked",
-    "concretize",
-    "datamodel",
-    "field",
-    "frozenmodel",
     # datamodels
     "Coerced",
     "DataModel",
@@ -113,6 +98,7 @@ __all__ = [
     "SymbolTableTrait",
     "ValidatedSymbolTableTrait",
     "VisitorWithSymbolTableTrait",
+    "PreserveLocationVisitor",
     # trees
     "bfs_walk_items",
     "bfs_walk_values",
@@ -122,7 +108,7 @@ __all__ = [
     "pre_walk_values",
     "walk_items",
     "walk_values",
-    "# type_definition",
+    # type_definitions
     "NOTHING",
     "ConstrainedStr",
     "Enum",
