@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Data Model class creation and other utils.
 
@@ -22,6 +16,7 @@ from __future__ import annotations
 import dataclasses
 import functools
 import sys
+import types
 import typing
 import warnings
 
@@ -30,11 +25,11 @@ import attrs
 
 
 try:
-    # For perfomance reasons, try to use cytoolz when possible (using cython)
-    import cytoolz as toolz  # type: ignore[import]
+    # For performance reasons, try to use cytoolz when possible (using cython)
+    import cytoolz as toolz
 except ModuleNotFoundError:
     # Fall back to pure Python toolz
-    import toolz  # type: ignore[import] # noqa: F401
+    import toolz
 
 from .. import exceptions, extended_typing as xtyping, type_validation as type_val, utils
 from ..extended_typing import (
@@ -84,8 +79,7 @@ Attribute: TypeAlias = attr.Attribute
 
 
 class DataModelTP(_AttrsClassTP, xtyping.DevToolsPrettyPrintable, Protocol):
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        ...
+    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
 
     __datamodel_fields__: ClassVar[utils.FrozenNamespace[Attribute]] = cast(
         utils.FrozenNamespace[Attribute], None
@@ -116,8 +110,7 @@ class GenericDataModelTP(DataModelTP, Protocol):
     @classmethod
     def __class_getitem__(
         cls: Type[GenericDataModelTP], args: Union[Type, Tuple[Type, ...]]
-    ) -> Union[DataModelTP, GenericDataModelTP]:
-        ...
+    ) -> Union[DataModelTP, GenericDataModelTP]: ...
 
 
 _DM = TypeVar("_DM", bound="DataModel")
@@ -158,10 +151,7 @@ Unchecked = xtyping.Annotated[_T, _UNCHECKED_TYPE_TAG]
 """Type hint marker to define fields that should NOT be type-checked at initialization."""
 
 
-if sys.version_info >= (3, 10):
-    _dataclass_opts: Final[dict[str, Any]] = {"slots": True}
-else:
-    _dataclass_opts: Final[Dict[str, Any]] = {}
+_dataclass_opts: Final[dict[str, Any]] = {"slots": True}
 
 
 @dataclasses.dataclass(**_dataclass_opts)
@@ -210,22 +200,16 @@ def field_type_validator_factory(
 ) -> FieldTypeValidatorFactory:
     """Create a factory of field type validators from a factory of regular type validators."""
     if use_cache:
-        factory = cast(
-            type_val.TypeValidatorFactory,
-            utils.optional_lru_cache(func=factory),
-        )
+        factory = cast(type_val.TypeValidatorFactory, utils.optional_lru_cache(func=factory))
 
-    def _field_type_validator_factory(
-        type_annotation: TypeAnnotation,
-        name: str,
-    ) -> FieldValidator:
+    def _field_type_validator_factory(type_annotation: TypeAnnotation, name: str) -> FieldValidator:
         """Field type validator for datamodels, supporting forward references."""
         if isinstance(type_annotation, ForwardRef):
             return ForwardRefValidator(factory)
         else:
             simple_validator = factory(type_annotation, name, required=True)
             return ValidatorAdapter(
-                simple_validator, f"{getattr(simple_validator,'__name__', 'TypeValidator')}"
+                simple_validator, f"{getattr(simple_validator, '__name__', 'TypeValidator')}"
             )
 
     return _field_type_validator_factory
@@ -269,7 +253,7 @@ def datamodel(
     cls: Literal[None] = None,
     /,
     *,
-    repr: bool = _REPR_DEFAULT,  # noqa: A002  # shadowing 'repr' python builtin
+    repr: bool = _REPR_DEFAULT,
     eq: bool = _EQ_DEFAULT,
     order: bool = _ORDER_DEFAULT,
     unsafe_hash: bool = _UNSAFE_HASH_DEFAULT,
@@ -280,16 +264,15 @@ def datamodel(
     coerce: bool = _COERCE_DEFAULT,
     generic: bool = _GENERIC_DEFAULT,
     type_validation_factory: Optional[FieldTypeValidatorFactory] = DefaultFieldTypeValidatorFactory,
-) -> Callable[[Type[_T]], Type[_T]]:
-    ...
+) -> Callable[[Type[_T]], Type[_T]]: ...
 
 
 @overload
-def datamodel(  # noqa: F811  # redefinion of unused symbol
+def datamodel(  # redefinition of unused symbol
     cls: Type[_T],
     /,
     *,
-    repr: bool = _REPR_DEFAULT,  # noqa: A002  # shadowing 'repr' python builtin
+    repr: bool = _REPR_DEFAULT,
     eq: bool = _EQ_DEFAULT,
     order: bool = _ORDER_DEFAULT,
     unsafe_hash: bool = _UNSAFE_HASH_DEFAULT,
@@ -300,16 +283,15 @@ def datamodel(  # noqa: F811  # redefinion of unused symbol
     coerce: bool = _COERCE_DEFAULT,
     generic: bool = _GENERIC_DEFAULT,
     type_validation_factory: Optional[FieldTypeValidatorFactory] = DefaultFieldTypeValidatorFactory,
-) -> Type[_T]:
-    ...
+) -> Type[_T]: ...
 
 
 # TODO(egparedes): Use @dataclass_transform(eq_default=True, field_specifiers=("field",))
-def datamodel(  # noqa: F811  # redefinion of unused symbol
+def datamodel(  # redefinition of unused symbol
     cls: Optional[Type[_T]] = None,
     /,
     *,
-    repr: bool = _REPR_DEFAULT,  # noqa: A002  # shadowing 'repr' python builtin
+    repr: bool = _REPR_DEFAULT,  # noqa: A002 [builtin-argument-shadowing]
     eq: bool = _EQ_DEFAULT,
     order: bool = _ORDER_DEFAULT,
     unsafe_hash: bool = _UNSAFE_HASH_DEFAULT,
@@ -363,7 +345,7 @@ def datamodel(  # noqa: F811  # redefinion of unused symbol
             for all fields.
         generic: If ``True`` (default is ``False``) the class should be a ``Generic[]`` class,
             and the generated Data Model will support creation of new Data Model subclasses
-            when using concrete types as arguments of ``DataModelClass[some_concret_type]``.
+            when using concrete types as arguments of ``DataModelClass[some_concrete_type]``.
         type_validation_factory: Type validation factory used to build the field type validators.
             If ``None``, type validators will not be generators.
 
@@ -383,7 +365,7 @@ def datamodel(  # noqa: F811  # redefinion of unused symbol
     }
 
     if cls is None:  # called as: @datamodel()
-        return functools.partial(_make_datamodel, **datamodel_options)
+        return functools.partial(_make_datamodel, **datamodel_options)  # type: ignore[arg-type, return-value]
     else:  # called as: @datamodel
         return _make_datamodel(
             cls,
@@ -398,7 +380,7 @@ class _DataModelDecoratorTP(Protocol[_T]):
         cls: Optional[Type[_T]] = None,
         /,
         *,
-        repr: bool = _REPR_DEFAULT,  # noqa: A002  # shadowing 'repr' python builtin
+        repr: bool = _REPR_DEFAULT,  # noqa: A002 [builtin-argument-shadowing]
         eq: bool = _EQ_DEFAULT,
         order: bool = _ORDER_DEFAULT,
         unsafe_hash: bool = _UNSAFE_HASH_DEFAULT,
@@ -410,8 +392,7 @@ class _DataModelDecoratorTP(Protocol[_T]):
         type_validation_factory: Optional[
             FieldTypeValidatorFactory
         ] = DefaultFieldTypeValidatorFactory,
-    ) -> Union[Type[_T], Callable[[Type[_T]], Type[_T]]]:
-        ...
+    ) -> Union[Type[_T], Callable[[Type[_T]], Type[_T]]]: ...
 
 
 frozenmodel: _DataModelDecoratorTP = functools.partial(datamodel, frozen=True)
@@ -424,16 +405,13 @@ frozen_model = frozenmodel
 if xtyping.TYPE_CHECKING:
 
     class DataModel(DataModelTP):
-        def __init__(self, *args: Any, **kwargs: Any) -> None:
-            ...
+        def __init__(self, *args: Any, **kwargs: Any) -> None: ...
 
         def __pretty__(
             self, fmt: Callable[[Any], Any], **kwargs: Any
-        ) -> Generator[Any, None, None]:
-            ...
+        ) -> Generator[Any, None, None]: ...
 
 else:
-
     # TODO(egparedes): use @dataclass_transform(eq_default=True, field_specifiers=("field",))
     class DataModel:
         """Base class to automatically convert any subclass into a Data Model.
@@ -454,9 +432,7 @@ else:
             cls,
             /,
             *,
-            repr: bool  # noqa: A002  # shadowing 'repr' python builtin
-            | None
-            | Literal["inherited"] = "inherited",
+            repr: (bool | None | Literal["inherited"]) = "inherited",  # noqa: A002 [builtin-argument-shadowing]
             eq: bool | None | Literal["inherited"] = "inherited",
             order: bool | None | Literal["inherited"] = "inherited",
             unsafe_hash: bool | None | Literal["inherited"] = "inherited",
@@ -464,8 +440,9 @@ else:
             match_args: bool | Literal["inherited"] = "inherited",
             kw_only: bool | Literal["inherited"] = "inherited",
             coerce: bool | Literal["inherited"] = "inherited",
-            type_validation_factory: Optional[FieldTypeValidatorFactory]
-            | Literal["inherited"] = "inherited",
+            type_validation_factory: (
+                Optional[FieldTypeValidatorFactory] | Literal["inherited"]
+            ) = "inherited",
             **kwargs: Any,
         ) -> None:
             dm_opts = kwargs.pop(_DM_OPTS, [])
@@ -501,11 +478,7 @@ else:
                 raise TypeError("Subclasses of a frozen DataModel cannot be unfrozen.")
 
             _make_datamodel(
-                cls,
-                slots=False,
-                generic=generic,
-                **datamodel_kwargs,
-                _stacklevel_offset=1,
+                cls, slots=False, generic=generic, **datamodel_kwargs, _stacklevel_offset=1
             )
 
 
@@ -514,16 +487,15 @@ def field(
     default: Any = NOTHING,
     default_factory: Optional[Callable[[], Any]] = None,
     init: bool = True,
-    repr: bool = True,  # noqa: A002   # shadowing 'repr' python builtin
-    hash: Optional[bool] = None,  # noqa: A002   # shadowing 'hash' python builtin
+    repr: bool = True,  # noqa: A002 [builtin-argument-shadowing]
+    hash: Optional[bool] = None,  # noqa: A002 [builtin-argument-shadowing]
     compare: bool = True,
     metadata: Optional[Mapping[Any, Any]] = None,
     kw_only: bool = _KW_ONLY_DEFAULT,
     converter: Callable[[Any], Any] | Literal["coerce"] | None = None,
-    validator: AttrsValidator
-    | FieldValidator
-    | Sequence[AttrsValidator | FieldValidator]
-    | None = None,
+    validator: (
+        AttrsValidator | FieldValidator | Sequence[AttrsValidator | FieldValidator] | None
+    ) = None,
 ) -> Any:  # attr.s lies in some typings
     """Define a new attribute on a class with advanced options.
 
@@ -551,7 +523,7 @@ def field(
             have their own key, to use as a namespace in the metadata.
         kw_only: If ``True`` (default is ``False``), make this field keyword-only in the
             generated ``__init__`` (if ``init`` is ``False``, this parameter is ignored).
-        converter: Callable that is automatically called to convert attribute’s value.
+        converter: Callable that is automatically called to convert attribute's value.
             It is given the passed-in value, and the returned value will be used as the
             new value of the attribute before being passed to the validator, if any.
             If ``"coerce"`` is passed, a naive coercer converter will be generated.
@@ -567,7 +539,7 @@ def field(
         >>> from typing import List
         >>> @datamodel
         ... class C:
-        ...     mylist: List[int] = field(default_factory=lambda : [1, 2, 3])
+        ...     mylist: List[int] = field(default_factory=lambda: [1, 2, 3])
         >>> c = C()
         >>> c.mylist
         [1, 2, 3]
@@ -668,7 +640,7 @@ def get_fields(model: Union[DataModel, Type[DataModel]]) -> utils.FrozenNamespac
         >>> fields(Model)  # doctest:+ELLIPSIS
         FrozenNamespace(...name=Attribute(name='name', default=NOTHING, ...
 
-    """  # noqa: RST201  # doctest conventions confuse RST validator
+    """
     if not is_datamodel(model):
         raise TypeError(f"Invalid datamodel instance or class: '{model}'.")
     if not isinstance(model, type):
@@ -702,8 +674,8 @@ def asdict(
         ...     x: int
         ...     y: int
         >>> c = C(x=1, y=2)
-        >>> assert asdict(c) == {'x': 1, 'y': 2}
-    """  # noqa: RST301  # sphinx.napoleon conventions confuse RST validator
+        >>> assert asdict(c) == {"x": 1, "y": 2}
+    """
     if not is_datamodel(instance) or isinstance(instance, type):
         raise TypeError(f"Invalid datamodel instance: '{instance}'.")
     return attrs.asdict(instance, value_serializer=value_serializer)
@@ -739,8 +711,7 @@ _DataModelT = TypeVar("_DataModelT", bound=DataModel)
 
 
 def update_forward_refs(
-    model_cls: Type[_DataModelT],
-    localns: Optional[Dict[str, Any]] = None,
+    model_cls: Type[_DataModelT], localns: Optional[Dict[str, Any]] = None
 ) -> Type[_DataModelT]:
     """Update Data Model class meta-information replacing forwarded type annotations with actual types.
 
@@ -792,7 +763,7 @@ def concretize(
     *type_args: Type,
     class_name: Optional[str] = None,
     module: Optional[str] = None,
-    support_pickling: bool = True,  # noqa
+    support_pickling: bool = True,
     overwrite_definition: bool = True,
 ) -> Type[DataModelT]:
     """Generate a new concrete subclass of a generic Data Model.
@@ -813,9 +784,12 @@ def concretize(
         overwrite_definition: If ``True``, a previous definition of the class in
             the target module will be overwritten.
 
-    """  # noqa: RST301  # doctest conventions confuse RST validator
+    """
     concrete_cls: Type[DataModelT] = _make_concrete_with_cache(
-        datamodel_cls, *type_args, class_name=class_name, module=module
+        datamodel_cls,  # type: ignore[arg-type]
+        *type_args,
+        class_name=class_name,
+        module=module,
     )
     assert isinstance(concrete_cls, type) and is_datamodel(concrete_cls)
 
@@ -830,7 +804,8 @@ def concretize(
                 RuntimeWarning(
                     f"Existing '{class_name}' symbol in module '{module}' contains a reference"
                     "to a different object."
-                )
+                ),
+                stacklevel=2,
             )
 
     return concrete_cls
@@ -884,24 +859,13 @@ def _substitute_typevars(
         return type_params_map[type_hint], True
     elif getattr(type_hint, "__parameters__", []):
         return type_hint[tuple(type_params_map[tp] for tp in type_hint.__parameters__)], True
-        # TODO(egparedes): WIP fix for partial specialization
-        # # Type hint is a generic model: replace all the concretized type vars
-        # noqa: e800 replaced = False
-        # noqa: e800 new_args = []
-        # noqa: e800 for tp in type_hint.__parameters__:
-        # noqa: e800     if tp in type_params_map:
-        # noqa: e800         new_args.append(type_params_map[tp])
-        # noqa: e800         replaced = True
-        # noqa: e800     else:
-        # noqa: e800         new_args.append(type_params_map[tp])
-        # noqa: e800 return type_hint[tuple(new_args)], replaced
     else:
         return type_hint, False
 
 
 def _make_counting_attr_from_attribute(
     field_attrib: Attribute, *, include_type: bool = False, **kwargs: Any
-) -> Any:  # attr.s lies a bit in some typing definitons
+) -> Any:  # attr.s lies a bit in some typing definitions
     args = [
         "default",
         "validator",
@@ -982,35 +946,24 @@ def _make_data_model_class_getitem() -> classmethod:
         """
         type_args: Tuple[Type] = args if isinstance(args, tuple) else (args,)
         concrete_cls: Type[DataModelT] = concretize(cls, *type_args)
-        res = xtyping.StdGenericAliasType(concrete_cls, type_args)
-        if sys.version_info < (3, 9):
-            # in Python 3.8, xtyping.StdGenericAliasType (aka typing._GenericAlias)
-            # does not copy all required `__dict__` entries, so do it manually
-            for k, v in concrete_cls.__dict__.items():
-                if k not in res.__dict__:
-                    res.__dict__[k] = v
-        return res
+        return concrete_cls
 
     return classmethod(__class_getitem__)
 
 
 def _make_type_converter(type_annotation: TypeAnnotation, name: str) -> TypeConverter[_T]:
-    # TODO(egparedes): if a "typing tree" structure is implemented, refactor this code as a tree traversal.
-    #
+    # TODO(egparedes): if a "typing tree" structure is implemented, refactor this code
+    # as a tree traversal.
     if xtyping.is_actual_type(type_annotation) and not isinstance(None, type_annotation):
         assert not xtyping.get_args(type_annotation)
         assert isinstance(type_annotation, type)
 
         def _type_converter(value: Any) -> _T:
             try:
-                return (
-                    value
-                    if isinstance(value, type_annotation)  # type: ignore[arg-type]
-                    else type_annotation(value)  # type: ignore
-                )
+                return value if isinstance(value, type_annotation) else type_annotation(value)
             except Exception as error:
                 raise TypeError(
-                    f"Error during coertion of given value '{value}' for field '{name}'."
+                    f"Error during coercion of given value '{value}' for field '{name}'."
                 ) from error
 
         return _type_converter
@@ -1041,17 +994,17 @@ def _make_type_converter(type_annotation: TypeAnnotation, name: str) -> TypeConv
         return _make_type_converter(origin_type, name)
 
     raise exceptions.EveTypeError(
-        f"Automatic type coertion for {type_annotation} types is not supported."
+        f"Automatic type coercion for {type_annotation} types is not supported."
     )
 
 
 _KNOWN_MUTABLE_TYPES: Final = (list, dict, set)
 
 
-def _make_datamodel(  # noqa: C901  # too complex but still readable and documented
+def _make_datamodel(
     cls: Type[_T],
     *,
-    repr: bool,  # noqa: A002   # shadowing 'repr' python builtin
+    repr: bool,  # noqa: A002 [builtin-argument-shadowing]
     eq: bool,
     order: bool,
     unsafe_hash: bool,
@@ -1099,7 +1052,7 @@ def _make_datamodel(  # noqa: C901  # too complex but still readable and documen
 
         # Create type validator if validation is enabled
         if type_validation_factory is None or _UNCHECKED_TYPE_TAG in type_extras:
-            type_validator = lambda a, b, c: None  # noqa: E731
+            type_validator = lambda a, b, c: None  # noqa: E731 [lambda-assignment]
         else:
             type_validator = type_validation_factory(type_hint, qualified_field_name)
 
@@ -1130,25 +1083,21 @@ def _make_datamodel(  # noqa: C901  # too complex but still readable and documen
                     )
 
         else:
-            # Create field converter if automatic coertion is enabled
+            # Create field converter if automatic coercion is enabled
             converter: TypeConverter = cast(
                 TypeConverter,
                 _make_type_converter(type_hint, qualified_field_name) if coerce_field else None,
             )
             if attr_value_in_cls is NOTHING:
                 # The field has no definition in the class dict, it's only an annotation
-                setattr(
-                    cls,
-                    key,
-                    attrs.field(converter=converter, validator=type_validator),
-                )
+                setattr(cls, key, attrs.field(converter=converter, validator=type_validator))
 
             else:
                 # The field contains the default value in the class dict
                 if isinstance(attr_value_in_cls, _KNOWN_MUTABLE_TYPES):
                     warnings.warn(
                         f"'{attr_value_in_cls.__class__.__name__}' value used as default in '{cls.__name__}.{key}'.\n"
-                        "Mutable types should not defbe normally used as field defaults (use 'default_factory' instead).",
+                        "Mutable types should not be used as field defaults (use 'default_factory' instead).",
                         stacklevel=_stacklevel_offset + 2,
                     )
                 setattr(
@@ -1185,9 +1134,7 @@ def _make_datamodel(  # noqa: C901  # too complex but still readable and documen
             if base_field_attr:
                 # Create a new field in the current class cloning the existing
                 # definition and add the new validator (attrs recommendation)
-                field_c_attr = _make_counting_attr_from_attribute(
-                    base_field_attr,
-                )
+                field_c_attr = _make_counting_attr_from_attribute(base_field_attr)
                 setattr(cls, qualified_field_name, field_c_attr)
             else:
                 raise TypeError(
@@ -1210,7 +1157,8 @@ def _make_datamodel(  # noqa: C901  # too complex but still readable and documen
         cls.__attrs_pre_init__ = cls.__pre_init__  # type: ignore[attr-defined]  # adding new attribute
 
     if "__attrs_post_init__" in cls.__dict__ and not hasattr(
-        cls.__attrs_post_init__, _DATAMODEL_TAG  # type: ignore[attr-defined]  # mypy doesn't know about __attr_post_init__
+        cls.__attrs_post_init__,  # type: ignore[attr-defined]  # mypy doesn't know about __attr_post_init__
+        _DATAMODEL_TAG,
     ):
         raise TypeError(f"'{cls.__name__}' class contains forbidden custom '__attrs_post_init__'.")
     cls.__attrs_post_init__ = _make_post_init(has_post_init="__post_init__" in cls.__dict__)  # type: ignore[attr-defined]  # adding new attribute
@@ -1304,8 +1252,14 @@ def _make_concrete_with_cache(
     if not is_generic_datamodel_class(datamodel_cls):
         raise TypeError(f"'{datamodel_cls.__name__}' is not a generic model class.")
     for t in type_args:
+        _accepted_types: tuple[type, ...] = (
+            type,
+            type(None),
+            xtyping.StdGenericAliasType,
+            types.UnionType,
+        )
         if not (
-            isinstance(t, (type, type(None), xtyping.StdGenericAliasType))
+            isinstance(t, _accepted_types)
             or (getattr(type(t), "__module__", None) in ("typing", "typing_extensions"))
         ):
             raise TypeError(
@@ -1321,11 +1275,7 @@ def _make_concrete_with_cache(
     # Replace field definitions with the new actual types for generic fields
     type_params_map = dict(zip(datamodel_cls.__parameters__, type_args))
     model_fields = getattr(datamodel_cls, MODEL_FIELD_DEFINITIONS_ATTR)
-    new_annotations = {
-        # TODO(egparedes): ?
-        # noqa: e800 "__args__": "ClassVar[Tuple[Union[Type, TypeVar], ...]]",
-        # noqa: e800 "__parameters__": "ClassVar[Tuple[TypeVar, ...]]",
-    }
+    new_annotations = {}
 
     new_field_c_attrs = {}
     for field_name, field_type in xtyping.get_type_hints(datamodel_cls).items():
@@ -1358,8 +1308,16 @@ def _make_concrete_with_cache(
         "__module__": module if module else datamodel_cls.__module__,
         **new_field_c_attrs,
     }
-
     concrete_cls = type(class_name, (datamodel_cls,), namespace)
+
+    # Update the tuple of generic parameters in the new class, in case
+    # this is a partial concretization
+    assert hasattr(concrete_cls, "__parameters__")
+    concrete_cls.__parameters__ = tuple(
+        type_params_map[tp_var]
+        for tp_var in datamodel_cls.__parameters__
+        if isinstance(type_params_map[tp_var], typing.TypeVar)
+    )
     assert concrete_cls.__module__ == module or not module
 
     if MODEL_FIELD_DEFINITIONS_ATTR not in concrete_cls.__dict__:
@@ -1392,8 +1350,7 @@ if xtyping.TYPE_CHECKING:
         @classmethod
         def __class_getitem__(
             cls: Type[GenericDataModelTP], args: Union[Type, Tuple[Type, ...]]
-        ) -> Union[DataModelTP, GenericDataModelTP]:
-            ...
+        ) -> Union[DataModelTP, GenericDataModelTP]: ...
 
 else:
 
