@@ -1,20 +1,13 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Basic pattern matching utilities."""
 
-
 from __future__ import annotations
 
 from functools import singledispatch
@@ -31,9 +24,9 @@ class ObjectPattern:
 
     Examples:
         >>> class Foo:
-        ...    def __init__(self, bar, baz):
-        ...        self.bar = bar
-        ...        self.baz = baz
+        ...     def __init__(self, bar, baz):
+        ...         self.bar = bar
+        ...         self.baz = baz
         >>> assert ObjectPattern(Foo, bar=1).match(Foo(1, 2))
     """
 
@@ -62,7 +55,7 @@ class ObjectPattern:
         return next(get_differences(self, other), None) is None
 
     def __str__(self) -> str:
-        attrs_str = ", ".join([f"{str(k)}={str(v)}" for k, v in self.fields.items()])
+        attrs_str = ", ".join([f"{k!s}={v!s}" for k, v in self.fields.items()])
         return f"{self.cls.__name__}({attrs_str})"
 
 
@@ -86,10 +79,7 @@ def get_differences(a: Any, b: Any, path: str = "") -> Iterator[Tuple[str, str]]
 @get_differences.register
 def _(a: ObjectPattern, b: Any, path: str = "") -> Iterator[Tuple[str, str]]:
     if not isinstance(b, a.cls):
-        yield (
-            path,
-            f"Expected an instance of class {a.cls.__name__}, but got {type(b).__name__}",
-        )
+        yield (path, f"Expected an instance of class {a.cls.__name__}, but got {type(b).__name__}")
     else:
         for k in a.fields.keys():
             if not hasattr(b, k):
