@@ -1,24 +1,18 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 """Visitor classes to work with IR trees."""
 
-
 from __future__ import annotations
 
 import collections.abc
 import copy
+from typing import ClassVar
 
 from . import concepts, trees
 from .extended_typing import Any
@@ -51,7 +45,7 @@ class NodeVisitor:
         3. ``self.generic_visit()``.
 
     This dispatching mechanism is implemented in the main :meth:`visit`
-    method and can be overriden in subclasses. Therefore, a simple way to extend
+    method and can be overridden in subclasses. Therefore, a simple way to extend
     the behavior of a visitor is by inheriting from lightweight `trait` classes
     with a custom ``visit()`` method, which wraps the call to the superclass'
     ``visit()`` and adds extra pre and post visit logic. Check :mod:`eve.traits`
@@ -88,7 +82,7 @@ class NodeVisitor:
 
     Notes:
         If you want to apply changes to nodes during the traversal,
-        use the :class:`NodeMutator` subclass, which handles correctly
+        use the :class:`NodeTranslator` subclass, which handles correctly
         structural modifications of the visited tree.
 
     """
@@ -151,8 +145,9 @@ class NodeTranslator(NodeVisitor):
 
     """
 
-    def generic_visit(self, node: concepts.RootNode, **kwargs: Any) -> Any:
+    PRESERVED_ANNEX_ATTRS: ClassVar[tuple[str, ...]] = ()
 
+    def generic_visit(self, node: concepts.RootNode, **kwargs: Any) -> Any:
         memo = kwargs.get("__memo__", None)
 
         if isinstance(node, concepts.Node):
@@ -161,8 +156,11 @@ class NodeTranslator(NodeVisitor):
                     name: new_child
                     for name, child in node.iter_children_items()
                     if (new_child := self.visit(child, **kwargs)) is not NOTHING
-                },
+                }
             )
+            if self.PRESERVED_ANNEX_ATTRS and getattr(node, "__node_annex__", None):
+                self._preserve_annex(node, new_node)
+
             return new_node
 
         if isinstance(node, (list, tuple, set, collections.abc.Set)) or (
@@ -186,3 +184,27 @@ class NodeTranslator(NodeVisitor):
             )
 
         return copy.deepcopy(node, memo=memo)
+
+    def visit(self, node: concepts.RootNode, **kwargs: Any) -> Any:
+        new_node = super().visit(node, **kwargs)
+
+        if (
+            isinstance(node, concepts.Node)
+            and isinstance(new_node, concepts.Node)
+            and self.PRESERVED_ANNEX_ATTRS
+            and getattr(node, "__node_annex__", None)
+        ):
+            self._preserve_annex(node, new_node)
+
+        return new_node
+
+    def _preserve_annex(self, node: concepts.Node, new_node: concepts.Node) -> None:
+        # access to `new_node.annex` implicitly creates the `__node_annex__` attribute in the property getter
+        new_annex_dict = new_node.annex.__dict__
+        old_annex_dict = node.annex.__dict__
+        for key in (old_annex_dict.keys() & self.PRESERVED_ANNEX_ATTRS) - new_annex_dict.keys():
+            # Note: The annex item's value of the new node might not be equal
+            # (in the sense that an equality comparison returns false), to
+            # the old one, but in the context of the pass, they are
+            # equivalent. Therefore, we don't assert equality here.
+            new_annex_dict[key] = old_annex_dict[key]
