@@ -1,16 +1,10 @@
 # GT4Py - GridTools Framework
 #
-# Copyright (c) 2014-2023, ETH Zurich
+# Copyright (c) 2014-2024, ETH Zurich
 # All rights reserved.
 #
-# This file is part of the GT4Py project and the GridTools framework.
-# GT4Py is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or any later
-# version. See the LICENSE.txt file at the top-level directory of this
-# distribution for a copy of the license or check <https://www.gnu.org/licenses/>.
-#
-# SPDX-License-Identifier: GPL-3.0-or-later
+# Please, refer to the LICENSE file in the root directory.
+# SPDX-License-Identifier: BSD-3-Clause
 
 from __future__ import annotations
 
@@ -21,8 +15,7 @@ from gt4py.eve.traits import SymbolTableTrait
 from gt4py.next.program_processors.codegens.gtfn.gtfn_ir_common import Expr, Sym, SymRef
 
 
-class Stmt(Node):
-    ...
+class Stmt(Node): ...
 
 
 class AssignStmt(Stmt):
@@ -35,8 +28,7 @@ class InitStmt(AssignStmt):
     init_type: str = "auto"
 
 
-class EmptyListInitializer(Expr):
-    ...
+class EmptyListInitializer(Expr): ...
 
 
 class Conditional(Stmt):
@@ -52,6 +44,6 @@ class ReturnStmt(Stmt):
 
 
 class ImperativeFunctionDefinition(Node, SymbolTableTrait):
-    id: Coerced[SymbolName]  # noqa: A003
+    id: Coerced[SymbolName]
     params: List[Sym]
     fun: List[Stmt]
